@@ -1,27 +1,27 @@
 /*
-    Copyright (c) 2013-2017 Cong Xu
-    All rights reserved.
+	Copyright (c) 2013-2022 Cong Xu
+	All rights reserved.
 
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions are met:
 
-    Redistributions of source code must retain the above copyright notice, this
-    list of conditions and the following disclaimer.
-    Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
+	Redistributions of source code must retain the above copyright notice, this
+	list of conditions and the following disclaimer.
+	Redistributions in binary form must reproduce the above copyright notice,
+	this list of conditions and the following disclaimer in the documentation
+	and/or other materials provided with the distribution.
 
-    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+	POSSIBILITY OF SUCH DAMAGE.
 */
 #include "briefing_screens.h"
 
@@ -31,51 +31,71 @@
 #include <cdogs/files.h>
 #include <cdogs/font.h>
 #include <cdogs/grafx_bg.h>
+#include <cdogs/log.h>
+#include <cdogs/music.h>
 #include <cdogs/objective.h>
 
+#include "animated_counter.h"
 #include "autosave.h"
 #include "hiscores.h"
+#include "loading_screens.h"
 #include "menu_utils.h"
 #include "password.h"
 #include "prep.h"
 #include "prep_equip.h"
 #include "screens_end.h"
 
-
 static void DrawObjectiveInfo(const Objective *o, const struct vec2i pos);
 
 typedef struct
 {
 	EventWaitResult waitResult;
-	const CampaignSetting *c;
+	CampaignSetting *c;
+	GameMode gameMode;
+	const CampaignEntry *entry;
 } ScreenCampaignIntroData;
 static void CampaignIntroTerminate(GameLoopData *data);
+static void CampaignIntroOnEnter(GameLoopData *data);
 static void CampaignIntroOnExit(GameLoopData *data);
 static void CampaignIntroInput(GameLoopData *data);
 static GameLoopResult CampaignIntroUpdate(GameLoopData *data, LoopRunner *l);
 static void CampaignIntroDraw(GameLoopData *data);
-GameLoopData *ScreenCampaignIntro(CampaignSetting *c)
+GameLoopData *ScreenCampaignIntro(CampaignSetting *c, const GameMode gameMode, const CampaignEntry *entry)
 {
 	ScreenCampaignIntroData *data;
 	CMALLOC(data, sizeof *data);
 	data->c = c;
+	data->gameMode = gameMode;
+	data->entry = entry;
 	return GameLoopDataNew(
-		data, CampaignIntroTerminate, NULL, CampaignIntroOnExit,
-		CampaignIntroInput, CampaignIntroUpdate, CampaignIntroDraw);
+		data, CampaignIntroTerminate, CampaignIntroOnEnter,
+		CampaignIntroOnExit, CampaignIntroInput, CampaignIntroUpdate,
+		CampaignIntroDraw);
 }
 static void CampaignIntroTerminate(GameLoopData *data)
 {
 	ScreenCampaignIntroData *mData = data->Data;
 	CFREE(mData);
 }
+static void CampaignIntroOnEnter(GameLoopData *data)
+{
+	ScreenCampaignIntroData *sData = data->Data;
+	gCampaign.Entry.Mode = sData->gameMode;
+	if (!gCampaign.IsLoaded && !CampaignLoad(&gCampaign, sData->entry))
+	{
+		// Failed to load
+		LOG(LM_MAIN, LL_ERROR, "cannot load campaign %s", sData->entry->Info);
+	}
+	else
+	{
+		MusicPlayFromChunk(
+			&gSoundDevice.music, MUSIC_MENU, &sData->c->CustomSongs[MUSIC_MENU]);
+	}
+}
 static void CampaignIntroOnExit(GameLoopData *data)
 {
 	const ScreenCampaignIntroData *sData = data->Data;
-	if (sData->waitResult != EVENT_WAIT_CANCEL)
-	{
-		SoundPlay(&gSoundDevice, StrSound("mg"));
-	}
-	else
+	if (sData->waitResult == EVENT_WAIT_CANCEL)
 	{
 		CampaignUnload(&gCampaign);
 	}
@@ -84,21 +104,27 @@ static void CampaignIntroInput(GameLoopData *data)
 {
 	ScreenCampaignIntroData *sData = data->Data;
 	sData->waitResult = EventWaitForAnyKeyOrButton();
+	if (sData->waitResult == EVENT_WAIT_OK)
+	{
+		MenuPlaySound(MENU_SOUND_ENTER);
+	}
 }
 static GameLoopResult CampaignIntroUpdate(GameLoopData *data, LoopRunner *l)
 {
 	const ScreenCampaignIntroData *sData = data->Data;
 
-	if (!IsIntroNeeded(gCampaign.Entry.Mode) ||
+	if (!gCampaign.IsLoaded || sData->waitResult == EVENT_WAIT_CANCEL)
+	{
+		LoopRunnerPop(l);
+	}
+	else if (!IsIntroNeeded(gCampaign.Entry.Mode) ||
 		sData->waitResult == EVENT_WAIT_OK)
 	{
 		// Switch to num players selection
-		LoopRunnerChange(
-			l, NumPlayersSelection(&gGraphicsDevice, &gEventHandlers));
-	}
-	else if (sData->waitResult == EVENT_WAIT_CANCEL)
-	{
-		LoopRunnerPop(l);
+		LoopRunnerPush(
+			l, ScreenLoading(
+				   "Loading campaign...", true,
+				   NumPlayersSelection(&gGraphicsDevice, &gEventHandlers), true));
 	}
 	return UPDATE_RESULT_OK;
 }
@@ -136,7 +162,6 @@ static void CampaignIntroDraw(GameLoopData *data)
 	BlitUpdateFromBuf(&gGraphicsDevice, gGraphicsDevice.screen);
 }
 
-
 typedef struct
 {
 	char *Title;
@@ -150,15 +175,18 @@ typedef struct
 	struct vec2i ObjectiveDescPos;
 	struct vec2i ObjectiveInfoPos;
 	int ObjectiveHeight;
+	CampaignSetting *C;
 	const struct MissionOptions *MissionOptions;
 	EventWaitResult waitResult;
 } MissionBriefingData;
 static void MissionBriefingTerminate(GameLoopData *data);
+static void MissionBriefingOnEnter(GameLoopData *data);
 static void MissionBriefingOnExit(GameLoopData *data);
 static void MissionBriefingInput(GameLoopData *data);
 static GameLoopResult MissionBriefingUpdate(GameLoopData *data, LoopRunner *l);
 static void MissionBriefingDraw(GameLoopData *data);
-GameLoopData *ScreenMissionBriefing(const struct MissionOptions *m)
+GameLoopData *ScreenMissionBriefing(
+	CampaignSetting *c, const struct MissionOptions *m)
 {
 	const int w = gGraphicsDevice.cachedConfig.Res.x;
 	const int h = gGraphicsDevice.cachedConfig.Res.y;
@@ -171,25 +199,15 @@ GameLoopData *ScreenMissionBriefing(const struct MissionOptions *m)
 	if (m->missionData->Title)
 	{
 		CMALLOC(mData->Title, strlen(m->missionData->Title) + 32);
-		sprintf(mData->Title, "Mission %d: %s",
-			m->index + 1, m->missionData->Title);
+		sprintf(
+			mData->Title, "Mission %d: %s", m->index + 1,
+			m->missionData->Title);
 		mData->TitleOpts = FontOptsNew();
 		mData->TitleOpts.HAlign = ALIGN_CENTER;
 		mData->TitleOpts.Area = gGraphicsDevice.cachedConfig.Res;
 		mData->TitleOpts.Pad.y = y - 25;
 	}
 
-	// Password
-	if (m->index > 0)
-	{
-		sprintf(
-			mData->Password, "Password: %s", gAutosave.LastMission.Password);
-		mData->PasswordOpts = FontOptsNew();
-		mData->PasswordOpts.HAlign = ALIGN_CENTER;
-		mData->PasswordOpts.Area = gGraphicsDevice.cachedConfig.Res;
-		mData->PasswordOpts.Pad.y = y - 15;
-	}
-
 	// Description
 	if (m->missionData->Description)
 	{
@@ -211,11 +229,13 @@ GameLoopData *ScreenMissionBriefing(const struct MissionOptions *m)
 			svec2i(w - (w / 6), mData->ObjectiveDescPos.y + FontH());
 		mData->ObjectiveHeight = h / 12;
 	}
+	mData->C = c;
 	mData->MissionOptions = m;
 
 	return GameLoopDataNew(
-		mData, MissionBriefingTerminate, NULL, MissionBriefingOnExit,
-		MissionBriefingInput, MissionBriefingUpdate, MissionBriefingDraw);
+		mData, MissionBriefingTerminate, MissionBriefingOnEnter,
+		MissionBriefingOnExit, MissionBriefingInput, MissionBriefingUpdate,
+		MissionBriefingDraw);
 }
 static void MissionBriefingTerminate(GameLoopData *data)
 {
@@ -226,15 +246,21 @@ static void MissionBriefingTerminate(GameLoopData *data)
 	CFREE(mData->TypewriterBuf);
 	CFREE(mData);
 }
+static void MissionBriefingOnEnter(GameLoopData *data)
+{
+	MissionBriefingData *mData = data->Data;
+	if (IsMissionBriefingNeeded(gCampaign.Entry.Mode, mData->Description))
+	{
+		MusicPlayFromChunk(
+			&gSoundDevice.music, MUSIC_BRIEFING,
+			&mData->C->CustomSongs[MUSIC_BRIEFING]);
+	}
+}
 static void MissionBriefingOnExit(GameLoopData *data)
 {
 	const MissionBriefingData *mData = data->Data;
 
-	if (mData->waitResult == EVENT_WAIT_OK)
-	{
-		SoundPlay(&gSoundDevice, StrSound("mg"));
-	}
-	else
+	if (mData->waitResult != EVENT_WAIT_OK)
 	{
 		CampaignUnload(&gCampaign);
 	}
@@ -262,11 +288,13 @@ static void MissionBriefingInput(GameLoopData *data)
 				}
 				// Otherwise, exit out of loop
 				mData->waitResult = EVENT_WAIT_OK;
+				MenuPlaySound(MENU_SOUND_ENTER);
 			}
 		}
 	}
 	// Check if anyone pressed escape
-	if (EventIsEscape(&gEventHandlers, cmds, GetMenuCmd(&gEventHandlers)))
+	if (EventIsEscape(
+			&gEventHandlers, cmds, GetMenuCmd(&gEventHandlers, false)))
 	{
 		mData->waitResult = EVENT_WAIT_CANCEL;
 	}
@@ -275,7 +303,7 @@ static GameLoopResult MissionBriefingUpdate(GameLoopData *data, LoopRunner *l)
 {
 	MissionBriefingData *mData = data->Data;
 
-	if (!IsMissionBriefingNeeded(gCampaign.Entry.Mode))
+	if (!IsMissionBriefingNeeded(gCampaign.Entry.Mode, mData->Description))
 	{
 		mData->waitResult = EVENT_WAIT_OK;
 		goto bail;
@@ -296,6 +324,13 @@ static GameLoopResult MissionBriefingUpdate(GameLoopData *data, LoopRunner *l)
 		return UPDATE_RESULT_DRAW;
 	}
 
+	// Auto skip if on demo mode
+	if (gEventHandlers.DemoQuitTimer > 0)
+	{
+		mData->waitResult = EVENT_WAIT_OK;
+		goto bail;
+	}
+
 	return UPDATE_RESULT_OK;
 
 bail:
@@ -324,17 +359,17 @@ static void MissionBriefingDraw(GameLoopData *data)
 	// Display objectives
 	CA_FOREACH(
 		const Objective, o, mData->MissionOptions->missionData->Objectives)
-		// Do not brief optional objectives
-		if (o->Required == 0)
-		{
-			continue;
-		}
-		struct vec2i offset = svec2i(0, _ca_index * mData->ObjectiveHeight);
-		FontStr(o->Description, svec2i_add(mData->ObjectiveDescPos, offset));
-		// Draw the icons slightly offset so that tall icons don't overlap each
-		// other
-		offset.x = -16 * (_ca_index & 1);
-		DrawObjectiveInfo(o, svec2i_add(mData->ObjectiveInfoPos, offset));
+	// Do not brief optional objectives
+	if (o->Required == 0)
+	{
+		continue;
+	}
+	struct vec2i offset = svec2i(0, _ca_index * mData->ObjectiveHeight);
+	FontStr(o->Description, svec2i_add(mData->ObjectiveDescPos, offset));
+	// Draw the icons slightly offset so that tall icons don't overlap each
+	// other
+	offset.x = -16 * (_ca_index & 1);
+	DrawObjectiveInfo(o, svec2i_add(mData->ObjectiveInfoPos, offset));
 	CA_FOREACH_END()
 
 	BlitUpdateFromBuf(&gGraphicsDevice, gGraphicsDevice.screen);
@@ -342,25 +377,38 @@ static void MissionBriefingDraw(GameLoopData *data)
 
 #define PERFECT_BONUS 500
 
+typedef struct
+{
+	const PlayerData *Pd;
+	AnimatedCounter Score;
+	AnimatedCounter Total;
+	AnimatedCounter Time;
+	AnimatedCounter TimeTotal;
+	AnimatedCounter HealthResurrection;
+	AnimatedCounter ButcherNinjaFriendly;
+} PlayerSummaryDrawData;
 typedef struct
 {
 	MenuSystem ms;
-	const CampaignOptions *c;
+	const Campaign *c;
 	struct MissionOptions *m;
 	bool completed;
+	AnimatedCounter AccessBonus;
+	AnimatedCounter TimeBonus;
+	PlayerSummaryDrawData pDatas[MAX_LOCAL_PLAYERS];
 } MissionSummaryData;
 static void MissionSummaryTerminate(GameLoopData *data);
 static void MissionSummaryOnEnter(GameLoopData *data);
 static GameLoopResult MissionSummaryUpdate(GameLoopData *data, LoopRunner *l);
 static void MissionSummaryDraw(GameLoopData *data);
 static void MissionSummaryMenuDraw(
-	const menu_t *menu, GraphicsDevice *g,
-	const struct vec2i p, const struct vec2i size, const void *data);
+	const menu_t *menu, GraphicsDevice *g, const struct vec2i p,
+	const struct vec2i size, const void *data);
 GameLoopData *ScreenMissionSummary(
-	const CampaignOptions *c, struct MissionOptions *m, const bool completed)
+	const Campaign *c, struct MissionOptions *m, const bool completed)
 {
 	MissionSummaryData *mData;
-	CMALLOC(mData, sizeof *mData);
+	CCALLOC(mData, sizeof *mData);
 
 	const int h = FontH() * 10;
 	MenuSystemInit(
@@ -381,78 +429,197 @@ GameLoopData *ScreenMissionSummary(
 	}
 	mData->ms.allowAborts = true;
 	MenuAddExitType(&mData->ms, MENU_TYPE_RETURN);
-	MenuSystemAddCustomDisplay(&mData->ms, MissionSummaryMenuDraw, m);
+	MenuSystemAddCustomDisplay(&mData->ms, MissionSummaryMenuDraw, mData);
 
 	mData->c = c;
 	mData->m = m;
 	mData->completed = completed;
 
 	return GameLoopDataNew(
-		mData, MissionSummaryTerminate, MissionSummaryOnEnter, NULL,
-		NULL, MissionSummaryUpdate, MissionSummaryDraw);
+		mData, MissionSummaryTerminate, MissionSummaryOnEnter, NULL, NULL,
+		MissionSummaryUpdate, MissionSummaryDraw);
 }
 static void MissionSummaryTerminate(GameLoopData *data)
 {
 	MissionSummaryData *mData = data->Data;
 
 	MenuSystemTerminate(&mData->ms);
+	AnimatedCounterTerminate(&mData->AccessBonus);
+	AnimatedCounterTerminate(&mData->TimeBonus);
+	for (int i = 0; i < MAX_LOCAL_PLAYERS; i++)
+	{
+		AnimatedCounterTerminate(&mData->pDatas[i].Score);
+		AnimatedCounterTerminate(&mData->pDatas[i].Total);
+		AnimatedCounterTerminate(&mData->pDatas[i].HealthResurrection);
+		AnimatedCounterTerminate(&mData->pDatas[i].ButcherNinjaFriendly);
+	}
 	CFREE(mData);
 }
 static bool AreAnySurvived(void);
 static int GetAccessBonus(const struct MissionOptions *m);
 static int GetTimeBonus(const struct MissionOptions *m, int *secondsOut);
 static void ApplyBonuses(PlayerData *p, const int bonus);
+static int GetHealthBonus(const PlayerData *p);
+static int GetResurrectionFee(const PlayerData *p);
+static int GetButcherPenalty(const PlayerData *p);
+static int GetNinjaBonus(const PlayerData *p);
+static int GetFriendlyBonus(const PlayerData *p);
 static void MissionSummaryOnEnter(GameLoopData *data)
 {
 	MissionSummaryData *mData = data->Data;
 
-	if (mData->completed)
+	if (mData->completed && CanLevelSelect(mData->c->Entry.Mode))
 	{
-		// Save password
-		MissionSave ms;
-		MissionSaveInit(&ms);
-		ms.Campaign = mData->c->Entry;
-		// Don't make password for next level if there is none
-		int passwordIndex = mData->m->index + 1;
-		if (passwordIndex == mData->c->Entry.NumMissions)
-		{
-			passwordIndex--;
-		}
-		strcpy(ms.Password, MakePassword(passwordIndex, 0));
-		ms.MissionsCompleted = mData->m->index + 1;
-		AutosaveAddMission(&gAutosave, &ms);
+		AutosaveAdd(
+			&gAutosave, &mData->c->Entry, mData->m->index,
+			mData->m->NextMission, &gPlayerDatas);
 		AutosaveSave(&gAutosave, GetConfigFilePath(AUTOSAVE_FILE));
 	}
 
 	// Calculate bonus scores
 	// Bonuses only apply if at least one player has lived
+	const int accessBonus = GetAccessBonus(mData->m);
 	if (AreAnySurvived())
 	{
 		int bonus = 0;
 		// Objective bonuses
 		CA_FOREACH(const Objective, o, mData->m->missionData->Objectives)
-			if (ObjectiveIsPerfect(o))
-			{
-				bonus += PERFECT_BONUS;
-			}
+		if (ObjectiveIsPerfect(o))
+		{
+			bonus += PERFECT_BONUS;
+		}
 		CA_FOREACH_END()
-		bonus += GetAccessBonus(mData->m);
+		bonus += accessBonus;
 		bonus += GetTimeBonus(mData->m, NULL);
 
 		CA_FOREACH(PlayerData, p, gPlayerDatas)
-			ApplyBonuses(p, bonus);
+		ApplyBonuses(p, bonus);
+		p->Totals.TimeTicks += mData->m->time;
 		CA_FOREACH_END()
 	}
+
+	// Skip menu
+	if (mData->m->missionData->SkipDebrief)
+	{
+		return;
+	}
+
+	if (mData->completed)
+	{
+		MusicPlayFromChunk(
+			&gSoundDevice.music, MUSIC_END,
+			&gCampaign.Setting.CustomSongs[MUSIC_END]);
+	}
+	else
+	{
+		MusicPlayFromChunk(
+			&gSoundDevice.music, MUSIC_LOSE,
+			&gCampaign.Setting.CustomSongs[MUSIC_LOSE]);
+	}
+
+	// Init mission bonuses
+	if (AreAnySurvived())
+	{
+		if (accessBonus > 0)
+		{
+			mData->AccessBonus =
+				AnimatedCounterNew("Access bonus: ", accessBonus);
+		}
+		int seconds;
+		const int timeBonus = GetTimeBonus(mData->m, &seconds);
+		char buf[256];
+		sprintf(buf, "Time bonus: %d secs x 25 = ", seconds);
+		mData->TimeBonus = AnimatedCounterNew(buf, timeBonus);
+	}
+
+	// Init per-player summaries
+	int idx = 0;
+	CA_FOREACH(PlayerData, pd, gPlayerDatas)
+	if (!pd->IsLocal)
+	{
+		continue;
+	}
+	mData->pDatas[idx].Pd = pd;
+	mData->pDatas[idx].Score = AnimatedCounterNew("Score: ", pd->Stats.Score);
+	mData->pDatas[idx].Total = AnimatedCounterNew("Total: ", pd->Totals.Score);
+	mData->pDatas[idx].Time =
+		AnimatedCounterNew("Time: ", mData->m->time / FPS_FRAMELIMIT);
+	mData->pDatas[idx].TimeTotal = AnimatedCounterNew(
+		"Total Time: ", pd->Totals.TimeTicks / FPS_FRAMELIMIT);
+	if (pd->survived)
+	{
+		const int healthBonus = GetHealthBonus(pd);
+		if (healthBonus != 0)
+		{
+			mData->pDatas[idx].HealthResurrection =
+				AnimatedCounterNew("Health bonus: ", healthBonus);
+		}
+		const int resurrectionFee = GetResurrectionFee(pd);
+		if (resurrectionFee != 0)
+		{
+			mData->pDatas[idx].HealthResurrection =
+				AnimatedCounterNew("Resurrection fee: ", resurrectionFee);
+		}
+
+		const int butcherPenalty = GetButcherPenalty(pd);
+		if (butcherPenalty != 0)
+		{
+			mData->pDatas[idx].ButcherNinjaFriendly =
+				AnimatedCounterNew("Butcher penalty: ", butcherPenalty);
+		}
+		const int ninjaBonus = GetNinjaBonus(pd);
+		if (ninjaBonus != 0)
+		{
+			mData->pDatas[idx].ButcherNinjaFriendly =
+				AnimatedCounterNew("Ninja bonus: ", ninjaBonus);
+		}
+		const int friendlyBonus = GetFriendlyBonus(pd);
+		if (friendlyBonus != 0)
+		{
+			mData->pDatas[idx].ButcherNinjaFriendly =
+				AnimatedCounterNew("Friendly bonus: ", friendlyBonus);
+		}
+	}
+	idx++;
+	CA_FOREACH_END()
 }
 static GameLoopResult MissionSummaryUpdate(GameLoopData *data, LoopRunner *l)
 {
 	MissionSummaryData *mData = data->Data;
 
-	const GameLoopResult result = MenuUpdate(&mData->ms);
-	if (result == UPDATE_RESULT_OK)
+	GameLoopResult result = MenuUpdate(&mData->ms);
+	if (result == UPDATE_RESULT_DRAW)
 	{
-		gCampaign.IsComplete = mData->completed &&
-			gCampaign.MissionIndex == (int)gCampaign.Setting.Missions.size - 1;
+		bool done = true;
+		done = AnimatedCounterUpdate(&mData->AccessBonus, 1) && done;
+		done = AnimatedCounterUpdate(&mData->TimeBonus, 1) && done;
+		for (int i = 0; i < MAX_LOCAL_PLAYERS; i++)
+		{
+			done = AnimatedCounterUpdate(&mData->pDatas[i].Score, 1) && done;
+			done = AnimatedCounterUpdate(&mData->pDatas[i].Total, 1) && done;
+			done = AnimatedCounterUpdate(&mData->pDatas[i].Time, 1) && done;
+			done =
+				AnimatedCounterUpdate(&mData->pDatas[i].TimeTotal, 1) && done;
+			done = AnimatedCounterUpdate(
+					   &mData->pDatas[i].HealthResurrection, 1) &&
+				   done;
+			done = AnimatedCounterUpdate(
+					   &mData->pDatas[i].ButcherNinjaFriendly, 1) &&
+				   done;
+		}
+
+		// Skip after animations are done if in demo mode
+		if (done && gEventHandlers.DemoQuitTimer > 0)
+		{
+			result = UPDATE_RESULT_OK;
+		}
+	}
+
+	if (result == UPDATE_RESULT_OK || mData->m->missionData->SkipDebrief)
+	{
+		gCampaign.IsComplete =
+			mData->completed &&
+			mData->m->NextMission == (int)gCampaign.Setting.Missions.size;
 		if (gCampaign.IsComplete)
 		{
 			LoopRunnerChange(l, ScreenVictory(&gCampaign));
@@ -460,7 +627,7 @@ static GameLoopResult MissionSummaryUpdate(GameLoopData *data, LoopRunner *l)
 		else if (!mData->completed)
 		{
 			// Check if we want to return to menu or replay mission
-			mData->m->IsQuit = mData->ms.current->u.returnCode == 1;
+			gCampaign.IsQuit = mData->ms.current->u.returnCode == 1;
 			LoopRunnerPop(l);
 		}
 		else
@@ -480,10 +647,10 @@ static void MissionSummaryDraw(GameLoopData *data)
 static bool AreAnySurvived(void)
 {
 	CA_FOREACH(const PlayerData, p, gPlayerDatas)
-		if (p->survived)
-		{
-			return true;
-		}
+	if (p->survived)
+	{
+		return true;
+	}
 	CA_FOREACH_END()
 	return false;
 }
@@ -493,10 +660,8 @@ static int GetAccessBonus(const struct MissionOptions *m)
 }
 static int GetTimeBonus(const struct MissionOptions *m, int *secondsOut)
 {
-	int seconds =
-		60 +
-		(int)m->missionData->Objectives.size * 30 -
-		m->time / FPS_FRAMELIMIT;
+	int seconds = 60 + (int)m->missionData->Objectives.size * 30 -
+				  m->time / FPS_FRAMELIMIT;
 	if (seconds < 0)
 	{
 		seconds = 0;
@@ -507,11 +672,6 @@ static int GetTimeBonus(const struct MissionOptions *m, int *secondsOut)
 	}
 	return seconds * 25;
 }
-static int GetHealthBonus(const PlayerData *p);
-static int GetResurrectionFee(const PlayerData *p);
-static int GetButcherPenalty(const PlayerData *p);
-static int GetNinjaBonus(const PlayerData *p);
-static int GetFriendlyBonus(const PlayerData *p);
 static void ApplyBonuses(PlayerData *p, const int bonus)
 {
 	// Apply bonuses to surviving players only
@@ -560,8 +720,8 @@ static int GetNinjaBonus(const PlayerData *p)
 				break;
 			}
 		}
-		if (wc != NULL && !wc->CanShoot && p->Stats.Friendlies == 0 &&
-			p->Stats.Kills > 5)
+		if (wc != NULL && !WeaponClassCanShoot(wc) &&
+			p->Stats.Friendlies == 0 && p->Stats.Kills > 5)
 		{
 			return 50 * p->Stats.Kills;
 		}
@@ -573,138 +733,114 @@ static int GetFriendlyBonus(const PlayerData *p)
 	return (p->Stats.Kills == 0 && p->Stats.Friendlies == 0) ? 500 : 0;
 }
 static void DrawPlayerSummary(
-	const struct vec2i pos, const struct vec2i size, PlayerData *data);
+	const struct vec2i pos, const struct vec2i size,
+	const PlayerSummaryDrawData *data);
 static void MissionSummaryMenuDraw(
-	const menu_t *menu, GraphicsDevice *g,
-	const struct vec2i p, const struct vec2i size, const void *data)
+	const menu_t *menu, GraphicsDevice *g, const struct vec2i p,
+	const struct vec2i size, const void *data)
 {
 	UNUSED(menu);
 	UNUSED(p);
 	UNUSED(size);
-	const struct MissionOptions *m = data;
+	const MissionSummaryData *mData = data;
 
 	const int w = gGraphicsDevice.cachedConfig.Res.x;
 	const int h = gGraphicsDevice.cachedConfig.Res.y;
 
-	// Display password
-	if (strlen(gAutosave.LastMission.Password) > 0)
-	{
-		char s[64];
-		sprintf(s, "Last password: %s", gAutosave.LastMission.Password);
-		FontOpts opts = FontOptsNew();
-		opts.HAlign = ALIGN_CENTER;
-		opts.VAlign = ALIGN_END;
-		opts.Area = g->cachedConfig.Res;
-		opts.Pad.y = opts.Area.y / 12;
-		FontStrOpt(s, svec2i_zero(), opts);
-	}
-
 	// Display objectives and bonuses
 	struct vec2i pos = svec2i(w / 6, h / 2 + h / 10);
 	int idx = 1;
-	CA_FOREACH(const Objective, o, m->missionData->Objectives)
-		// Do not mention optional objectives with none completed
-		if (o->done == 0 && !ObjectiveIsRequired(o))
-		{
-			continue;
-		}
+	CA_FOREACH(const Objective, o, mData->m->missionData->Objectives)
+	// Do not mention optional objectives with none completed
+	if (o->done == 0 && !ObjectiveIsRequired(o))
+	{
+		continue;
+	}
 
-		// Objective icon
-		DrawObjectiveInfo(o, svec2i_add(pos, svec2i(-26, FontH())));
+	// Objective icon
+	DrawObjectiveInfo(o, svec2i_add(pos, svec2i(-26, FontH())));
 
-		// Objective completion text
-		char s[100];
-		sprintf(s, "Objective %d: %d of %d, %d required",
-			idx, o->done, o->Count, o->Required);
-		FontOpts opts = FontOptsNew();
-		opts.Area = gGraphicsDevice.cachedConfig.Res;
-		opts.Pad = pos;
-		if (!ObjectiveIsRequired(o))
-		{
-			// Show optional objectives in purple
-			opts.Mask = colorPurple;
-		}
-		FontStrOpt(s, svec2i_zero(), opts);
+	// Objective completion text
+	char s[100];
+	sprintf(
+		s, "Objective %d: %d of %d, %d required", idx, o->done, o->Count,
+		o->Required);
+	FontOpts opts = FontOptsNew();
+	opts.Area = g->cachedConfig.Res;
+	opts.Pad = pos;
+	if (!ObjectiveIsRequired(o))
+	{
+		// Show optional objectives in purple
+		opts.Mask = colorPurple;
+	}
+	FontStrOpt(s, svec2i_zero(), opts);
 
-		// Objective status text
-		opts = FontOptsNew();
-		opts.HAlign = ALIGN_END;
-		opts.Area = gGraphicsDevice.cachedConfig.Res;
-		opts.Pad = pos;
-		if (!ObjectiveIsComplete(o))
-		{
-			opts.Mask = colorRed;
-			FontStrOpt("Failed", svec2i_zero(), opts);
-		}
-		else if (ObjectiveIsPerfect(o) && AreAnySurvived())
-		{
-			opts.Mask = colorGreen;
-			char buf[16];
-			sprintf(buf, "Perfect: %d", PERFECT_BONUS);
-			FontStrOpt(buf, svec2i_zero(), opts);
-		}
-		else if (ObjectiveIsRequired(o))
-		{
-			FontStrOpt("Done", svec2i_zero(), opts);
-		}
-		else
-		{
-			FontStrOpt("Bonus!", svec2i_zero(), opts);
-		}
+	// Objective status text
+	opts = FontOptsNew();
+	opts.HAlign = ALIGN_END;
+	opts.Area = g->cachedConfig.Res;
+	opts.Pad = pos;
+	if (!ObjectiveIsComplete(o))
+	{
+		opts.Mask = colorRed;
+		FontStrOpt("Failed", svec2i_zero(), opts);
+	}
+	else if (ObjectiveIsPerfect(o) && AreAnySurvived())
+	{
+		opts.Mask = colorGreen;
+		char buf[16];
+		sprintf(buf, "Perfect: %d", PERFECT_BONUS);
+		FontStrOpt(buf, svec2i_zero(), opts);
+	}
+	else if (ObjectiveIsRequired(o))
+	{
+		FontStrOpt("Done", svec2i_zero(), opts);
+	}
+	else
+	{
+		FontStrOpt("Bonus!", svec2i_zero(), opts);
+	}
 
-		pos.y += 15;
-		idx++;
+	pos.y += 15;
+	idx++;
 	CA_FOREACH_END()
 
 	// Draw other bonuses
-	if (AreAnySurvived())
+	if (mData->AccessBonus.prefix)
 	{
-		char s[64];
-
-		sprintf(s, "Access bonus: %d", GetAccessBonus(m));
-		FontStr(s, pos);
-
+		AnimatedCounterDraw(&mData->AccessBonus, pos);
 		pos.y += FontH() + 1;
-		int seconds;
-		const int timeBonus = GetTimeBonus(m, &seconds);
-		sprintf(s, "Time bonus: %d secs x 25 = %d", seconds, timeBonus);
-		FontStr(s, pos);
+	}
+	if (mData->TimeBonus.prefix)
+	{
+		AnimatedCounterDraw(&mData->TimeBonus, pos);
 	}
 
 	// Draw per-player summaries
-	PlayerData *pds[MAX_LOCAL_PLAYERS];
-	idx = 0;
-	CA_FOREACH(PlayerData, pd, gPlayerDatas)
-		if (!pd->IsLocal)
-		{
-			continue;
-		}
-		pds[idx] = pd;
-		idx++;
-	CA_FOREACH_END()
 	struct vec2i playerSize;
-	switch (idx)
+	switch (GetNumPlayers(PLAYER_ANY, false, true))
 	{
 	case 1:
 		playerSize = svec2i(w, h / 2);
-		DrawPlayerSummary(svec2i_zero(), playerSize, pds[0]);
+		DrawPlayerSummary(svec2i_zero(), playerSize, &mData->pDatas[0]);
 		break;
 	case 2:
 		// side by side
 		playerSize = svec2i(w / 2, h / 2);
-		DrawPlayerSummary(svec2i_zero(), playerSize, pds[0]);
-		DrawPlayerSummary(svec2i(w / 2, 0), playerSize, pds[1]);
+		DrawPlayerSummary(svec2i_zero(), playerSize, &mData->pDatas[0]);
+		DrawPlayerSummary(svec2i(w / 2, 0), playerSize, &mData->pDatas[1]);
 		break;
-	case 3:	// fallthrough
+	case 3: // fallthrough
 	case 4:
 		// 2x2
 		playerSize = svec2i(w / 2, h / 4);
-		DrawPlayerSummary(svec2i_zero(), playerSize, pds[0]);
-		DrawPlayerSummary(svec2i(w / 2, 0), playerSize, pds[1]);
-		DrawPlayerSummary(svec2i(0, h / 4), playerSize, pds[2]);
+		DrawPlayerSummary(svec2i_zero(), playerSize, &mData->pDatas[0]);
+		DrawPlayerSummary(svec2i(w / 2, 0), playerSize, &mData->pDatas[1]);
+		DrawPlayerSummary(svec2i(0, h / 4), playerSize, &mData->pDatas[2]);
 		if (idx == 4)
 		{
-			DrawPlayerSummary(svec2i(w / 2, h / 4), playerSize, pds[3]);
+			DrawPlayerSummary(
+				svec2i(w / 2, h / 4), playerSize, &mData->pDatas[3]);
 		}
 		break;
 	default:
@@ -715,19 +851,20 @@ static void MissionSummaryMenuDraw(
 // Display compact player summary, with player on left half and score summaries
 // on right half
 static void DrawPlayerSummary(
-	const struct vec2i pos, const struct vec2i size, PlayerData *data)
+	const struct vec2i pos, const struct vec2i size,
+	const PlayerSummaryDrawData *data)
 {
 	char s[50];
 	const int totalTextHeight = FontH() * 7;
 	// display text on right half
-	struct vec2i textPos = svec2i(
-		pos.x + size.x / 2, CENTER_Y(pos, size, totalTextHeight));
+	struct vec2i textPos =
+		svec2i(pos.x + size.x / 2, CENTER_Y(pos, size, totalTextHeight));
 
 	DisplayCharacterAndName(
-		svec2i_add(pos, svec2i(size.x / 4, size.y / 2)),
-		&data->Char, DIRECTION_DOWN, data->name, colorWhite);
+		svec2i_add(pos, svec2i(size.x / 4, size.y / 2)), &data->Pd->Char,
+		DIRECTION_DOWN, data->Pd->name, colorWhite, data->Pd->guns[0]);
 
-	if (data->survived)
+	if (data->Pd->survived)
 	{
 		FontStr("Completed mission", textPos);
 	}
@@ -737,54 +874,27 @@ static void DrawPlayerSummary(
 	}
 
 	textPos.y += 2 * FontH();
-	sprintf(s, "Score: %d", data->Stats.Score);
-	FontStr(s, textPos);
+	AnimatedCounterDraw(&data->Score, textPos);
 	textPos.y += FontH();
-	sprintf(s, "Total: %d", data->Totals.Score);
-	FontStr(s, textPos);
+	AnimatedCounterDraw(&data->Total, textPos);
 	textPos.y += FontH();
-	sprintf(s, "Missions: %d", data->missions + (data->survived ? 1 : 0));
+	AnimatedCounterTimeDraw(&data->Time, textPos);
+	textPos.y += FontH();
+	AnimatedCounterTimeDraw(&data->TimeTotal, textPos);
+	textPos.y += FontH();
+	sprintf(
+		s, "Missions: %d", data->Pd->missions + (data->Pd->survived ? 1 : 0));
 	FontStr(s, textPos);
 	textPos.y += FontH();
 
-	// Display bonuses if the player has survived
-	if (data->survived)
+	if (data->HealthResurrection.prefix)
 	{
-		const int healthBonus = GetHealthBonus(data);
-		if (healthBonus != 0)
-		{
-			sprintf(s, "Health bonus: %d", healthBonus);
-		}
-		const int resurrectionFee = GetResurrectionFee(data);
-		if (resurrectionFee != 0)
-		{
-			sprintf(s, "Resurrection fee: %d", resurrectionFee);
-		}
-		if (healthBonus != 0 || resurrectionFee != 0)
-		{
-			FontStr(s, textPos);
-			textPos.y += FontH();
-		}
-
-		const int butcherPenalty = GetButcherPenalty(data);
-		if (butcherPenalty != 0)
-		{
-			sprintf(s, "Butcher penalty: %d", butcherPenalty);
-		}
-		const int ninjaBonus = GetNinjaBonus(data);
-		if (ninjaBonus != 0)
-		{
-			sprintf(s, "Ninja bonus: %d", ninjaBonus);
-		}
-		const int friendlyBonus = GetFriendlyBonus(data);
-		if (friendlyBonus != 0)
-		{
-			sprintf(s, "Friendly bonus: %d", friendlyBonus);
-		}
-		if (butcherPenalty != 0 || ninjaBonus != 0 || friendlyBonus != 0)
-		{
-			FontStr(s, textPos);
-		}
+		AnimatedCounterDraw(&data->HealthResurrection, textPos);
+		textPos.y += FontH();
+	}
+	if (data->ButcherNinjaFriendly.prefix)
+	{
+		AnimatedCounterDraw(&data->ButcherNinjaFriendly, textPos);
 	}
 }
 
@@ -794,36 +904,31 @@ static void DrawObjectiveInfo(const Objective *o, const struct vec2i pos)
 
 	switch (o->Type)
 	{
-	case OBJECTIVE_KILL:
-		{
-			const Character *cd = CArrayGet(
-				&store->OtherChars, CharacterStoreGetSpecialId(store, 0));
-			DrawHead(
-				gGraphicsDevice.gameWindow.renderer, cd, DIRECTION_DOWN, pos);
-		}
-		break;
-	case OBJECTIVE_RESCUE:
-		{
-			const Character *cd = CArrayGet(
-				&store->OtherChars, CharacterStoreGetPrisonerId(store, 0));
-			DrawHead(
-				gGraphicsDevice.gameWindow.renderer, cd, DIRECTION_DOWN, pos);
-		}
-		break;
+	case OBJECTIVE_KILL: {
+		const Character *cd = CArrayGet(
+			&store->OtherChars, CharacterStoreGetSpecialId(store, 0));
+		DrawHead(gGraphicsDevice.gameWindow.renderer, cd, DIRECTION_DOWN, pos);
+	}
+	break;
+	case OBJECTIVE_RESCUE: {
+		const Character *cd = CArrayGet(
+			&store->OtherChars, CharacterStoreGetPrisonerId(store, 0));
+		DrawHead(gGraphicsDevice.gameWindow.renderer, cd, DIRECTION_DOWN, pos);
+	}
+	break;
 	case OBJECTIVE_COLLECT:
-		{
-			const Pic *p = CPicGetPic(&o->u.Pickup->Pic, 0);
-			Blit(&gGraphicsDevice, p,
-				svec2i_subtract(pos, svec2i_scale_divide(p->size, 2)));
-		}
-		break;
-	case OBJECTIVE_DESTROY:
-		{
-			struct vec2i picOffset;
-			const Pic *p = MapObjectGetPic(o->u.MapObject, &picOffset);
-			Blit(&gGraphicsDevice, p, svec2i_add(pos, picOffset));
-		}
+		CPicDraw(
+			&gGraphicsDevice, &o->u.Pickup->Pic,
+			svec2i_subtract(pos, svec2i(-4, -4)), NULL);
 		break;
+	case OBJECTIVE_DESTROY: {
+		struct vec2i picOffset;
+		const Pic *p = MapObjectGetPic(o->u.MapObject, &picOffset);
+		PicRender(
+			p, gGraphicsDevice.gameWindow.renderer, svec2i_add(pos, picOffset),
+			colorWhite, 0, svec2_one(), SDL_FLIP_NONE, Rect2iZero());
+	}
+	break;
 	case OBJECTIVE_INVESTIGATE:
 		// Don't draw
 		return;
