@@ -1,30 +1,30 @@
 /*
-    C-Dogs SDL
-    A port of the legendary (and fun) action/arcade cdogs.
+	C-Dogs SDL
+	A port of the legendary (and fun) action/arcade cdogs.
 
-    Copyright (c) 2013-2015, 2017-2018 Cong Xu
-    All rights reserved.
+	Copyright (c) 2013-2015, 2017-2018, 2020-2022 Cong Xu
+	All rights reserved.
 
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions are met:
 
-    Redistributions of source code must retain the above copyright notice, this
-    list of conditions and the following disclaimer.
-    Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
+	Redistributions of source code must retain the above copyright notice, this
+	list of conditions and the following disclaimer.
+	Redistributions in binary form must reproduce the above copyright notice,
+	this list of conditions and the following disclaimer in the documentation
+	and/or other materials provided with the distribution.
 
-    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+	POSSIBILITY OF SUCH DAMAGE.
 */
 #include "ai_coop.h"
 
@@ -36,8 +36,7 @@
 #define CONFUSION_STATE_TICKS_MIN 25
 #define CONFUSION_STATE_TICKS_RANGE 25
 
-#define DISTANCE_TO_KEEP_OUT_OF_WAY SQUARED(4*16)
-
+#define DISTANCE_TO_KEEP_OUT_OF_WAY SQUARED(4 * 16)
 
 static int AICoopGetCmdNormal(TActor *actor);
 int AICoopGetCmd(TActor *actor, const int ticks)
@@ -64,9 +63,8 @@ int AICoopGetCmd(TActor *actor, const int ticks)
 		actor->aiContext->Delay = MAX(0, actor->aiContext->Delay - ticks);
 		if (actor->aiContext->Delay == 0)
 		{
-			actor->aiContext->Delay =
-				CONFUSION_STATE_TICKS_MIN +
-				(rand() % CONFUSION_STATE_TICKS_RANGE);
+			actor->aiContext->Delay = CONFUSION_STATE_TICKS_MIN +
+									  (rand() % CONFUSION_STATE_TICKS_RANGE);
 			if (s->Type == AI_CONFUSION_CONFUSED)
 			{
 				s->Type = AI_CONFUSION_CORRECT;
@@ -76,9 +74,8 @@ int AICoopGetCmd(TActor *actor, const int ticks)
 				ActorSetAIState(actor, AI_STATE_CONFUSED);
 				s->Type = AI_CONFUSION_CONFUSED;
 				// Generate the confused action
-				s->Cmd = rand() &
-					(CMD_LEFT | CMD_RIGHT | CMD_UP | CMD_DOWN |
-					CMD_BUTTON1 | CMD_BUTTON2);
+				s->Cmd = rand() & (CMD_LEFT | CMD_RIGHT | CMD_UP | CMD_DOWN |
+								   CMD_BUTTON1 | CMD_BUTTON2);
 			}
 		}
 		// Choose confusion action based on state
@@ -136,24 +133,25 @@ static int AICoopGetCmdNormal(TActor *actor)
 	struct vec2i v;
 	struct vec2 dangerBulletPos = svec2_zero();
 	for (v.x = actorTilePos.x - 1;
-		v.x <= actorTilePos.x + 1 && svec2_is_zero(dangerBulletPos);
-		v.x++)
+		 v.x <= actorTilePos.x + 1 && svec2_is_zero(dangerBulletPos); v.x++)
 	{
 		for (v.y = actorTilePos.y - 1;
-			v.y <= actorTilePos.y + 1 && svec2_is_zero(dangerBulletPos);
-			v.y++)
+			 v.y <= actorTilePos.y + 1 && svec2_is_zero(dangerBulletPos);
+			 v.y++)
 		{
 			const Tile *t = MapGetTile(&gMap, v);
-			if (t == NULL) continue;
+			if (t == NULL)
+				continue;
 			CA_FOREACH(const ThingId, tid, t->things)
-				// Only look for bullets
-				if (tid->Kind != KIND_MOBILEOBJECT) continue;
-				const TMobileObject *mo = CArrayGet(&gMobObjs, tid->Id);
-				if (mo->bulletClass->HurtAlways)
-				{
-					dangerBulletPos = mo->Pos;
-					break;
-				}
+			// Only look for bullets
+			if (tid->Kind != KIND_MOBILEOBJECT)
+				continue;
+			const TMobileObject *mo = CArrayGet(&gMobObjs, tid->Id);
+			if (mo->bulletClass->HurtAlways)
+			{
+				dangerBulletPos = mo->thing.Pos;
+				break;
+			}
 			CA_FOREACH_END()
 		}
 	}
@@ -165,7 +163,7 @@ static int AICoopGetCmdNormal(TActor *actor)
 
 	// Check the weapon for ammo
 	int lowAmmoGun = -1;
-	if (ConfigGetBool(&gConfig, "Game.Ammo"))
+	if (gCampaign.Setting.Ammo && actor->aiContext->OnGunId == -1)
 	{
 		// Check all our weapons
 		// Prefer guns using ammo
@@ -175,11 +173,12 @@ static int AICoopGetCmdNormal(TActor *actor)
 		for (int i = 0; i < MAX_GUNS; i++)
 		{
 			const Weapon *w = &actor->guns[i];
-			if (w->Gun == NULL)
+			// TODO: support multi guns?
+			if (w->Gun == NULL || w->Gun->Type != GUNTYPE_NORMAL)
 			{
 				continue;
 			}
-			const int ammoAmount = ActorWeaponGetAmmo(actor, w->Gun);
+			const int ammoAmount = ActorWeaponGetAmmo(actor, w->Gun, 0);
 			if ((mostPreferredWeaponWithAmmo == -1 && ammoAmount != 0) ||
 				(!mostPreferredWeaponHasAmmo && ammoAmount > 0))
 			{
@@ -187,11 +186,11 @@ static int AICoopGetCmdNormal(TActor *actor)
 				mostPreferredWeaponWithAmmo = i;
 				mostPreferredWeaponHasAmmo = ammoAmount > 0;
 			}
-			if (w->Gun->AmmoId != -1)
+			if (w->Gun->u.Normal.AmmoId != -1)
 			{
-				const Ammo *ammo = AmmoGetById(&gAmmo, w->Gun->AmmoId);
-				if (lowAmmoGun == -1 &&
-					ammoAmount < ammo->Amount * AMMO_STARTING_MULTIPLE)
+				const Ammo *ammo =
+					AmmoGetById(&gAmmo, w->Gun->u.Normal.AmmoId);
+				if (lowAmmoGun == -1 && ammoAmount < ammo->Amount)
 				{
 					lowAmmoGun = i;
 				}
@@ -214,12 +213,11 @@ static int AICoopGetCmdNormal(TActor *actor)
 	// - if we have a free slot
 	if (actor->aiContext->OnGunId != -1)
 	{
-		const WeaponClass *wc =
-			IdWeaponClass(actor->aiContext->OnGunId);
-		if ((lowAmmoGun != -1 && !ActorHasGun(actor, wc)) ||
-			(wc->IsGrenade ?
-			ActorGetNumGrenades(actor) < MAX_GRENADES :
-			ActorGetNumGuns(actor) < MAX_GUNS))
+		const WeaponClass *wc = IdWeaponClass(actor->aiContext->OnGunId);
+		if ((lowAmmoGun != -1 && ActorFindGun(actor, wc) < 0) ||
+			(wc->Type == GUNTYPE_GRENADE
+				 ? ActorGetNumGrenades(actor) < MAX_GRENADES
+				 : ActorGetNumGuns(actor) < MAX_GUNS))
 		{
 			actor->aiContext->OnGunId = -1;
 			// Pick it up
@@ -240,10 +238,10 @@ static int AICoopGetCmdNormal(TActor *actor)
 		for (int uid = 0; uid < actor->PlayerUID; uid++)
 		{
 			const PlayerData *pd = PlayerDataGetByUID(uid);
-			if (pd == NULL || !IsPlayerAlive(pd)) continue;
+			if (pd == NULL || !IsPlayerAlive(pd))
+				continue;
 			const TActor *p = ActorGetByUID(pd->ActorUID);
-			const float distance2 = svec2_distance_squared(
-				actor->Pos, p->Pos);
+			const float distance2 = svec2_distance_squared(actor->Pos, p->Pos);
 			if (!closestPlayer || distance2 < minDistance2)
 			{
 				minDistance2 = distance2;
@@ -266,7 +264,7 @@ static int AICoopGetCmdNormal(TActor *actor)
 	{
 		distanceTooFarFromPlayer = 2.0f;
 	}
-	if (closestPlayer && minDistance2 > SQUARED(distanceTooFarFromPlayer*16))
+	if (closestPlayer && minDistance2 > SQUARED(distanceTooFarFromPlayer * 16))
 	{
 		ActorSetAIState(actor, AI_STATE_FOLLOW);
 		return SmartGoto(actor, closestPlayer->Pos, minDistance2);
@@ -277,8 +275,8 @@ static int AICoopGetCmdNormal(TActor *actor)
 	if (closestEnemy)
 	{
 		const float minEnemyDistance = CHEBYSHEV_DISTANCE(
-			actor->Pos.x, actor->Pos.y,
-			closestEnemy->Pos.x, closestEnemy->Pos.y);
+			actor->Pos.x, actor->Pos.y, closestEnemy->Pos.x,
+			closestEnemy->Pos.y);
 		// Also only engage if there's a clear shot
 		if (minEnemyDistance > 0 && minEnemyDistance < 12 * 16 &&
 			AIHasClearShot(actor->Pos, closestEnemy->Pos))
@@ -302,7 +300,7 @@ static int AICoopGetCmdNormal(TActor *actor)
 	// Look for objectives nearby to complete
 	int cmd;
 	if (TryCompleteNearbyObjective(
-		actor, closestPlayer, distanceTooFarFromPlayer, &cmd))
+			actor, closestPlayer, distanceTooFarFromPlayer, &cmd))
 	{
 		return cmd;
 	}
@@ -311,15 +309,14 @@ static int AICoopGetCmdNormal(TActor *actor)
 	// run into them, but keep away if we're too close
 	if (closestPlayer)
 	{
-		if (minDistance2 > SQUARED(2*16))
+		if (minDistance2 > SQUARED(2 * 16))
 		{
 			ActorSetAIState(actor, AI_STATE_FOLLOW);
 			return SmartGoto(actor, closestPlayer->Pos, minDistance2);
 		}
-		else if (minDistance2 < SQUARED(4*16/3))
+		else if (minDistance2 < SQUARED(4 * 16 / 3))
 		{
-			return
-				CmdGetReverse(AIGotoDirect(actor->Pos, closestPlayer->Pos));
+			return CmdGetReverse(AIGotoDirect(actor->Pos, closestPlayer->Pos));
 		}
 	}
 
@@ -351,7 +348,7 @@ static int SmartGoto(
 		svec2i_is_equal(tilePos, actor->aiContext->LastTile))
 	{
 		cmd = AIGoto(actor, o->thing.Pos, true);
-		if (ACTOR_GET_WEAPON(actor)->lock <= 0)
+		if (ActorGetCanFireBarrel(actor, ACTOR_GET_WEAPON(actor)) >= 0)
 		{
 			cmd |= CMD_BUTTON1;
 		}
@@ -385,8 +382,7 @@ static int SmartGoto(
 typedef struct
 {
 	float Distance2;
-	union
-	{
+	union {
 		const Objective *Objective;
 		int UID;
 	} u;
@@ -423,20 +419,18 @@ static bool TryCompleteNearbyObjective(
 			hasNoUpdates =
 				objState->u.Obj && objState->LastDone == objState->u.Obj->done;
 			break;
-		case AI_OBJECTIVE_TYPE_KILL:
-			{
-				const TActor *target = ActorGetByUID(objState->u.UID);
-				hasNoUpdates = target->health > 0;
-				// Update target position
-				objState->Goal = target->thing.Pos;
-			}
-			break;
-		case AI_OBJECTIVE_TYPE_PICKUP:
-			{
-				const Pickup *p = PickupGetByUID(objState->u.UID);
-				hasNoUpdates = p != NULL && p->isInUse;
-			}
-			break;
+		case AI_OBJECTIVE_TYPE_KILL: {
+			const TActor *target = ActorGetByUID(objState->u.UID);
+			hasNoUpdates = target->health > 0;
+			// Update target position
+			objState->Goal = target->thing.Pos;
+		}
+		break;
+		case AI_OBJECTIVE_TYPE_PICKUP: {
+			const Pickup *p = PickupGetByUID(objState->u.UID);
+			hasNoUpdates = p != NULL && p->isInUse;
+		}
+		break;
 		default:
 			// Do nothing
 			break;
@@ -457,16 +451,23 @@ static bool TryCompleteNearbyObjective(
 	}
 
 	// First, check if mission complete;
-	// if so (and there's a path) go to exit
-	const struct vec2 exitPos = MapGetExitPos(&gMap);
-	if (CanCompleteMission(&gMission) && CanGetObjective(
-		exitPos, actor->Pos, closestPlayer, distanceTooFarFromPlayer))
+	// if so (and there's a path) go to an exit
+	if (CanCompleteMission(&gMission))
 	{
-		ActorSetAIState(actor, AI_STATE_NEXT_OBJECTIVE);
-		objState->Type = AI_OBJECTIVE_TYPE_EXIT;
-		objState->Goal = exitPos;
-		*cmdOut = GotoObjective(actor, 0);
-		return true;
+		for (int i = 0; i < (int)gMap.exits.size; i++)
+		{
+			const struct vec2 exitPos = MapGetExitPos(&gMap, i);
+			if (CanGetObjective(
+					exitPos, actor->Pos, closestPlayer,
+					distanceTooFarFromPlayer))
+			{
+				ActorSetAIState(actor, AI_STATE_NEXT_OBJECTIVE);
+				objState->Type = AI_OBJECTIVE_TYPE_EXIT;
+				objState->Goal = exitPos;
+				*cmdOut = GotoObjective(actor, 0);
+				return true;
+			}
+		}
 	}
 
 	// Find all the objective/key locations, sort according to distance
@@ -476,41 +477,99 @@ static bool TryCompleteNearbyObjective(
 
 	// Starting from the closest objectives, find one we can go to
 	CA_FOREACH(ClosestObjective, c, objectives)
-		if (CanGetObjective(
+	if (CanGetObjective(
 			c->Pos, actor->Pos, closestPlayer, distanceTooFarFromPlayer))
+	{
+		ActorSetAIState(actor, AI_STATE_NEXT_OBJECTIVE);
+		objState->Type = c->Type;
+		objState->IsDestructible = c->IsDestructible;
+		switch (c->Type)
 		{
-			ActorSetAIState(actor, AI_STATE_NEXT_OBJECTIVE);
-			objState->Type = c->Type;
-			objState->IsDestructible = c->IsDestructible;
-			switch (c->Type)
-			{
-			case AI_OBJECTIVE_TYPE_KEY:
-				objState->LastDone = KeycardCount(gMission.KeyFlags);
-				break;
-			case AI_OBJECTIVE_TYPE_NORMAL:
-				objState->u.Obj = c->u.Objective;
-				objState->LastDone = c->u.Objective->done;
-				break;
-			case AI_OBJECTIVE_TYPE_KILL:
-				objState->u.UID = c->u.UID;
-				break;
-			case AI_OBJECTIVE_TYPE_PICKUP:
-				objState->u.UID = c->u.UID;
-				break;
-			default:
-				// Do nothing
-				break;
-			}
-			objState->Goal = c->Pos;
-			*cmdOut = GotoObjective(actor, c->Distance2);
-			return true;
+		case AI_OBJECTIVE_TYPE_KEY:
+			objState->LastDone = KeycardCount(gMission.KeyFlags);
+			break;
+		case AI_OBJECTIVE_TYPE_NORMAL:
+			objState->u.Obj = c->u.Objective;
+			objState->LastDone = c->u.Objective->done;
+			break;
+		case AI_OBJECTIVE_TYPE_KILL:
+			objState->u.UID = c->u.UID;
+			break;
+		case AI_OBJECTIVE_TYPE_PICKUP:
+			objState->u.UID = c->u.UID;
+			break;
+		default:
+			// Do nothing
+			break;
 		}
+		objState->Goal = c->Pos;
+		*cmdOut = GotoObjective(actor, c->Distance2);
+		return true;
+	}
 	CA_FOREACH_END()
 	return false;
 }
-static bool OnClosestPickupGun(
-	ClosestObjective *co, const Pickup *p,
-	const TActor *actor, const TActor *closestPlayer);
+static bool ShouldPickupGun(const PickupEffect *pe, const TActor *actor,
+							const TActor *closestPlayer);
+static AIObjectiveType GetPickupObjectiveType(const Pickup *p, int *uid, const TActor *actor, const TActor *closestPlayer)
+{
+	CA_FOREACH(const PickupEffect, pe, p->class->Effects)
+	switch (pe->Type)
+	{
+	case PICKUP_KEYCARD:
+		return AI_OBJECTIVE_TYPE_KEY;
+	case PICKUP_JEWEL:
+		*uid = p->UID;
+		return AI_OBJECTIVE_TYPE_PICKUP;
+	case PICKUP_HEALTH:
+		// Pick up if we are on low health, and lower than lead player
+		if (actor->health > ModeMaxHealth(gCampaign.Entry.Mode) / 4 ||
+			(closestPlayer != NULL && actor->health >= closestPlayer->health))
+		{
+			continue;
+		}
+		*uid = p->UID;
+		return AI_OBJECTIVE_TYPE_PICKUP;
+	case PICKUP_AMMO: {
+		// Pick up if we use this ammo, have less ammo than starting,
+		// and lower than lead player, who uses the ammo
+		const int ammoId = pe->u.Ammo.Id;
+		const Ammo *ammo = AmmoGetById(&gAmmo, ammoId);
+		const int ammoAmount = *(int *)CArrayGet(&actor->ammo, ammoId);
+		if (!ActorUsesAmmo(actor, ammoId) || ammoAmount > ammo->Amount ||
+			(closestPlayer != NULL && ActorUsesAmmo(closestPlayer, ammoId) &&
+			 ammoAmount > *(int *)CArrayGet(&closestPlayer->ammo, ammoId)))
+		{
+			continue;
+		}
+		*uid = p->UID;
+		return AI_OBJECTIVE_TYPE_PICKUP;
+	}
+	case PICKUP_GUN:
+		if (ShouldPickupGun(pe, actor, closestPlayer))
+		{
+			*uid = p->UID;
+			return AI_OBJECTIVE_TYPE_PICKUP;
+		}
+		break;
+	case PICKUP_SHOW_MAP:
+		*uid = p->UID;
+		return AI_OBJECTIVE_TYPE_PICKUP;
+	case PICKUP_LIVES:
+		// Pick up if we have less lives than lead player
+		if (closestPlayer != NULL && PlayerDataGetByUID(actor->PlayerUID)->Lives >= PlayerDataGetByUID(closestPlayer->PlayerUID)->Lives)
+		{
+			continue;
+		}
+		*uid = p->UID;
+		return AI_OBJECTIVE_TYPE_PICKUP;
+	default:
+		// Not something we want to pick up
+		continue;
+	}
+	CA_FOREACH_END()
+	return AI_OBJECTIVE_TYPE_NORMAL;
+}
 static int CompareClosestObjective(const void *v1, const void *v2);
 static void FindObjectivesSortedByDistance(
 	CArray *objectives, const TActor *actor, const TActor *closestPlayer)
@@ -520,8 +579,7 @@ static void FindObjectivesSortedByDistance(
 	// If PVP, find the closest enemy and go to them
 	if (IsPVP(gCampaign.Entry.Mode))
 	{
-		const TActor *closestEnemy =
-			AIGetClosestVisibleEnemy(actor, true);
+		const TActor *closestEnemy = AIGetClosestVisibleEnemy(actor, true);
 		if (closestEnemy != NULL)
 		{
 			ClosestObjective co;
@@ -537,197 +595,165 @@ static void FindObjectivesSortedByDistance(
 
 	// Look for pickups
 	CA_FOREACH(const Pickup, p, gPickups)
-		if (!p->isInUse)
-		{
-			continue;
-		}
-		ClosestObjective co;
-		memset(&co, 0, sizeof co);
-		co.Pos = p->thing.Pos;
-		co.IsDestructible = false;
-		co.Type = AI_OBJECTIVE_TYPE_NORMAL;
-		switch (p->class->Type)
-		{
-		case PICKUP_KEYCARD:
-			co.Type = AI_OBJECTIVE_TYPE_KEY;
-			break;
-		case PICKUP_JEWEL:
-			break;
-		case PICKUP_HEALTH:
-			// Pick up if we are on low health, and lower than lead player
-			if (actor->health > ModeMaxHealth(gCampaign.Entry.Mode) / 4)
-			{
-				continue;
-			}
-			co.Type = AI_OBJECTIVE_TYPE_PICKUP;
-			co.u.UID = p->UID;
-			break;
-		case PICKUP_AMMO:
-			{
-				// Pick up if we use this ammo, have less ammo than starting,
-				// and lower than lead player, who uses the ammo
-				const int ammoId = p->class->u.Ammo.Id;
-				const Ammo *ammo = AmmoGetById(&gAmmo, ammoId);
-				const int ammoAmount = *(int *)CArrayGet(&actor->ammo, ammoId);
-				if (!ActorUsesAmmo(actor, ammoId) ||
-					ammoAmount > ammo->Amount * AMMO_STARTING_MULTIPLE ||
-					(closestPlayer != NULL &&
-					ActorUsesAmmo(closestPlayer, ammoId) &&
-					ammoAmount > *(int *)CArrayGet(&closestPlayer->ammo, ammoId)))
-				{
-					continue;
-				}
-				co.Type = AI_OBJECTIVE_TYPE_PICKUP;
-				co.u.UID = p->UID;
-			}
-			break;
-		case PICKUP_GUN:
-			if (!OnClosestPickupGun(&co, p, actor, closestPlayer))
-			{
-				continue;
-			}
-			break;
-		default:
-			// Not something we want to pick up
-			continue;
-		}
-		// Check if the pickup is actually accessible
-		// This is because random spawning may cause some pickups to be spawned
-		// in inaccessible areas
-		if (!TileCanWalk(MapGetTile(&gMap, Vec2ToTile(co.Pos))))
-		{
-			continue;
-		}
-		co.Distance2 = svec2_distance_squared(actor->Pos, co.Pos);
-		if (co.Type == AI_OBJECTIVE_TYPE_NORMAL)
-		{
-			const int objective = ObjectiveFromThing(p->thing.flags);
-			co.u.Objective =
-				CArrayGet(&gMission.missionData->Objectives, objective);
-		}
-		CArrayPushBack(objectives, &co);
+	if (!p->isInUse)
+	{
+		continue;
+	}
+	ClosestObjective co;
+	memset(&co, 0, sizeof co);
+	co.Pos = p->thing.Pos;
+	co.IsDestructible = false;
+	co.Type = GetPickupObjectiveType(p, &co.u.UID, actor, closestPlayer);
+	if (co.Type == AI_OBJECTIVE_TYPE_NORMAL)
+	{
+		continue;
+	}
+	// Check if the pickup is actually accessible
+	// This is because random spawning may cause some pickups to be spawned
+	// in inaccessible areas
+	if (!TileCanWalk(MapGetTile(&gMap, Vec2ToTile(co.Pos))))
+	{
+		continue;
+	}
+	co.Distance2 = svec2_distance_squared(actor->Pos, co.Pos);
+	if (co.Type == AI_OBJECTIVE_TYPE_NORMAL)
+	{
+		const int objective = ObjectiveFromThing(p->thing.flags);
+		co.u.Objective =
+			CArrayGet(&gMission.missionData->Objectives, objective);
+	}
+	CArrayPushBack(objectives, &co);
 	CA_FOREACH_END()
 
 	// Look for destructibles
 	CA_FOREACH(const TObject, o, gObjs)
-		if (!o->isInUse)
-		{
-			continue;
-		}
-		ClosestObjective co;
-		memset(&co, 0, sizeof co);
-		co.Pos = o->thing.Pos;
-		co.IsDestructible = true;
-		co.Type = AI_OBJECTIVE_TYPE_NORMAL;
-		if (!(o->thing.flags & THING_OBJECTIVE))
-		{
-			continue;
-		}
-		// Destructible objective; go towards it and fire
-		co.Distance2 = svec2_distance_squared(actor->Pos, co.Pos);
-		if (co.Type == AI_OBJECTIVE_TYPE_NORMAL)
-		{
-			const int objective = ObjectiveFromThing(o->thing.flags);
-			co.u.Objective =
-				CArrayGet(&gMission.missionData->Objectives, objective);
-		}
-		CArrayPushBack(objectives, &co);
+	if (!o->isInUse)
+	{
+		continue;
+	}
+	ClosestObjective co;
+	memset(&co, 0, sizeof co);
+	co.Pos = o->thing.Pos;
+	co.IsDestructible = true;
+	co.Type = AI_OBJECTIVE_TYPE_NORMAL;
+	if (!(o->thing.flags & THING_OBJECTIVE))
+	{
+		continue;
+	}
+	// Destructible objective; go towards it and fire
+	co.Distance2 = svec2_distance_squared(actor->Pos, co.Pos);
+	if (co.Type == AI_OBJECTIVE_TYPE_NORMAL)
+	{
+		const int objective = ObjectiveFromThing(o->thing.flags);
+		co.u.Objective =
+			CArrayGet(&gMission.missionData->Objectives, objective);
+	}
+	CArrayPushBack(objectives, &co);
 	CA_FOREACH_END()
 
 	// Look for kill or rescue objectives
 	CA_FOREACH(const TActor, a, gActors)
-		if (!a->isInUse)
-		{
-			continue;
-		}
-		const Thing *ti = &a->thing;
-		if (!(ti->flags & THING_OBJECTIVE))
-		{
-			continue;
-		}
-		const int objective = ObjectiveFromThing(ti->flags);
-		const Objective *o =
-			CArrayGet(&gMission.missionData->Objectives, objective);
-		if (o->Type != OBJECTIVE_KILL && o->Type != OBJECTIVE_RESCUE)
-		{
-			continue;
-		}
-		// Only rescue those that need to be rescued
-		if (o->Type == OBJECTIVE_RESCUE && !(a->flags & FLAGS_PRISONER))
-		{
-			continue;
-		}
-		ClosestObjective co;
-		memset(&co, 0, sizeof co);
-		co.Pos = ti->Pos;
-		co.IsDestructible = false;
-		co.Type = AI_OBJECTIVE_TYPE_NORMAL;
-		co.Distance2 = svec2_distance_squared(actor->Pos, co.Pos);
-		co.u.Objective = o;
-		CArrayPushBack(objectives, &co);
+	if (!a->isInUse)
+	{
+		continue;
+	}
+	const Thing *ti = &a->thing;
+	if (!(ti->flags & THING_OBJECTIVE))
+	{
+		continue;
+	}
+	const int objective = ObjectiveFromThing(ti->flags);
+	const Objective *o =
+		CArrayGet(&gMission.missionData->Objectives, objective);
+	if (o->Type != OBJECTIVE_KILL && o->Type != OBJECTIVE_RESCUE)
+	{
+		continue;
+	}
+	// Only rescue those that need to be rescued
+	if (o->Type == OBJECTIVE_RESCUE && !(a->flags & FLAGS_PRISONER))
+	{
+		continue;
+	}
+	ClosestObjective co;
+	memset(&co, 0, sizeof co);
+	co.Pos = ti->Pos;
+	co.IsDestructible = false;
+	co.Type = AI_OBJECTIVE_TYPE_NORMAL;
+	co.Distance2 = svec2_distance_squared(actor->Pos, co.Pos);
+	co.u.Objective = o;
+	CArrayPushBack(objectives, &co);
 	CA_FOREACH_END()
 
 	// Look for explore objectives
 	CA_FOREACH(const Objective, o, gMission.missionData->Objectives)
-		if (o->Type != OBJECTIVE_INVESTIGATE)
-		{
-			continue;
-		}
-		if (ObjectiveIsComplete(o) || MapGetExploredPercentage(&gMap) == 100)
-		{
-			continue;
-		}
-		// Find the nearest unexplored tile
-		const struct vec2i actorTile = Vec2ToTile(actor->Pos);
-		const struct vec2i unexploredTile = MapSearchTileAround(
-			&gMap, actorTile, MapTileIsUnexplored);
-		ClosestObjective co;
-		memset(&co, 0, sizeof co);
-		co.Pos = Vec2CenterOfTile(unexploredTile);
-		co.IsDestructible = false;
-		co.Type = AI_OBJECTIVE_TYPE_NORMAL;
-		co.Distance2 = svec2_distance_squared(actor->Pos, co.Pos);
-		co.u.Objective = o;
-		CArrayPushBack(objectives, &co);
+	if (o->Type != OBJECTIVE_INVESTIGATE)
+	{
+		continue;
+	}
+	if (ObjectiveIsComplete(o) || MapGetExploredPercentage(&gMap) == 100)
+	{
+		continue;
+	}
+	// Find the nearest unexplored tile
+	const struct vec2i actorTile = Vec2ToTile(actor->Pos);
+	const struct vec2i unexploredTile =
+		MapSearchTileAround(&gMap, actorTile, MapTileIsUnexplored);
+	ClosestObjective co;
+	memset(&co, 0, sizeof co);
+	co.Pos = Vec2CenterOfTile(unexploredTile);
+	co.IsDestructible = false;
+	co.Type = AI_OBJECTIVE_TYPE_NORMAL;
+	co.Distance2 = svec2_distance_squared(actor->Pos, co.Pos);
+	co.u.Objective = o;
+	CArrayPushBack(objectives, &co);
 	CA_FOREACH_END()
 
 	// Sort according to distance
 	qsort(
-		objectives->data,
-		objectives->size, objectives->elemSize, CompareClosestObjective);
+		objectives->data, objectives->size, objectives->elemSize,
+		CompareClosestObjective);
 }
-static bool OnClosestPickupGun(
-	ClosestObjective *co, const Pickup *p,
-	const TActor *actor, const TActor *closestPlayer)
+static bool ShouldPickupGun(const PickupEffect *pe, const TActor *actor,
+	const TActor *closestPlayer)
 {
-	if (!ConfigGetBool(&gConfig, "Game.Ammo"))
+	if (!gCampaign.Setting.Ammo)
 	{
 		return false;
 	}
+	const WeaponClass *pickupGun = IdWeaponClass(pe->u.GunId);
+	const int weaponIndexStart =
+		pickupGun->Type == GUNTYPE_GRENADE ? MAX_GUNS : 0;
+	const int weaponIndexEnd =
+		pickupGun->Type == GUNTYPE_GRENADE ? MAX_WEAPONS : MAX_GUNS;
 	// Pick up if:
 	// - we have a gun with less ammo than starting,
 	// - and lower than lead player, who uses the ammo,
 	// - or if there is a free weapon slot,
-	// - and the
+	// - and the gun is the same type (normal, grenade)
 	bool hasGunLowOnAmmo = false;
-	for (int i = 0; i < MAX_WEAPONS; i++)
+	for (int i = weaponIndexStart; i < weaponIndexEnd; i++)
 	{
-		if (actor->guns[i].Gun == NULL)
+		const WeaponClass *wc = actor->guns[i].Gun;
+		if (wc == NULL)
 		{
 			hasGunLowOnAmmo = true;
 			break;
 		}
-		const int ammoId = actor->guns[i].Gun->AmmoId;
+		// TODO: support multi guns
+		if (wc->Type != GUNTYPE_NORMAL)
+		{
+			continue;
+		}
+		const int ammoId = wc->u.Normal.AmmoId;
 		if (ammoId < 0)
 		{
 			continue;
 		}
 		const Ammo *ammo = AmmoGetById(&gAmmo, ammoId);
 		const int ammoAmount = *(int *)CArrayGet(&actor->ammo, ammoId);
-		if (ammoAmount >= ammo->Amount * AMMO_STARTING_MULTIPLE ||
-			(closestPlayer != NULL &&
-			ActorUsesAmmo(closestPlayer, ammoId) &&
-			ammoAmount > *(int *)CArrayGet(&closestPlayer->ammo, ammoId)))
+		if (ammoAmount >= ammo->Amount ||
+			(closestPlayer != NULL && ActorUsesAmmo(closestPlayer, ammoId) &&
+			 ammoAmount > *(int *)CArrayGet(&closestPlayer->ammo, ammoId)))
 		{
 			continue;
 		}
@@ -739,13 +765,10 @@ static bool OnClosestPickupGun(
 		return false;
 	}
 
-	if (ActorHasGun(actor, IdWeaponClass(p->class->u.GunId)))
+	if (ActorFindGun(actor, IdWeaponClass(pe->u.GunId)) >= 0)
 	{
 		return false;
 	}
-
-	co->Type = AI_OBJECTIVE_TYPE_PICKUP;
-	co->u.UID = p->UID;
 	return true;
 }
 static int CompareClosestObjective(const void *v1, const void *v2)
@@ -793,8 +816,7 @@ static bool IsPosCloseEnoughToPlayer(
 	{
 		return true;
 	}
-	const float distanceFromPlayer2 =
-		svec2_distance_squared(pos, player->Pos);
+	const float distanceFromPlayer2 = svec2_distance_squared(pos, player->Pos);
 	const float distanceMax2 = SQUARED(distanceTooFarFromPlayer * 16);
 	return distanceFromPlayer2 < distanceMax2;
 }
@@ -811,22 +833,23 @@ static int GotoObjective(TActor *actor, const float objDistance2)
 		objState->Type == AI_OBJECTIVE_TYPE_NORMAL && objState->IsDestructible;
 	if (!isDestruction ||
 		svec2_distance_squared(actor->Pos, goal) > SQUARED(3 * 16) ||
-		!AIHasClearShot(actor->Pos, goal))
+		!AIHasClearView(actor, goal, 10 * 16))
 	{
 		cmd = SmartGoto(actor, goal, objDistance2);
 	}
-	else if (isDestruction && ACTOR_GET_WEAPON(actor)->lock <= 0)
+	else if (
+		isDestruction &&
+		ActorGetCanFireBarrel(actor, ACTOR_GET_WEAPON(actor)) >= 0)
 	{
 		cmd = AIHunt(actor, goal);
 		if (AIHasClearShot(actor->Pos, goal))
 		{
-			 cmd |= CMD_BUTTON1;
+			cmd |= CMD_BUTTON1;
 		}
 	}
 	return cmd;
 }
 
-static bool PlayerHasWeapon(const PlayerData *p, const WeaponClass *wc);
 void AICoopSelectWeapons(
 	PlayerData *p, const int player, const CArray *weapons)
 {
@@ -844,7 +867,8 @@ void AICoopSelectWeapons(
 			break;
 		}
 		const WeaponClass **wc = CArrayGet(weapons, idx);
-		if ((*wc)->IsGrenade)
+		// TODO: support multi guns
+		if ((*wc)->Type != GUNTYPE_NORMAL)
 		{
 			continue;
 		}
@@ -852,7 +876,7 @@ void AICoopSelectWeapons(
 		gunCount++;
 	}
 
-	if (ConfigGetBool(&gConfig, "Game.Ammo"))
+	if (gCampaign.Setting.Ammo)
 	{
 		// Select pistol as an infinite-ammo backup
 		const WeaponClass *pistol = StrWeaponClass("Pistol");
@@ -872,18 +896,6 @@ void AICoopSelectWeapons(
 		}
 	}
 }
-static bool PlayerHasWeapon(const PlayerData *p, const WeaponClass *wc)
-{
-	for (int i = 0; i < MAX_WEAPONS; i++)
-	{
-		const WeaponClass *wc2 = p->guns[i];
-		if (wc == wc2)
-		{
-			return true;
-		}
-	}
-	return false;
-}
 
 void AICoopOnPickupGun(TActor *a, const int gunId)
 {
