@@ -1,27 +1,27 @@
 /*
-    Copyright (c) 2014-2017 Cong Xu
-    All rights reserved.
+	Copyright (c) 2014-2017, 2019, 2021-2022 Cong Xu
+	All rights reserved.
 
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions are met:
 
-    Redistributions of source code must retain the above copyright notice, this
-    list of conditions and the following disclaimer.
-    Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
+	Redistributions of source code must retain the above copyright notice, this
+	list of conditions and the following disclaimer.
+	Redistributions in binary form must reproduce the above copyright notice,
+	this list of conditions and the following disclaimer in the documentation
+	and/or other materials provided with the distribution.
 
-    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+	POSSIBILITY OF SUCH DAMAGE.
 */
 #include "font.h"
 
@@ -29,12 +29,13 @@
 #include <string.h>
 
 #ifdef __EMSCRIPTEN__
-#include <SDL/SDL_image.h>
+#include <SDL2/SDL_image.h>
 #else
 #include <SDL_image.h>
 #endif
 
 #include "blit.h"
+#include "log.h"
 #include "pic.h"
 #include "sys_config.h"
 #include "utils.h"
@@ -54,23 +55,26 @@ FontOpts FontOptsNew(void)
 	return opts;
 }
 
-void FontLoad(Font *f, const char *imgPath, const bool isProportional)
+void FontLoad(
+	Font *f, const char *imgPath, const bool isProportional,
+	const struct vec2i spaceSize)
 {
 	char buf[CDOGS_PATH_MAX];
 	GetDataFilePath(buf, imgPath);
 	SDL_RWops *rwops = SDL_RWFromFile(buf, "rb");
 	CASSERT(IMG_isPNG(rwops), "Error: font file is not PNG");
 	SDL_Surface *image = IMG_Load_RW(rwops, 0);
+	rwops->close(rwops);
 	if (!image)
 	{
-		fprintf(stderr, "Cannot load font image: %s\n", IMG_GetError());
-		goto bail;
+		LOG(LM_GFX, LL_ERROR, "Cannot load font image: %s", IMG_GetError());
+		return;
 	}
 	if (image->format->BytesPerPixel != 4)
 	{
 		perror("Cannot load non-32-bit image");
-		fprintf(stderr, "Only 32-bit depth images supported\n");
-		goto bail;
+		LOG(LM_GFX, LL_ERROR, "Only 32-bit depth images supported");
+		return;
 	}
 
 	CArrayInit(&f->Chars, sizeof(Pic));
@@ -81,14 +85,14 @@ void FontLoad(Font *f, const char *imgPath, const bool isProportional)
 		f->Size.y + f->Padding.Top + f->Padding.Bottom);
 	if (step.x * f->Stride > image->w || step.y > image->h)
 	{
-		fprintf(stderr,
+		LOG(LM_GFX, LL_ERROR,
 			"Error: font image not big enough for font data "
-			"Image %dx%d Size %dx%d Stride %d Padding %d,%d,%d,%d\n",
+			"Image %dx%d Size %dx%d Stride %d Padding %d,%d,%d,%d",
 			image->w, image->h,
 			f->Size.x, f->Size.y, f->Stride,
 			f->Padding.Left, f->Padding.Top,
 			f->Padding.Right, f->Padding.Bottom);
-		goto bail;
+		return;
 	}
 
 	// Load letters from image file
@@ -108,7 +112,11 @@ void FontLoad(Font *f, const char *imgPath, const bool isProportional)
 				&p, f->Size,
 				svec2i_add(pos, svec2i(f->Padding.Left, f->Padding.Top)),
 				image);
-			if (isProportional)
+			if (chars == ' ')
+			{
+				PicShrink(&p, spaceSize, svec2i_zero());
+			}
+			else if (isProportional)
 			{
 				PicTrim(&p, true, false);
 			}
@@ -116,10 +124,6 @@ void FontLoad(Font *f, const char *imgPath, const bool isProportional)
 		}
 	}
 	SDL_UnlockSurface(image);
-
-bail:
-	SDL_FreeSurface(image);
-	rwops->close(rwops);
 }
 void FontTerminate(Font *f)
 {
@@ -138,6 +142,10 @@ int FontH(void)
 {
 	return gFont.Size.y + gFont.Gap.y;
 }
+struct vec2i FontChSize(const char c)
+{
+	return svec2i(FontW(c), FontH());
+}
 int FontStrW(const char *s)
 {
 	return FontSubstrW(s, (int)strlen(s));
@@ -208,14 +216,7 @@ struct vec2i FontCh(const char c, const struct vec2i pos)
 {
 	return FontChMask(c, pos, colorWhite);
 }
-static struct vec2i FontChColor(
-	const char c, const struct vec2i pos, const color_t color, const bool blend);
 struct vec2i FontChMask(const char c, const struct vec2i pos, const color_t mask)
-{
-	return FontChColor(c, pos, mask, false);
-}
-static struct vec2i FontChColor(
-	const char c, const struct vec2i pos, const color_t color, const bool blend)
 {
 	int idx = (int)c - FIRST_CHAR;
 	if (idx < 0)
@@ -228,14 +229,9 @@ static struct vec2i FontChColor(
 		idx = FIRST_CHAR;
 	}
 	const Pic *pic = CArrayGet(&gFont.Chars, idx);
-	if (blend)
-	{
-		BlitBlend(&gGraphicsDevice, pic, pos, color);
-	}
-	else
-	{
-		BlitMasked(&gGraphicsDevice, pic, pos, color, true);
-	}
+	PicRender(
+		pic, gGraphicsDevice.gameWindow.renderer, pos, mask, 0, svec2_one(),
+		SDL_FLIP_NONE, Rect2iZero());
 	// Add gap between characters
 	return svec2i(pos.x + pic->size.x + gFont.Gap.x, pos.y);
 }
@@ -243,14 +239,7 @@ struct vec2i FontStr(const char *s, struct vec2i pos)
 {
 	return FontStrMask(s, pos, colorWhite);
 }
-static struct vec2i FontStrColor(
-	const char *s, struct vec2i pos, const color_t c, const bool blend);
 struct vec2i FontStrMask(const char *s, struct vec2i pos, const color_t mask)
-{
-	return FontStrColor(s, pos, mask, false);
-}
-static struct vec2i FontStrColor(
-	const char *s, struct vec2i pos, const color_t c, const bool blend)
 {
 	if (s == NULL)
 	{
@@ -266,7 +255,7 @@ static struct vec2i FontStrColor(
 		}
 		else
 		{
-			pos = FontChColor(*s, pos, c, blend);
+			pos = FontChMask(*s, pos, mask);
 		}
 		s++;
 	}
