@@ -2,7 +2,7 @@
     C-Dogs SDL
     A port of the legendary (and fun) action/arcade cdogs.
 
-    Copyright (c) 2013-2017, Cong Xu
+    Copyright (c) 2013-2019, 2021 Cong Xu
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,7 @@
 #pragma once
 
 #include "c_array.h"
+#include "character.h"
 #include "particle.h"
 #include "proto/msg.pb.h"
 
@@ -48,8 +49,9 @@ typedef enum
 	GAME_EVENT_PLAYER_DATA,
 	GAME_EVENT_PLAYER_REMOVE,
 	GAME_EVENT_TILE_SET,
+
+	GAME_EVENT_THING_DAMAGE,
 	GAME_EVENT_MAP_OBJECT_ADD,
-	GAME_EVENT_MAP_OBJECT_DAMAGE,
 	GAME_EVENT_MAP_OBJECT_REMOVE,
 	GAME_EVENT_CLIENT_READY,
 	GAME_EVENT_NET_GAME_START,
@@ -75,11 +77,12 @@ typedef enum
 	GAME_EVENT_ACTOR_PICKUP_ALL,
 	GAME_EVENT_ACTOR_REPLACE_GUN,
 	GAME_EVENT_ACTOR_HEAL,
-	GAME_EVENT_ACTOR_HIT,
 	GAME_EVENT_ACTOR_ADD_AMMO,
 	GAME_EVENT_ACTOR_USE_AMMO,
 	GAME_EVENT_ACTOR_DIE,
+	GAME_EVENT_PLAYER_ADD_LIVES,
 	GAME_EVENT_ACTOR_MELEE,
+	GAME_EVENT_ACTOR_PILOT,
 
 	GAME_EVENT_ADD_PICKUP,
 	GAME_EVENT_REMOVE_PICKUP,
@@ -97,6 +100,7 @@ typedef enum
 	GAME_EVENT_RESCUE_CHARACTER,
 	GAME_EVENT_OBJECTIVE_UPDATE,
 	GAME_EVENT_ADD_KEYS,
+	GAME_EVENT_DOOR_TOGGLE,
 
 	// Can complete mission
 	GAME_EVENT_MISSION_COMPLETE,
@@ -120,7 +124,7 @@ typedef struct
 	bool Enqueue;
 	// Whether to broadcast these events only after game start
 	bool GameStart;
-	const pb_field_t *Fields;
+	const pb_msgdesc_t *Fields;
 } GameEventEntry;
 GameEventEntry GameEventGetEntry(const GameEventType e);
 
@@ -133,13 +137,18 @@ typedef struct
 		NPlayerData PlayerData;
 		NPlayerRemove PlayerRemove;
 		NTileSet TileSet;
+		NThingDamage ThingDamage;
 		NMapObjectAdd MapObjectAdd;
-		NMapObjectDamage MapObjectDamage;
 		NMapObjectRemove MapObjectRemove;
 		NConfig Config;
 		NScore Score;
 		NSound SoundAt;
-		int ShakeAmount;
+		struct
+		{
+			int Amount;
+			bool CameraSubjectOnly;
+			int ActorUID;
+		} Shake;
 		struct
 		{
 			char Message[256];
@@ -156,11 +165,12 @@ typedef struct
 		NActorPickupAll ActorPickupAll;
 		NActorReplaceGun ActorReplaceGun;
 		NActorHeal Heal;
-		NActorHit ActorHit;
 		NActorAddAmmo AddAmmo;
 		NActorUseAmmo UseAmmo;
 		NActorDie ActorDie;
+		NPlayerAddLives PlayerAddLives;
 		NActorMelee Melee;
+		NActorPilot Pilot;
 		NAddPickup AddPickup;
 		NRemovePickup RemovePickup;
 		struct
@@ -181,6 +191,7 @@ typedef struct
 		NRescueCharacter Rescue;
 		NObjectiveUpdate ObjectiveUpdate;
 		NAddKeys AddKeys;
+		NDoorToggle DoorToggle;
 		NMissionComplete MissionComplete;
 		NMissionEnd MissionEnd;
 	} u;
@@ -196,3 +207,4 @@ void GameEventsEnqueue(CArray *store, GameEvent e);
 void GameEventsClear(CArray *store);
 
 GameEvent GameEventNew(GameEventType type);
+GameEvent GameEventNewActorAdd(const struct vec2 pos, const Character *c, const bool isNPC);
