@@ -22,7 +22,7 @@
     This file incorporates work covered by the following copyright and
     permission notice:
 
-    Copyright (c) 2013-2017, Cong Xu
+    Copyright (c) 2013-2019, 2021 Cong Xu
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -50,6 +50,7 @@
 
 #include <json/json.h>
 #include "ammo.h"
+#include "character.h"
 #include "pic_manager.h"
 #include "pickup_class.h"
 
@@ -71,7 +72,8 @@ const char *PlacementFlagStr(const int i);
 typedef enum
 {
 	MAP_OBJECT_TYPE_NORMAL,
-	MAP_OBJECT_TYPE_PICKUP_SPAWNER
+	MAP_OBJECT_TYPE_PICKUP_SPAWNER,
+	MAP_OBJECT_TYPE_ACTOR_SPAWNER
 } MapObjectType;
 
 // Pickups to spawn when map objects are destroyed
@@ -87,7 +89,12 @@ typedef struct
 	char *Name;
 	CPic Pic;
 	struct vec2i Offset;
-	char *Wreck;
+	struct
+	{
+		char *MO;
+		Mix_Chunk *Sound;
+		char *Bullet;
+	} Wreck;
 	struct vec2i Size;
 	struct vec2 PosOffset;
 	int Health;
@@ -96,13 +103,24 @@ typedef struct
 	CArray DestroyGuns;	// of const WeaponClass *
 	// Bit field composed of bits shifted by PlacementFlags
 	int Flags;
-	bool DrawLast;
+	bool DrawBelow;
+	bool DrawAbove;
+	char *FootstepSound;
+	color_t FootprintMask;
 	MapObjectType Type;
 	union
 	{
 		const PickupClass *PickupClass;
+		struct
+		{
+			int CharId;
+			int Counter;
+		} Character;
 	} u;
 	CArray DestroySpawn;	// of MapObjectDestroySpawn
+	struct {
+		float HealthThreshold;	// Smoke if map object damaged below this ratio
+	} DamageSmoke;
 } MapObject;
 typedef struct
 {
@@ -123,7 +141,7 @@ MapObject *IntMapObject(const int m);
 MapObject *IndexMapObject(const int i);
 // Get index of destructible map object; used by editor
 int DestructibleMapObjectIndex(const MapObject *mo);
-MapObject *RandomBloodMapObject(const MapObjects *mo);
+const MapObject *GetRandomBloodPool(void);
 int MapObjectGetFlags(const MapObject *mo);
 
 void MapObjectsInit(
@@ -140,10 +158,6 @@ int MapObjectsCount(const MapObjects *classes);
 const Pic *MapObjectGetPic(const MapObject *mo, struct vec2i *offset);
 
 bool MapObjectIsTileOK(
-	const MapObject *obj, unsigned short tile, const bool isEmpty,
-	unsigned short tileAbove);
-bool MapObjectIsTileOKStrict(
-	const MapObject *obj, const unsigned short tile, const bool isEmpty,
-	const unsigned short tileAbove, const unsigned short tileBelow,
-	const int numWallsAdjacent, const int numWallsAround);
+	const MapObject *obj, const Tile *tile, const Tile *tileAbove);
 struct vec2 MapObjectGetPlacementPos(const MapObject *mo, const struct vec2i tilePos);
+bool MapObjectIsOnWall(const MapObject *mo);
