@@ -332,6 +332,8 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 		}
 	}
 
+	LoadInt(&wc->Price, node, "Price");
+
 	LOG(LM_MAP, LL_DEBUG, "loaded %s name(%s) lock(%d)...",
 		GunTypeStr(wc->Type), wc->name, wc->Lock);
 	LOG(LM_MAP, LL_DEBUG, "...canDrop(%s)", wc->CanDrop ? "true" : "false");
@@ -373,6 +375,8 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 		LOG(LM_MAP, LL_DEBUG, "...auto(%s)",
 			wc->u.Normal.Auto ? "true" : "false");
 	}
+
+	LOG(LM_MAP, LL_DEBUG, "price(%d)", wc->Price);
 }
 void WeaponClassesTerminate(WeaponClasses *wcs)
 {
@@ -626,6 +630,10 @@ bool WeaponClassCanShoot(const WeaponClass *wc)
 }
 int WeaponClassNumBarrels(const WeaponClass *wc)
 {
+	if (wc == NULL)
+	{
+		return 0;
+	}
 	return wc->Type == GUNTYPE_MULTI ? 2 : 1;
 }
 const WeaponClass *WeaponClassGetBarrel(
