@@ -1,29 +1,29 @@
 /*
-    C-Dogs SDL
-    A port of the legendary (and fun) action/arcade cdogs.
-    Copyright (c) 2013-2016, Cong Xu
-    All rights reserved.
+	C-Dogs SDL
+	A port of the legendary (and fun) action/arcade cdogs.
+	Copyright (c) 2013-2016, 2019-2020 Cong Xu
+	All rights reserved.
 
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions are met:
 
-    Redistributions of source code must retain the above copyright notice, this
-    list of conditions and the following disclaimer.
-    Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
+	Redistributions of source code must retain the above copyright notice, this
+	list of conditions and the following disclaimer.
+	Redistributions in binary form must reproduce the above copyright notice,
+	this list of conditions and the following disclaimer in the documentation
+	and/or other materials provided with the distribution.
 
-    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+	POSSIBILITY OF SUCH DAMAGE.
 */
 #include "editor_ui_objectives.h"
 
@@ -34,10 +34,9 @@
 #include "editor_ui.h"
 #include "editor_ui_common.h"
 
-
 typedef struct
 {
-	CampaignOptions *Campaign;
+	Campaign *Campaign;
 	int MissionObjectiveIndex;
 } MissionObjectiveData;
 static char *MissionGetObjectiveDescription(UIObject *o, void *data)
@@ -100,19 +99,31 @@ static char **MissionGetObjectiveDescriptionSrc(void *data)
 	}
 	return &((Objective *)CArrayGet(&m->Objectives, i))->Description;
 }
+static EditorResult MissionChangeObjectiveDescription(void *data, int d)
+{
+	// Dummy change func to return result file changed
+	UNUSED(data);
+	UNUSED(d);
+	return EDITOR_RESULT_CHANGED;
+}
 typedef struct
 {
-	CampaignOptions *co;
+	Campaign *co;
 	int index;
 } MissionIndexData;
 static const char *MissionGetObjectiveStr(UIObject *o, void *vData)
 {
 	UNUSED(o);
 	MissionIndexData *data = vData;
-	if (!CampaignGetCurrentMission(data->co)) return NULL;
-	if ((int)CampaignGetCurrentMission(data->co)->Objectives.size <= data->index) return NULL;
-	return ObjectiveTypeStr(((const Objective *)CArrayGet(
-		&CampaignGetCurrentMission(data->co)->Objectives, data->index))->Type);
+	if (!CampaignGetCurrentMission(data->co))
+		return NULL;
+	if ((int)CampaignGetCurrentMission(data->co)->Objectives.size <=
+		data->index)
+		return NULL;
+	return ObjectiveTypeStr(
+		((const Objective *)CArrayGet(
+			 &CampaignGetCurrentMission(data->co)->Objectives, data->index))
+			->Type);
 }
 static Objective *GetMissionObjective(const Mission *m, const int idx)
 {
@@ -123,14 +134,17 @@ static const char *MissionGetObjectiveRequired(UIObject *o, void *vData)
 	static char s[128];
 	UNUSED(o);
 	MissionIndexData *data = vData;
-	if (!CampaignGetCurrentMission(data->co)) return NULL;
+	if (!CampaignGetCurrentMission(data->co))
+		return NULL;
 	if ((int)CampaignGetCurrentMission(data->co)->Objectives.size <=
 		data->index)
 	{
 		return NULL;
 	}
-	sprintf(s, "%d", GetMissionObjective(
-		CampaignGetCurrentMission(data->co), data->index)->Required);
+	sprintf(
+		s, "%d",
+		GetMissionObjective(CampaignGetCurrentMission(data->co), data->index)
+			->Required);
 	return s;
 }
 static const char *MissionGetObjectiveTotal(UIObject *o, void *vData)
@@ -138,14 +152,17 @@ static const char *MissionGetObjectiveTotal(UIObject *o, void *vData)
 	static char s[128];
 	UNUSED(o);
 	MissionIndexData *data = vData;
-	if (!CampaignGetCurrentMission(data->co)) return NULL;
+	if (!CampaignGetCurrentMission(data->co))
+		return NULL;
 	if ((int)CampaignGetCurrentMission(data->co)->Objectives.size <=
 		data->index)
 	{
 		return NULL;
 	}
-	sprintf(s, "out of %d", GetMissionObjective(
-		CampaignGetCurrentMission(data->co), data->index)->Count);
+	sprintf(
+		s, "out of %d",
+		GetMissionObjective(CampaignGetCurrentMission(data->co), data->index)
+			->Count);
 	return s;
 }
 static const char *MissionGetObjectiveFlags(UIObject *o, void *vData)
@@ -154,20 +171,22 @@ static const char *MissionGetObjectiveFlags(UIObject *o, void *vData)
 	static char s[128];
 	UNUSED(o);
 	MissionIndexData *data = vData;
-	if (!CampaignGetCurrentMission(data->co)) return NULL;
+	if (!CampaignGetCurrentMission(data->co))
+		return NULL;
 	if ((int)CampaignGetCurrentMission(data->co)->Objectives.size <=
 		data->index)
 	{
 		return NULL;
 	}
-	flags = GetMissionObjective(
-		CampaignGetCurrentMission(data->co), data->index)->Flags;
+	flags =
+		GetMissionObjective(CampaignGetCurrentMission(data->co), data->index)
+			->Flags;
 	if (!flags)
 	{
 		return "(normal)";
 	}
-	sprintf(s, "%s %s %s %s %s",
-		(flags & OBJECTIVE_HIDDEN) ? "hidden" : "",
+	sprintf(
+		s, "%s %s %s %s %s", (flags & OBJECTIVE_HIDDEN) ? "hidden" : "",
 		(flags & OBJECTIVE_POSKNOWN) ? "pos.known" : "",
 		(flags & OBJECTIVE_HIACCESS) ? "access" : "",
 		(flags & OBJECTIVE_UNKNOWNCOUNT) ? "no-count" : "",
@@ -177,12 +196,12 @@ static const char *MissionGetObjectiveFlags(UIObject *o, void *vData)
 
 typedef struct
 {
-	CampaignOptions *C;
+	Campaign *C;
 	int ObjectiveIdx;
 	ObjectiveType Type;
 } ObjectiveChangeTypeData;
 static void MissionResetObjectiveIndex(Objective *o);
-static void ObjectiveChangeType(void *vData, int d)
+static EditorResult ObjectiveChangeType(void *vData, int d)
 {
 	UNUSED(d);
 	ObjectiveChangeTypeData *data = vData;
@@ -190,10 +209,11 @@ static void ObjectiveChangeType(void *vData, int d)
 		CampaignGetCurrentMission(data->C), data->ObjectiveIdx);
 	if (o->Type == data->Type)
 	{
-		return;
+		return EDITOR_RESULT_NONE;
 	}
 	o->Type = data->Type;
 	MissionResetObjectiveIndex(o);
+	return EDITOR_RESULT_CHANGED;
 }
 static void MissionResetObjectiveIndex(Objective *o)
 {
@@ -202,12 +222,11 @@ static void MissionResetObjectiveIndex(Objective *o)
 	case OBJECTIVE_COLLECT:
 		o->u.Pickup = IntScorePickupClass(0);
 		break;
-	case OBJECTIVE_DESTROY:
-		{
-			const char **destructibleName =
-				CArrayGet(&gMapObjects.Destructibles, 0);
-			o->u.MapObject = StrMapObject(*destructibleName);
-		}
+	case OBJECTIVE_DESTROY: {
+		const char **destructibleName =
+			CArrayGet(&gMapObjects.Destructibles, 0);
+		o->u.MapObject = StrMapObject(*destructibleName);
+	}
 		CASSERT(o->u.MapObject != NULL, "cannot find map object");
 		break;
 	default:
@@ -215,14 +234,15 @@ static void MissionResetObjectiveIndex(Objective *o)
 		break;
 	}
 }
-static void MissionChangeObjectiveRequired(void *vData, int d)
+static EditorResult MissionChangeObjectiveRequired(void *vData, int d)
 {
 	MissionIndexData *data = vData;
-	Objective *o = GetMissionObjective(
-		CampaignGetCurrentMission(data->co), data->index);
+	Objective *o =
+		GetMissionObjective(CampaignGetCurrentMission(data->co), data->index);
 	o->Required = CLAMP_OPPOSITE(o->Required + d, 0, MIN(100, o->Count));
+	return EDITOR_RESULT_CHANGED;
 }
-static void MissionChangeObjectiveTotal(void *vData, int d)
+static EditorResult MissionChangeObjectiveTotal(void *vData, int d)
 {
 	MissionIndexData *data = vData;
 	const Mission *m = CampaignGetCurrentMission(data->co);
@@ -232,27 +252,28 @@ static void MissionChangeObjectiveTotal(void *vData, int d)
 	if (m->Type == MAPTYPE_STATIC)
 	{
 		CA_FOREACH(const ObjectivePositions, op, m->u.Static.Objectives)
-			if (op->Index == data->index)
-			{
-				o->Count = MAX(o->Count, (int)op->Positions.size);
-				break;
-			}
+		if (op->Index == data->index)
+		{
+			o->Count = MAX(o->Count, (int)op->PositionIndices.size);
+			break;
+		}
 		CA_FOREACH_END()
 	}
+	return EDITOR_RESULT_CHANGED;
 }
-static void MissionChangeObjectiveFlags(void *vData, int d)
+static EditorResult MissionChangeObjectiveFlags(void *vData, int d)
 {
 	MissionIndexData *data = vData;
-	Objective *o = GetMissionObjective(
-		CampaignGetCurrentMission(data->co), data->index);
+	Objective *o =
+		GetMissionObjective(CampaignGetCurrentMission(data->co), data->index);
 	// Max is combination of all flags, i.e. largest flag doubled less one
 	o->Flags = CLAMP_OPPOSITE(o->Flags + d, 0, OBJECTIVE_NOACCESS * 2 - 1);
+	return EDITOR_RESULT_CHANGED;
 }
 
-
 static UIObject *CreateObjectiveObjs(
-	struct vec2i pos, CampaignOptions *co, const int idx);
-void CreateObjectivesObjs(CampaignOptions *co, UIObject *c, struct vec2i pos)
+	struct vec2i pos, Campaign *co, const int idx);
+void CreateObjectivesObjs(Campaign *co, UIObject *c, struct vec2i pos)
 {
 	const int th = FontH();
 	struct vec2i objectivesPos = svec2i(0, 7 * th);
@@ -267,7 +288,8 @@ void CreateObjectivesObjs(CampaignOptions *co, UIObject *c, struct vec2i pos)
 		o2->Type = UITYPE_TEXTBOX;
 		o2->u.Textbox.TextLinkFunc = MissionGetObjectiveDescription;
 		o2->u.Textbox.TextSourceFunc = MissionGetObjectiveDescriptionSrc;
-		o2->IsDynamicData = 1;
+		o2->IsDynamicData = true;
+		o2->ChangeFunc = MissionChangeObjectiveDescription;
 		CMALLOC(o2->Data, sizeof(MissionObjectiveData));
 		((MissionObjectiveData *)o2->Data)->Campaign = co;
 		((MissionObjectiveData *)o2->Data)->MissionObjectiveIndex = i;
@@ -275,7 +297,7 @@ void CreateObjectivesObjs(CampaignOptions *co, UIObject *c, struct vec2i pos)
 		o2->Pos = pos;
 		CSTRDUP(
 			o2->Tooltip,
-			"Insert/CTRL-i, Delete/CTRL+d: add/remove objective");
+			"Insert/" KMOD_CMD_NAME "+i, Delete/" KMOD_CMD_NAME "+d: add/remove objective");
 		o2->CheckVisible = MissionCheckObjectiveDescription;
 		UIObjectAddChild(o2, CreateObjectiveObjs(objectivesPos, co, i));
 		UIObjectAddChild(c, o2);
@@ -284,9 +306,9 @@ void CreateObjectivesObjs(CampaignOptions *co, UIObject *c, struct vec2i pos)
 	UIObjectDestroy(o);
 }
 static void CreateObjectiveItemObjs(
-	UIObject *c, const struct vec2i pos, CampaignOptions *co, const int idx);
+	UIObject *c, const struct vec2i pos, Campaign *co, const int idx);
 static UIObject *CreateObjectiveObjs(
-	struct vec2i pos, CampaignOptions *co, const int idx)
+	struct vec2i pos, Campaign *co, const int idx)
 {
 	const int th = FontH();
 	UIObject *c;
@@ -297,7 +319,6 @@ static UIObject *CreateObjectiveObjs(
 
 	o = UIObjectCreate(UITYPE_LABEL, 0, svec2i_zero(), svec2i_zero());
 	o->Flags = UI_LEAVE_YC;
-	o->ChangesData = 1;
 
 	pos.y -= idx * th;
 	// Drop-down menu for objective type
@@ -316,7 +337,6 @@ static UIObject *CreateObjectiveObjs(
 	{
 		UIObject *oObjTypeChild =
 			UIObjectCreate(UITYPE_LABEL, 0, svec2i_zero(), svec2i(50, th));
-		oObjTypeChild->ChangesData = true;
 		oObjTypeChild->Pos.y = i * th;
 		UIObjectSetDynamicLabel(
 			oObjTypeChild, ObjectiveTypeStr((ObjectiveType)i));
@@ -369,12 +389,12 @@ static UIObject *CreateObjectiveObjs(
 	((MissionIndexData *)o2->Data)->index = idx;
 	o2->Pos = pos;
 	o2->Size = svec2i(100, th);
-	CSTRDUP(o2->Tooltip,
-		"hidden: not shown on map\n"
-		"pos.known: always shown on map\n"
-		"access: in locked room\n"
-		"no-count: don't show completed count\n"
-		"no-access: not in locked rooms");
+	CSTRDUP(
+		o2->Tooltip, "hidden: not shown on map\n"
+					 "pos.known: always shown on map\n"
+					 "access: in locked room\n"
+					 "no-count: don't show completed count\n"
+					 "no-access: not in locked rooms");
 	UIObjectAddChild(c, o2);
 
 	UIObjectDestroy(o);
@@ -383,7 +403,7 @@ static UIObject *CreateObjectiveObjs(
 
 typedef struct
 {
-	CampaignOptions *C;
+	Campaign *C;
 	int ObjIdx;
 	MapObject *M;
 } DestroyObjectiveData;
@@ -395,20 +415,19 @@ static void MissionDrawDestroyObjective(
 	UIObject *o, GraphicsDevice *g, struct vec2i pos, void *vData);
 static void MissionDrawRescueObjective(
 	UIObject *o, GraphicsDevice *g, struct vec2i pos, void *vData);
-static void MissionChangeCollectObjectiveIndex(void *vData, int d);
-static void MissionChangeRescueObjectiveIndex(void *vData, int d);
+static EditorResult MissionChangeCollectObjectiveIndex(void *vData, int d);
+static EditorResult MissionChangeRescueObjectiveIndex(void *vData, int d);
 static void MissionCheckObjectiveIsKill(UIObject *o, void *vData);
 static void MissionCheckObjectiveIsCollect(UIObject *o, void *vData);
 static void MissionCheckObjectiveIsDestroy(UIObject *o, void *vData);
 static void MissionCheckObjectiveIsRescue(UIObject *o, void *vData);
 static bool ObjectiveDestroyObjFunc(UIObject *o, MapObject *mo, void *data);
 static void CreateObjectiveItemObjs(
-	UIObject *c, const struct vec2i pos, CampaignOptions *co, const int idx)
+	UIObject *c, const struct vec2i pos, Campaign *co, const int idx)
 {
 	// TODO: context menu
 	UIObject *o = UIObjectCreate(UITYPE_CUSTOM, 0, pos, svec2i(30, 20));
 	o->Flags = UI_LEAVE_YC;
-	o->ChangesData = true;
 	UIObject *o2;
 
 	o2 = UIObjectCopy(o);
@@ -440,9 +459,10 @@ static void CreateObjectiveItemObjs(
 	((MissionIndexData *)o2->Data)->index = idx;
 	o2->CheckVisible = MissionCheckObjectiveIsDestroy;
 	CSTRDUP(o2->Tooltip, "Choose object to destroy");
-	UIObjectAddChild(o2, CreateAddMapItemObjs(
-		o2->Size, ObjectiveDestroyObjFunc, o2->Data,
-		sizeof(DestroyObjectiveData), true));
+	UIObjectAddChild(
+		o2, CreateAddMapItemObjs(
+				o2->Size, ObjectiveDestroyObjFunc, o2->Data,
+				sizeof(DestroyObjectiveData), true));
 	UIObjectAddChild(c, o2);
 
 	o2 = UIObjectCopy(o);
@@ -463,17 +483,19 @@ static void MissionDrawKillObjective(
 	MissionIndexData *data = vData;
 	UNUSED(g);
 	const Mission *m = CampaignGetCurrentMission(data->co);
-	if (m == NULL) return;
-	if ((int)m->Objectives.size <= data->index) return;
+	if (m == NULL)
+		return;
+	if ((int)m->Objectives.size <= data->index)
+		return;
 	// TODO: only one kill and rescue objective allowed
 	CharacterStore *store = &data->co->Setting.characters;
 	if (store->specialIds.size > 0)
 	{
-		Character *c = CArrayGet(
+		const Character *c = CArrayGet(
 			&store->OtherChars, CharacterStoreGetSpecialId(store, 0));
-		const struct vec2i drawPos =
-			svec2i_add(svec2i_add(pos, o->Pos), svec2i_scale_divide(o->Size, 2));
-		DrawCharacterSimple(c, drawPos, DIRECTION_DOWN, false, true);
+		const struct vec2i drawPos = svec2i_add(
+			svec2i_add(pos, o->Pos), svec2i_scale_divide(o->Size, 2));
+		DrawCharacterSimple(c, drawPos, DIRECTION_DOWN, false, true, c->Gun);
 	}
 }
 static void MissionDrawCollectObjective(
@@ -482,14 +504,19 @@ static void MissionDrawCollectObjective(
 	MissionIndexData *data = vData;
 	UNUSED(g);
 	const Mission *m = CampaignGetCurrentMission(data->co);
-	if (m == NULL) return;
-	if ((int)m->Objectives.size <= data->index) return;
+	if (m == NULL)
+		return;
+	if ((int)m->Objectives.size <= data->index)
+		return;
 	// TODO: only one kill and rescue objective allowed
 	const Objective *obj = CArrayGet(&m->Objectives, data->index);
 	const Pic *newPic = CPicGetPic(&obj->u.Pickup->Pic, 0);
 	const struct vec2i drawPos =
 		svec2i_add(svec2i_add(pos, o->Pos), svec2i_scale_divide(o->Size, 2));
-	Blit(g, newPic, svec2i_subtract(drawPos, svec2i_scale_divide(newPic->size, 2)));
+	PicRender(
+		newPic, g->gameWindow.renderer,
+		svec2i_subtract(drawPos, svec2i_scale_divide(newPic->size, 2)),
+		colorWhite, 0, svec2_one(), SDL_FLIP_NONE, Rect2iZero());
 }
 static void MissionDrawDestroyObjective(
 	UIObject *o, GraphicsDevice *g, struct vec2i pos, void *vData)
@@ -497,15 +524,19 @@ static void MissionDrawDestroyObjective(
 	MissionIndexData *data = vData;
 	UNUSED(g);
 	const Mission *m = CampaignGetCurrentMission(data->co);
-	if (m == NULL) return;
-	if ((int)m->Objectives.size <= data->index) return;
+	if (m == NULL)
+		return;
+	if ((int)m->Objectives.size <= data->index)
+		return;
 	// TODO: only one kill and rescue objective allowed
 	const Objective *obj = CArrayGet(&m->Objectives, data->index);
 	struct vec2i offset;
 	const Pic *newPic = MapObjectGetPic(obj->u.MapObject, &offset);
 	const struct vec2i drawPos =
 		svec2i_add(svec2i_add(pos, o->Pos), svec2i_scale_divide(o->Size, 2));
-	Blit(g, newPic, svec2i_add(drawPos, offset));
+	PicRender(
+		newPic, g->gameWindow.renderer, svec2i_add(drawPos, offset),
+		colorWhite, 0, svec2_one(), SDL_FLIP_NONE, Rect2iZero());
 }
 static void MissionDrawRescueObjective(
 	UIObject *o, GraphicsDevice *g, struct vec2i pos, void *vData)
@@ -513,55 +544,62 @@ static void MissionDrawRescueObjective(
 	MissionIndexData *data = vData;
 	UNUSED(g);
 	const Mission *m = CampaignGetCurrentMission(data->co);
-	if (m == NULL) return;
-	if ((int)m->Objectives.size <= data->index) return;
+	if (m == NULL)
+		return;
+	if ((int)m->Objectives.size <= data->index)
+		return;
 	// TODO: only one kill and rescue objective allowed
 	CharacterStore *store = &data->co->Setting.characters;
 	if (store->prisonerIds.size > 0)
 	{
-		Character *c = CArrayGet(
+		const Character *c = CArrayGet(
 			&store->OtherChars, CharacterStoreGetPrisonerId(store, 0));
-		const struct vec2i drawPos =
-			svec2i_add(svec2i_add(pos, o->Pos), svec2i_scale_divide(o->Size, 2));
-		DrawCharacterSimple(c, drawPos, DIRECTION_DOWN, false, true);
+		const struct vec2i drawPos = svec2i_add(
+			svec2i_add(pos, o->Pos), svec2i_scale_divide(o->Size, 2));
+		DrawCharacterSimple(c, drawPos, DIRECTION_DOWN, false, true, c->Gun);
 	}
 }
 
-static void MissionChangeCollectObjectiveIndex(void *vData, int d)
+static EditorResult MissionChangeCollectObjectiveIndex(void *vData, int d)
 {
 	MissionIndexData *data = vData;
-	Objective *o = GetMissionObjective(
-		CampaignGetCurrentMission(data->co), data->index);
+	Objective *o =
+		GetMissionObjective(CampaignGetCurrentMission(data->co), data->index);
 	int idx = PickupClassesGetScoreIdx(o->u.Pickup);
 	const int limit = PickupClassesGetScoreCount(&gPickupClasses) - 1;
 	idx = CLAMP_OPPOSITE(idx + d, 0, limit);
 	o->u.Pickup = IntScorePickupClass(idx);
+	return EDITOR_RESULT_CHANGED;
 }
-static void MissionSetDestroyObjective(void *vData, int d)
+static EditorResult MissionSetDestroyObjective(void *vData, int d)
 {
 	UNUSED(d);
 	DestroyObjectiveData *data = vData;
-	Objective *o = GetMissionObjective(
-		CampaignGetCurrentMission(data->C), data->ObjIdx);
+	Objective *o =
+		GetMissionObjective(CampaignGetCurrentMission(data->C), data->ObjIdx);
 	o->u.MapObject = data->M;
+	return EDITOR_RESULT_CHANGED;
 }
-static void MissionChangeRescueObjectiveIndex(void *vData, int d)
+static EditorResult MissionChangeRescueObjectiveIndex(void *vData, int d)
 {
 	MissionIndexData *data = vData;
-	Objective *o = GetMissionObjective(
-		CampaignGetCurrentMission(data->co), data->index);
+	Objective *o =
+		GetMissionObjective(CampaignGetCurrentMission(data->co), data->index);
 	int idx = o->u.Index;
-	const int limit = data->co->Setting.characters.OtherChars.size - 1;
+	const int limit = (int)(data->co->Setting.characters.OtherChars.size - 1);
 	idx = CLAMP_OPPOSITE(idx + d, 0, limit);
 	o->u.Index = idx;
+	return EDITOR_RESULT_CHANGED;
 }
 
 static void MissionCheckObjectiveIsKill(UIObject *o, void *vData)
 {
 	MissionIndexData *data = vData;
 	const Mission *m = CampaignGetCurrentMission(data->co);
-	if (m == NULL) return;
-	if ((int)m->Objectives.size <= data->index) return;
+	if (m == NULL)
+		return;
+	if ((int)m->Objectives.size <= data->index)
+		return;
 	const Objective *obj = CArrayGet(&m->Objectives, data->index);
 	o->IsVisible = obj->Type == OBJECTIVE_KILL;
 }
@@ -569,8 +607,10 @@ static void MissionCheckObjectiveIsCollect(UIObject *o, void *vData)
 {
 	MissionIndexData *data = vData;
 	const Mission *m = CampaignGetCurrentMission(data->co);
-	if (m == NULL) return;
-	if ((int)m->Objectives.size <= data->index) return;
+	if (m == NULL)
+		return;
+	if ((int)m->Objectives.size <= data->index)
+		return;
 	const Objective *obj = CArrayGet(&m->Objectives, data->index);
 	o->IsVisible = obj->Type == OBJECTIVE_COLLECT;
 }
@@ -578,8 +618,10 @@ static void MissionCheckObjectiveIsDestroy(UIObject *o, void *vData)
 {
 	MissionIndexData *data = vData;
 	const Mission *m = CampaignGetCurrentMission(data->co);
-	if (m == NULL) return;
-	if ((int)m->Objectives.size <= data->index) return;
+	if (m == NULL)
+		return;
+	if ((int)m->Objectives.size <= data->index)
+		return;
 	const Objective *obj = CArrayGet(&m->Objectives, data->index);
 	o->IsVisible = obj->Type == OBJECTIVE_DESTROY;
 }
@@ -587,8 +629,10 @@ static void MissionCheckObjectiveIsRescue(UIObject *o, void *vData)
 {
 	MissionIndexData *data = vData;
 	const Mission *m = CampaignGetCurrentMission(data->co);
-	if (m == NULL) return;
-	if ((int)m->Objectives.size <= data->index) return;
+	if (m == NULL)
+		return;
+	if ((int)m->Objectives.size <= data->index)
+		return;
 	const Objective *obj = CArrayGet(&m->Objectives, data->index);
 	o->IsVisible = obj->Type == OBJECTIVE_RESCUE;
 }
@@ -615,5 +659,6 @@ static void DrawMapItem(
 	UNUSED(g);
 	const DestroyObjectiveData *data = vData;
 	DisplayMapItem(
-		svec2i_add(svec2i_add(pos, o->Pos), svec2i_scale_divide(o->Size, 2)), data->M);
+		svec2i_add(svec2i_add(pos, o->Pos), svec2i_scale_divide(o->Size, 2)),
+		data->M);
 }
