@@ -1,60 +1,74 @@
 /*
-    Copyright (c) 2013-2018 Cong Xu
-    All rights reserved.
+	Copyright (c) 2013-2022 Cong Xu
+	All rights reserved.
 
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions are met:
 
-    Redistributions of source code must retain the above copyright notice, this
-    list of conditions and the following disclaimer.
-    Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
+	Redistributions of source code must retain the above copyright notice, this
+	list of conditions and the following disclaimer.
+	Redistributions in binary form must reproduce the above copyright notice,
+	this list of conditions and the following disclaimer in the documentation
+	and/or other materials provided with the distribution.
 
-    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+	POSSIBILITY OF SUCH DAMAGE.
 */
 #include "mainmenu.h"
 
 #include <stdio.h>
 #include <string.h>
 
+#include <cdogs/actor_placement.h>
+#include <cdogs/ai.h>
 #include <cdogs/config.h>
+#include <cdogs/config_io.h>
+#include <cdogs/files.h>
 #include <cdogs/font.h>
 #include <cdogs/grafx_bg.h>
+#include <cdogs/handle_game_events.h>
 #include <cdogs/log.h>
+#include <cdogs/los.h>
+#include <cdogs/map_build.h>
+#include <cdogs/music.h>
 #include <cdogs/net_client.h>
 #include <cdogs/net_server.h>
+#include <cdogs/quick_play.h>
 
 #include "autosave.h"
 #include "briefing_screens.h"
+#include "game.h"
+#include "loading_screens.h"
 #include "menu.h"
 #include "prep.h"
 
-
 typedef struct
 {
 	MenuSystem ms;
 	GraphicsDevice *graphics;
 	credits_displayer_t creditsDisplayer;
-	custom_campaigns_t campaigns;
+	CustomCampaigns campaigns;
 	GameMode lastGameMode;
 	bool wasClient;
+	DrawBuffer buffer;
+	HSV bgTint;
+	RunGameData rData;
+	GameMode gameMode;
+	const CampaignEntry *entry;
 } MainMenuData;
+static void MenuResetSize(MenuSystem *ms);
 static void MenuCreateAll(
-	MenuSystem *ms, LoopRunner *l,
-	custom_campaigns_t *campaigns,
-	EventHandlers *handlers,
-	GraphicsDevice *graphics);
+	MainMenuData *data, LoopRunner *l, EventHandlers *handlers);
+static void MainMenuReset(MainMenuData *data);
 static void MainMenuTerminate(GameLoopData *data);
 static void MainMenuOnEnter(GameLoopData *data);
 static void MainMenuOnExit(GameLoopData *data);
@@ -71,17 +85,53 @@ GameLoopData *MainMenu(GraphicsDevice *graphics, LoopRunner *l)
 	LoadAllCampaigns(&data->campaigns);
 	data->lastGameMode = GAME_MODE_QUICK_PLAY;
 	data->wasClient = false;
-	MenuCreateAll(
-		&data->ms, l, &data->campaigns, &gEventHandlers, data->graphics);
+	MenuCreateAll(data, l, &gEventHandlers);
 	MenuSetCreditsDisplayer(&data->ms, &data->creditsDisplayer);
+
 	return GameLoopDataNew(
-		data, MainMenuTerminate, MainMenuOnEnter, MainMenuOnExit,
-		NULL, MainMenuUpdate, MainMenuDraw);
+		data, MainMenuTerminate, MainMenuOnEnter, MainMenuOnExit, NULL,
+		MainMenuUpdate, MainMenuDraw);
 }
-static void MainMenuReset(MenuSystem *ms)
+static void GenerateLiveBackground(MainMenuData *data)
 {
-	GrafxMakeRandomBackground(ms->graphics, &gCampaign, &gMission, &gMap);
-	MenuResetSize(ms);
+	MissionOptionsTerminate(&gMission);
+	CampaignTerminate(&gCampaign);
+
+	CampaignSettingInit(&gCampaign.Setting);
+	SetupQuickPlayCampaign(&gCampaign.Setting, true);
+	CampaignAndMissionSetup(&gCampaign, &gMission);
+	GameEventsInit(&gGameEvents);
+	gCampaign.MissionIndex = 0;
+
+	MapBuild(&gMap, gMission.missionData, true, gMission.index, GAME_MODE_NORMAL, &gCampaign.Setting.characters);
+
+	// Add AI player
+	GameEvent e = GameEventNew(GAME_EVENT_PLAYER_DATA);
+	e.u.PlayerData = PlayerDataDefault(0);
+	e.u.PlayerData.UID = gNetClient.FirstPlayerUID;
+	GameEventsEnqueue(&gGameEvents, e);
+	HandleGameEvents(&gGameEvents, NULL, NULL, NULL, NULL);
+	CA_FOREACH(PlayerData, p, gPlayerDatas)
+	p->inputDevice = INPUT_DEVICE_AI;
+	PlacePlayer(&gMap, p, svec2_zero(), true);
+	CA_FOREACH_END()
+
+	const HSV tint = {rand() * 360.0 / RAND_MAX, rand() * 1.0 / RAND_MAX, 0.5};
+	data->bgTint = tint;
+	DrawBufferInit(&data->buffer, svec2i(X_TILES, Y_TILES), data->graphics);
+	InitializeBadGuys();
+	CreateEnemies();
+	MapMarkAllAsVisited(&gMap);
+
+	GameInit(&data->rData, &gCampaign, &gMission, &gMap);
+}
+static void MainMenuReset(MainMenuData *data)
+{
+	GenerateLiveBackground(data);
+
+	MenuResetSize(&data->ms);
+	
+	data->entry = NULL;
 }
 static void MainMenuTerminate(GameLoopData *data)
 {
@@ -103,13 +153,15 @@ static void MainMenuOnEnter(GameLoopData *data)
 		return;
 	}
 
-	MainMenuReset(&mData->ms);
+	MusicPlayGeneral(&gSoundDevice.music, MUSIC_MENU);
+	// Reset config - could have been set to other values by server
+	ConfigResetChanged(&gConfig);
+	CampaignSettingTerminateAll(&gCampaign.Setting);
+
+	MainMenuReset(mData);
 	NetClientDisconnect(&gNetClient);
 	NetServerClose(&gNetServer);
 	GameEventsTerminate(&gGameEvents);
-	// Reset config - could have been set to other values by server
-	ConfigResetChanged(&gConfig);
-	CampaignSettingTerminate(&gCampaign.Setting);
 
 	// Auto-enter the submenu corresponding to the last game mode
 	menu_t *startMenu = FindSubmenuByName(mData->ms.root, "Start");
@@ -140,7 +192,8 @@ static menu_t *FindSubmenuByName(menu_t *menu, const char *name)
 {
 	CASSERT(menu->type == MENU_TYPE_NORMAL, "invalid menu type");
 	CA_FOREACH(menu_t, submenu, menu->u.normal.subMenus)
-		if (strcmp(submenu->name, name) == 0) return submenu;
+	if (strcmp(submenu->name, name) == 0)
+		return submenu;
 	CA_FOREACH_END()
 	return menu;
 }
@@ -164,65 +217,71 @@ static GameLoopResult MainMenuUpdate(GameLoopData *data, LoopRunner *l)
 	if (gCampaign.IsLoaded)
 	{
 		// Loaded game already; skip menus and go straight to game
-		LoopRunnerPush(l, ScreenCampaignIntro(&gCampaign.Setting));
+		LoopRunnerPush(l, ScreenCampaignIntro(&gCampaign.Setting, gCampaign.Entry.Mode, &gCampaign.Entry));
 		return UPDATE_RESULT_OK;
 	}
 
+	LOSSetAllVisible(&mData->rData.map->LOS);
+	GameUpdate(&mData->rData, 1, NULL);
+
 	const GameLoopResult result = MenuUpdate(&mData->ms);
 	if (result == UPDATE_RESULT_OK)
 	{
-		if (gCampaign.IsLoaded)
+		if (mData->entry)
 		{
-			LoopRunnerPush(l, ScreenCampaignIntro(&gCampaign.Setting));
+			LoopRunnerPush(l, ScreenLoading("Loading game...", true, ScreenCampaignIntro(&gCampaign.Setting, mData->gameMode, mData->entry), false));
 		}
 		else
 		{
-			LoopRunnerPop(l);
+			LoopRunnerPush(l, ScreenLoading("Quitting...", true, NULL, true));
 		}
 	}
 	if (gEventHandlers.HasResolutionChanged)
 	{
-		MainMenuReset(&mData->ms);
+		MainMenuReset(mData);
 	}
 	return result;
 }
 static void MainMenuDraw(GameLoopData *data)
 {
+	if (gCampaign.IsLoaded)
+	{
+		// Loaded game already; skip menus and go straight to game
+		return;
+	}
 	MainMenuData *mData = data->Data;
 	MenuDraw(&mData->ms);
+	const struct vec2 pos =
+		Vec2CenterOfTile(svec2i_scale_divide(gMap.Size, 2));
+	DrawBufferArgs args;
+	memset(&args, 0, sizeof args);
+	GrafxDrawBackground(
+		mData->graphics, &mData->buffer, mData->bgTint, pos, &args);
 }
 
 static menu_t *MenuCreateStart(
-	const char *name, MenuSystem *ms, LoopRunner *l,
-	custom_campaigns_t *campaigns);
-static menu_t *MenuCreateOptions(const char *name, MenuSystem *ms);
+	const char *name, MainMenuData *mainMenu, LoopRunner *l);
+static menu_t *MenuCreateOptions(const char *name, MainMenuData *data);
 menu_t *MenuCreateQuit(const char *name);
 
 static void MenuCreateAll(
-	MenuSystem *ms, LoopRunner *l,
-	custom_campaigns_t *campaigns,
-	EventHandlers *handlers,
-	GraphicsDevice *graphics)
+	MainMenuData *data, LoopRunner *l, EventHandlers *handlers)
 {
 	MenuSystemInit(
-		ms,
-		handlers, graphics,
-		svec2i_zero(),
-		svec2i(
-			graphics->cachedConfig.Res.x,
-			graphics->cachedConfig.Res.y));
-	ms->root = ms->current = MenuCreateNormal(
-		"",
-		"",
-		MENU_TYPE_NORMAL,
+		&data->ms, handlers, data->graphics, svec2i_zero(),
+		data->graphics->cachedConfig.Res);
+	data->ms.root = data->ms.current = MenuCreateNormal(
+		"", "", MENU_TYPE_NORMAL,
 		MENU_DISPLAY_ITEMS_CREDITS | MENU_DISPLAY_ITEMS_AUTHORS);
-	MenuAddSubmenu(ms->root, MenuCreateStart("Start", ms, l, campaigns));
-	MenuAddSubmenu(ms->root, MenuCreateOptions("Options...", ms));
+	MenuAddSubmenu(
+		data->ms.root,
+		MenuCreateStart("Start", data, l));
+	MenuAddSubmenu(data->ms.root, MenuCreateOptions("Options...", data));
 #ifndef __EMSCRIPTEN__
-	MenuAddSubmenu(ms->root, MenuCreateQuit("Quit"));
-	MenuAddExitType(ms, MENU_TYPE_QUIT);
+	MenuAddSubmenu(data->ms.root, MenuCreateQuit("Quit"));
+	MenuAddExitType(&data->ms, MENU_TYPE_QUIT);
 #endif
-	MenuAddExitType(ms, MENU_TYPE_RETURN);
+	MenuAddExitType(&data->ms, MENU_TYPE_RETURN);
 }
 
 typedef struct
@@ -231,57 +290,45 @@ typedef struct
 	// so we can enable it if LAN servers are found
 	int MenuJoinIndex;
 } CheckLANServerData;
-static menu_t *MenuCreateContinue(const char *name, CampaignEntry *entry);
-static menu_t *MenuCreateQuickPlay(const char *name, CampaignEntry *entry);
+static menu_t *MenuCreateContinue(const char *name, MainMenuData *mainMenu, const CampaignEntry *entry);
+static menu_t *MenuCreateQuickPlay(const char *name, MainMenuData *mainMenu);
 static menu_t *MenuCreateCampaigns(
-	const char *name, const char *title,
-	campaign_list_t *list, const GameMode mode);
+	const char *name, const char *title, MainMenuData *mainMenu, CampaignList *list,
+	const GameMode mode);
 static menu_t *CreateJoinLANGame(
 	const char *name, const char *title, MenuSystem *ms, LoopRunner *l);
 static void CheckLANServers(menu_t *menu, void *data);
 static menu_t *MenuCreateStart(
-	const char *name, MenuSystem *ms, LoopRunner *l,
-	custom_campaigns_t *campaigns)
+	const char *name, MainMenuData *mainMenu, LoopRunner *l)
 {
 	menu_t *menu = MenuCreateNormal(name, "Start:", MENU_TYPE_NORMAL, 0);
+	const CampaignSave *cs = AutosaveGetLastCampaign(&gAutosave);
 	MenuAddSubmenu(
-		menu,
-		MenuCreateContinue("Continue", &gAutosave.LastMission.Campaign));
+		menu, MenuCreateContinue("Continue", mainMenu, &cs->Campaign));
 	const int menuContinueIndex = (int)menu->u.normal.subMenus.size - 1;
 	MenuAddSubmenu(
-		menu,
-		MenuCreateQuickPlay("Quick Play", &campaigns->quickPlayEntry));
+		menu, MenuCreateCampaigns(
+				  "Campaign", "Select a campaign:", mainMenu, &mainMenu->campaigns.campaignList,
+				  GAME_MODE_NORMAL));
 	MenuAddSubmenu(
-		menu,
-		MenuCreateCampaigns(
-		"Campaign",
-		"Select a campaign:",
-		&campaigns->campaignList,
-		GAME_MODE_NORMAL));
+		menu, MenuCreateQuickPlay("Quick Play", mainMenu));
 	MenuAddSubmenu(
-		menu,
-		MenuCreateCampaigns(
-		"Dogfight",
-		"Select a scenario:",
-		&campaigns->dogfightList,
-		GAME_MODE_DOGFIGHT));
+		menu, MenuCreateCampaigns(
+				  "Dogfight", "Select a dogfight scenario:",
+				  mainMenu, &mainMenu->campaigns.dogfightList, GAME_MODE_DOGFIGHT));
 	MenuAddSubmenu(
-		menu,
-		MenuCreateCampaigns(
-		"Deathmatch",
-		"Select a scenario:",
-		&campaigns->dogfightList,
-		GAME_MODE_DEATHMATCH));
+		menu, MenuCreateCampaigns(
+				  "Deathmatch", "Select a deathmatch scenario:",
+				  mainMenu, &mainMenu->campaigns.dogfightList, GAME_MODE_DEATHMATCH));
 	MenuAddSubmenu(
-		menu, CreateJoinLANGame("Join LAN game", "Choose LAN server", ms, l));
+		menu, CreateJoinLANGame("Join LAN game", "Choose LAN server", &mainMenu->ms, l));
 	CheckLANServerData *cdata;
 	CMALLOC(cdata, sizeof *cdata);
 	cdata->MenuJoinIndex = (int)menu->u.normal.subMenus.size - 1;
 	MenuAddSubmenu(menu, MenuCreateSeparator(""));
 	MenuAddSubmenu(menu, MenuCreateBack("Back"));
 
-	if (strlen(gAutosave.LastMission.Password) == 0 ||
-		!gAutosave.LastMission.IsValid)
+	if (!CampaignSaveIsValid(cs))
 	{
 		MenuDisableSubmenu(menu, menuContinueIndex);
 	}
@@ -296,12 +343,12 @@ static menu_t *MenuCreateStart(
 typedef struct
 {
 	GameMode GameMode;
-	CampaignEntry *Entry;
+	const CampaignEntry *Entry;
+	MainMenuData *MainMenu;
 } StartGameModeData;
 static void StartGameMode(menu_t *menu, void *data);
-
 static menu_t *CreateStartGameMode(
-	const char *name, GameMode mode, CampaignEntry *entry)
+	const char *name, MainMenuData *mainMenu, GameMode mode, const CampaignEntry *entry)
 {
 	menu_t *menu = MenuCreate(name, MENU_TYPE_RETURN);
 	menu->enterSound = MENU_SOUND_START;
@@ -309,6 +356,7 @@ static menu_t *CreateStartGameMode(
 	CCALLOC(data, sizeof *data);
 	data->GameMode = mode;
 	data->Entry = entry;
+	data->MainMenu = mainMenu;
 	MenuSetPostEnterFunc(menu, StartGameMode, data, true);
 	return menu;
 }
@@ -316,29 +364,28 @@ static void StartGameMode(menu_t *menu, void *data)
 {
 	UNUSED(menu);
 	StartGameModeData *mData = data;
-	gCampaign.Entry.Mode = mData->GameMode;
-	if (!CampaignLoad(&gCampaign, mData->Entry))
-	{
-		// Failed to load
-		printf("Error: cannot load campaign %s\n", mData->Entry->Info);
-	}
+	mData->MainMenu->gameMode = mData->GameMode;
+	mData->MainMenu->entry = mData->Entry;
 }
-static menu_t *MenuCreateContinue(const char *name, CampaignEntry *entry)
+static menu_t *MenuCreateContinue(const char *name, MainMenuData *mainMenu, const CampaignEntry *entry)
 {
-	return CreateStartGameMode(name, GAME_MODE_NORMAL, entry);
+	return CreateStartGameMode(name, mainMenu, GAME_MODE_NORMAL, entry);
 }
-static menu_t *MenuCreateQuickPlay(const char *name, CampaignEntry *entry)
+static menu_t *MenuCreateQuickPlay(const char *name, MainMenuData *mainMenu)
 {
-	return CreateStartGameMode(name, GAME_MODE_QUICK_PLAY, entry);
+	return CreateStartGameMode(name, mainMenu, GAME_MODE_QUICK_PLAY, &mainMenu->campaigns.quickPlayEntry);
 }
 
-static menu_t *MenuCreateCampaignItem(
-	CampaignEntry *entry, const GameMode mode);
+static menu_t *MenuCreateCampaignItem(MainMenuData *mainMenu, CampaignEntry *entry, const GameMode mode);
 
 static void CampaignsDisplayFilename(
-	const menu_t *menu, GraphicsDevice *g,
-	const struct vec2i pos, const struct vec2i size, const void *data)
+	const menu_t *menu, GraphicsDevice *g, const struct vec2i pos,
+	const struct vec2i size, const void *data)
 {
+	if (menu->u.normal.subMenus.size == 0)
+	{
+		return;
+	}
 	const menu_t *subMenu =
 		CArrayGet(&menu->u.normal.subMenus, menu->u.normal.index);
 	UNUSED(g);
@@ -360,44 +407,45 @@ static void CampaignsDisplayFilename(
 	FontStrOpt(s, pos, opts);
 }
 static menu_t *MenuCreateCampaigns(
-	const char *name, const char *title,
-	campaign_list_t *list, const GameMode mode)
+	const char *name, const char *title, MainMenuData *mainMenu, CampaignList *list,
+	const GameMode mode)
 {
 	menu_t *menu = MenuCreateNormal(name, title, MENU_TYPE_NORMAL, 0);
 	menu->u.normal.maxItems = 20;
 	menu->u.normal.align = MENU_ALIGN_CENTER;
-	CA_FOREACH(campaign_list_t, subList, list->subFolders)
-		char folderName[CDOGS_FILENAME_MAX];
-		sprintf(folderName, "%s/", subList->Name);
-		MenuAddSubmenu(
-			menu, MenuCreateCampaigns(folderName, title, subList, mode));
+	CA_FOREACH(CampaignList, subList, list->subFolders)
+	char folderName[CDOGS_FILENAME_MAX];
+	sprintf(folderName, "%s/", subList->Name);
+	MenuAddSubmenu(
+		menu, MenuCreateCampaigns(folderName, title, mainMenu, subList, mode));
 	CA_FOREACH_END()
 	CA_FOREACH(CampaignEntry, e, list->list)
-		MenuAddSubmenu(menu, MenuCreateCampaignItem(e, mode));
+	MenuAddSubmenu(menu, MenuCreateCampaignItem(mainMenu, e, mode));
 	CA_FOREACH_END()
 	MenuSetCustomDisplay(menu, CampaignsDisplayFilename, NULL);
 	return menu;
 }
 
-static menu_t *MenuCreateCampaignItem(
-	CampaignEntry *entry, const GameMode mode)
+static menu_t *MenuCreateCampaignItem(MainMenuData *mainMenu, CampaignEntry *entry, const GameMode mode)
 {
-	menu_t *menu = CreateStartGameMode(entry->Info, mode, entry);
+	menu_t *menu = CreateStartGameMode(entry->Info, mainMenu, mode, entry);
 	// Special colors:
 	// - Green for new campaigns
 	// - White (normal) for in-progress campaigns
 	// - Grey for complete campaigns
-	MissionSave m;
-	AutosaveLoadMission(&gAutosave, &m, entry->Path);
-	if (m.MissionsCompleted == entry->NumMissions)
+	const CampaignSave *m = AutosaveGetCampaign(&gAutosave, entry->Path);
+	if (m != NULL)
 	{
-		// Completed campaign
-		menu->color = colorGray;
-	}
-	else if (m.MissionsCompleted > 0)
-	{
-		// Campaign in progress
-		menu->color = colorYellow;
+		if ((int)m->MissionsCompleted.size == entry->NumMissions)
+		{
+			// Completed campaign
+			menu->color = colorGray;
+		}
+		else if (m->MissionsCompleted.size > 0)
+		{
+			// Campaign in progress
+			menu->color = colorYellow;
+		}
 	}
 
 	return menu;
@@ -436,30 +484,31 @@ static void CreateLANServerMenuItems(menu_t *menu, void *data)
 	// Clear and recreate all menu items
 	MenuClearSubmenus(menu);
 	CA_FOREACH(ScanInfo, si, gNetClient.ScannedAddrs)
-		char buf[512];
-		char ipbuf[256];
-		if (enet_address_get_host_ip(&si->Addr, ipbuf, sizeof ipbuf) < 0)
-		{
-			LOG(LM_MAIN, LL_WARN, "cannot find host ip");
-			ipbuf[0] = '?';
-			ipbuf[1] = '\0';
-		};
-		// e.g. "Bob's Server (123.45.67.89:12345) - Campaign: Ogre Rampage #4, p: 4/16 350ms"
-		sprintf(buf, "%s (%s:%u) - %s: %s (# %d), p: %d/%d %dms",
-			si->ServerInfo.Hostname, ipbuf, si->Addr.port,
-			GameModeStr(si->ServerInfo.GameMode),
-			si->ServerInfo.CampaignName, si->ServerInfo.MissionNumber,
-			si->ServerInfo.NumPlayers, si->ServerInfo.MaxPlayers,
-			si->LatencyMS);
-		menu_t *serverMenu = MenuCreate(buf, MENU_TYPE_RETURN);
-		serverMenu->enterSound = MENU_SOUND_START;
-		JoinLANGameData *jdata;
-		CMALLOC(jdata, sizeof *jdata);
-		jdata->MS = cData->MS;
-		jdata->L = cData->L;
-		jdata->AddrIndex = _ca_index;
-		MenuSetPostEnterFunc(serverMenu, JoinLANGame, jdata, true);
-		MenuAddSubmenu(menu, serverMenu);
+	char buf[512];
+	char ipbuf[256];
+	if (enet_address_get_host_ip(&si->Addr, ipbuf, sizeof ipbuf) < 0)
+	{
+		LOG(LM_MAIN, LL_WARN, "cannot find host ip");
+		ipbuf[0] = '?';
+		ipbuf[1] = '\0';
+	};
+	// e.g. "Bob's Server (123.45.67.89:12345) - Campaign: Ogre Rampage #4, p:
+	// 4/16 350ms"
+	sprintf(
+		buf, "%s (%s:%u) - %s: %s (# %d), p: %d/%d %dms",
+		si->ServerInfo.Hostname, ipbuf, si->Addr.port,
+		GameModeStr(si->ServerInfo.GameMode), si->ServerInfo.CampaignName,
+		si->ServerInfo.MissionNumber, si->ServerInfo.NumPlayers,
+		si->ServerInfo.MaxPlayers, si->LatencyMS);
+	menu_t *serverMenu = MenuCreate(buf, MENU_TYPE_RETURN);
+	serverMenu->enterSound = MENU_SOUND_START;
+	JoinLANGameData *jdata;
+	CMALLOC(jdata, sizeof *jdata);
+	jdata->MS = cData->MS;
+	jdata->L = cData->L;
+	jdata->AddrIndex = _ca_index;
+	MenuSetPostEnterFunc(serverMenu, JoinLANGame, jdata, true);
+	MenuAddSubmenu(menu, serverMenu);
 	CA_FOREACH_END()
 	MenuAddSubmenu(menu, MenuCreateSeparator(""));
 	MenuAddSubmenu(menu, MenuCreateBack("Back"));
@@ -471,8 +520,8 @@ static void JoinLANGame(menu_t *menu, void *data)
 	{
 		goto bail;
 	}
-	const ScanInfo *sinfo = CArrayGet(
-		&gNetClient.ScannedAddrs, jdata->AddrIndex);
+	const ScanInfo *sinfo =
+		CArrayGet(&gNetClient.ScannedAddrs, jdata->AddrIndex);
 	LOG(LM_MAIN, LL_INFO, "joining LAN game...");
 	if (NetClientTryConnect(&gNetClient, sinfo->Addr))
 	{
@@ -508,79 +557,124 @@ static void CheckLANServers(menu_t *menu, void *data)
 	}
 }
 
-static menu_t *MenuCreateOptionsGraphics(const char *name, MenuSystem *ms);
+static void PostInputConfigApply(menu_t *menu, int cmd, void *data)
+{
+	UNUSED(menu);
+	UNUSED(cmd);
+	MainMenuData *mData = data;
+	bool resetBg = false;
+	if (!ConfigApply(&gConfig, &resetBg))
+	{
+		LOG(LM_MAIN, LL_ERROR, "Failed to apply config; reset to last used");
+		ConfigResetChanged(&gConfig);
+	}
+	else
+	{
+		// Save config immediately
+		ConfigSave(&gConfig, GetConfigFilePath(CONFIG_FILE));
+		if (resetBg)
+		{
+			GenerateLiveBackground(mData);
+		}
+	}
+
+	MenuResetSize(&mData->ms);
+}
+
+static menu_t *MenuCreateConfigOptions(
+	const char *name, const char *title, const Config *c, MainMenuData *data,
+	const bool backOrReturn)
+{
+	menu_t *menu = MenuCreateNormal(name, title, MENU_TYPE_OPTIONS, 0);
+	CASSERT(
+		c->Type == CONFIG_TYPE_GROUP,
+		"Cannot make menu from non-group config");
+	CA_FOREACH(Config, child, c->u.Group)
+	MenuAddConfigOptionsItem(menu, child);
+	CA_FOREACH_END()
+	MenuAddSubmenu(menu, MenuCreateSeparator(""));
+	if (backOrReturn)
+	{
+		MenuAddSubmenu(menu, MenuCreateBack("Done"));
+	}
+	else
+	{
+		MenuAddSubmenu(menu, MenuCreateReturn("Done", 0));
+	}
+	MenuSetPostInputFunc(menu, PostInputConfigApply, data);
+	return menu;
+}
+
+static menu_t *MenuCreateOptionsGraphics(const char *name, MainMenuData *data);
 #if !defined(__ANDROID__) && !defined(__GCWZERO__)
-static menu_t *MenuCreateOptionsControls(const char *name, MenuSystem *ms);
+static menu_t *MenuCreateOptionsControls(const char *name, MainMenuData *data);
 #endif
 
-static menu_t *MenuCreateOptions(const char *name, MenuSystem *ms)
+static menu_t *MenuCreateOptions(const char *name, MainMenuData *data)
 {
-	menu_t *menu = MenuCreateNormal(
-		name,
-		"Options:",
-		MENU_TYPE_NORMAL,
-		0);
-	MenuAddSubmenu(menu, MenuCreateConfigOptions(
-		"Game...", "Game Options:", ConfigGet(&gConfig, "Game"), ms, true));
-	MenuAddSubmenu(menu, MenuCreateOptionsGraphics("Graphics...", ms));
-	MenuAddSubmenu(menu, MenuCreateConfigOptions(
-		"Interface...", "Interface Options:",
-		ConfigGet(&gConfig, "Interface"), ms, true));
+	menu_t *menu = MenuCreateNormal(name, "Options:", MENU_TYPE_NORMAL, 0);
+	MenuAddSubmenu(
+		menu, MenuCreateConfigOptions(
+				  "Game...", "Game Options:", ConfigGet(&gConfig, "Game"),
+				  data, true));
+	MenuAddSubmenu(menu, MenuCreateOptionsGraphics("Graphics...", data));
+	MenuAddSubmenu(
+		menu, MenuCreateConfigOptions(
+				  "Interface...", "Interface Options:",
+				  ConfigGet(&gConfig, "Interface"), data, true));
 #if !defined(__ANDROID__) && !defined(__GCWZERO__)
-	MenuAddSubmenu(menu, MenuCreateOptionsControls("Controls...", ms));
+	MenuAddSubmenu(menu, MenuCreateOptionsControls("Controls...", data));
 #endif
-	MenuAddSubmenu(menu, MenuCreateConfigOptions(
-		"Sound...", "Configure Sound:", ConfigGet(&gConfig, "Sound"), ms,
-		true));
-	MenuAddSubmenu(menu, MenuCreateConfigOptions(
-		"Quick Play...", "Quick Play Options:", ConfigGet(&gConfig, "QuickPlay"), ms,
-		true));
+	MenuAddSubmenu(
+		menu, MenuCreateConfigOptions(
+				  "Sound...", "Configure Sound:", ConfigGet(&gConfig, "Sound"),
+				  data, true));
+	MenuAddSubmenu(
+		menu, MenuCreateConfigOptions(
+				  "Quick Play...", "Quick Play Options:",
+				  ConfigGet(&gConfig, "QuickPlay"), data, true));
 	MenuAddSubmenu(menu, MenuCreateSeparator(""));
 	MenuAddSubmenu(menu, MenuCreateBack("Back"));
 	return menu;
 }
 
-menu_t *MenuCreateOptionsGraphics(const char *name, MenuSystem *ms)
+static menu_t *MenuCreateOptionsGraphics(const char *name, MainMenuData *data)
 {
-	menu_t *menu = MenuCreateNormal(
-		name,
-		"Graphics Options:",
-		MENU_TYPE_OPTIONS,
-		0);
+	menu_t *menu =
+		MenuCreateNormal(name, "Graphics Options:", MENU_TYPE_OPTIONS, 0);
 	MenuAddConfigOptionsItem(menu, ConfigGet(&gConfig, "Graphics.Brightness"));
 #ifndef __GCWZERO__
 #ifndef __ANDROID__
 	MenuAddConfigOptionsItem(menu, ConfigGet(&gConfig, "Graphics.Fullscreen"));
-	MenuAddConfigOptionsItem(
-		menu, ConfigGet(&gConfig, "Graphics.SecondWindow"));
-#endif	// ANDROID
+	// TODO: fix second window rendering
+	// MenuAddConfigOptionsItem(
+	//	menu, ConfigGet(&gConfig, "Graphics.SecondWindow"));
+#endif // ANDROID
 
-	MenuAddConfigOptionsItem(menu, ConfigGet(&gConfig, "Graphics.ScaleFactor"));
+	MenuAddConfigOptionsItem(
+		menu, ConfigGet(&gConfig, "Graphics.ScaleFactor"));
 	MenuAddConfigOptionsItem(menu, ConfigGet(&gConfig, "Graphics.ScaleMode"));
-#endif	// GCWZERO
+#endif // GCWZERO
 	MenuAddConfigOptionsItem(menu, ConfigGet(&gConfig, "Graphics.Shadows"));
 	MenuAddConfigOptionsItem(menu, ConfigGet(&gConfig, "Graphics.Gore"));
 	MenuAddConfigOptionsItem(menu, ConfigGet(&gConfig, "Graphics.Brass"));
 	MenuAddSubmenu(menu, MenuCreateSeparator(""));
 	MenuAddSubmenu(menu, MenuCreateBack("Done"));
-	MenuSetPostInputFunc(menu, PostInputConfigApply, ms);
+	MenuSetPostInputFunc(menu, PostInputConfigApply, data);
 	return menu;
 }
 
-menu_t *MenuCreateKeys(const char *name, MenuSystem *ms);
+static menu_t *MenuCreateKeys(const char *name, MainMenuData *data);
 
 #if !defined(__ANDROID__) && !defined(__GCWZERO__)
-menu_t *MenuCreateOptionsControls(const char *name, MenuSystem *ms)
+static menu_t *MenuCreateOptionsControls(const char *name, MainMenuData *data)
 {
-	menu_t *menu = MenuCreateNormal(
-		name,
-		"Configure Controls:",
-		MENU_TYPE_OPTIONS,
-		0);
-	MenuAddSubmenu(menu, MenuCreateKeys("Redefine keys...", ms));
+	menu_t *menu =
+		MenuCreateNormal(name, "Configure Controls:", MENU_TYPE_OPTIONS, 0);
+	MenuAddSubmenu(menu, MenuCreateKeys("Redefine keys...", data));
 	MenuAddSubmenu(menu, MenuCreateSeparator(""));
 	MenuAddSubmenu(menu, MenuCreateBack("Done"));
-	MenuSetPostInputFunc(menu, PostInputConfigApply, ms);
+	MenuSetPostInputFunc(menu, PostInputConfigApply, data);
 	return menu;
 }
 #endif
@@ -590,25 +684,20 @@ menu_t *MenuCreateQuit(const char *name)
 	return MenuCreate(name, MENU_TYPE_QUIT);
 }
 
-
 static void MenuCreateKeysSingleSection(
 	menu_t *menu, const char *sectionName, const int playerIndex);
 static menu_t *MenuCreateOptionChangeKey(
 	const key_code_e code, const int playerIndex, const bool isOptional);
 
-menu_t *MenuCreateKeys(const char *name, MenuSystem *ms)
+static menu_t *MenuCreateKeys(const char *name, MainMenuData *data)
 {
-	menu_t *menu = MenuCreateNormal(
-		name,
-		"",
-		MENU_TYPE_KEYS,
-		0);
+	menu_t *menu = MenuCreateNormal(name, "", MENU_TYPE_OPTIONS, 0);
 	MenuCreateKeysSingleSection(menu, "Keyboard 1", 0);
 	MenuCreateKeysSingleSection(menu, "Keyboard 2", 1);
 	MenuAddSubmenu(menu, MenuCreateOptionChangeKey(KEY_CODE_MAP, 0, true));
 	MenuAddSubmenu(menu, MenuCreateSeparator(""));
 	MenuAddSubmenu(menu, MenuCreateBack("Done"));
-	MenuSetPostInputFunc(menu, PostInputConfigApply, ms);
+	MenuSetPostInputFunc(menu, PostInputConfigApply, data);
 	return menu;
 }
 
@@ -643,3 +732,12 @@ static menu_t *MenuCreateOptionChangeKey(
 	menu->u.changeKey.isOptional = isOptional;
 	return menu;
 }
+
+static void MenuResetSize(MenuSystem *ms)
+{
+	// Update menu system so that resolution changes don't
+	// affect menu positions
+	ms->pos = svec2i_zero();
+	ms->size = svec2i(
+		ms->graphics->cachedConfig.Res.x, ms->graphics->cachedConfig.Res.y);
+}
