@@ -1,57 +1,56 @@
 /*
-    C-Dogs SDL
-    A port of the legendary (and fun) action/arcade cdogs.
-    Copyright (C) 1995 Ronny Wester
-    Copyright (C) 2003 Jeremy Chin
-    Copyright (C) 2003-2007 Lucas Martin-King
+	C-Dogs SDL
+	A port of the legendary (and fun) action/arcade cdogs.
+	Copyright (C) 1995 Ronny Wester
+	Copyright (C) 2003 Jeremy Chin
+	Copyright (C) 2003-2007 Lucas Martin-King
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-    This file incorporates work covered by the following copyright and
-    permission notice:
+	This file incorporates work covered by the following copyright and
+	permission notice:
 
-    Copyright (c) 2013-2014, 2016-2018 Cong Xu
-    All rights reserved.
+	Copyright (c) 2013-2014, 2016-2021 Cong Xu
+	All rights reserved.
 
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions are met:
 
-    Redistributions of source code must retain the above copyright notice, this
-    list of conditions and the following disclaimer.
-    Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
+	Redistributions of source code must retain the above copyright notice, this
+	list of conditions and the following disclaimer.
+	Redistributions in binary form must reproduce the above copyright notice,
+	this list of conditions and the following disclaimer in the documentation
+	and/or other materials provided with the distribution.
 
-    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+	POSSIBILITY OF SUCH DAMAGE.
 */
 #include "menu.h"
 
 #include <assert.h>
 
-#include <cdogs/config_io.h>
-#include <cdogs/files.h>
+#include <cdogs/draw/drawtools.h>
 #include <cdogs/font.h>
 #include <cdogs/gamedata.h>
 #include <cdogs/grafx_bg.h>
@@ -62,13 +61,12 @@
 #include <cdogs/sounds.h>
 #include <cdogs/utils.h>
 
-
 #define MS_CENTER_X(ms, w) CENTER_X((ms).pos, (ms).size, w)
 #define MS_CENTER_Y(ms, h) CENTER_Y((ms).pos, (ms).size, h)
 
 void MenuSystemInit(
-	MenuSystem *ms,
-	EventHandlers *handlers, GraphicsDevice *graphics, struct vec2i pos, struct vec2i size)
+	MenuSystem *ms, EventHandlers *handlers, GraphicsDevice *graphics,
+	struct vec2i pos, struct vec2i size)
 {
 	memset(ms, 0, sizeof *ms);
 	ms->root = ms->current = NULL;
@@ -92,7 +90,8 @@ void MenuSystemTerminate(MenuSystem *ms)
 	memset(ms, 0, sizeof *ms);
 }
 
-void MenuSetCreditsDisplayer(MenuSystem *menu, credits_displayer_t *creditsDisplayer)
+void MenuSetCreditsDisplayer(
+	MenuSystem *menu, credits_displayer_t *creditsDisplayer)
 {
 	menu->creditsDisplayer = creditsDisplayer;
 }
@@ -100,10 +99,10 @@ void MenuSetCreditsDisplayer(MenuSystem *menu, credits_displayer_t *creditsDispl
 int MenuHasExitType(MenuSystem *menu, menu_type_e exitType)
 {
 	CA_FOREACH(menu_type_e, m, menu->exitTypes)
-		if (*m == exitType)
-		{
-			return 1;
-		}
+	if (*m == exitType)
+	{
+		return 1;
+	}
 	CA_FOREACH_END()
 	return 0;
 }
@@ -133,6 +132,133 @@ int MenuIsExit(MenuSystem *ms)
 
 void MenuProcessChangeKey(menu_t *menu);
 
+bool MenuTypeHasSubMenus(const menu_type_e type)
+{
+	return type == MENU_TYPE_NORMAL || type == MENU_TYPE_OPTIONS;
+}
+
+static struct vec2i SubmenuGetSize(
+	const MenuSystem *ms, const menu_t *menu, const int idx)
+{
+	const menu_t *subMenu = CArrayGet(&menu->u.normal.subMenus, idx);
+	int maxWidth = 0;
+	CA_FOREACH(const menu_t, subMenu2, menu->u.normal.subMenus)
+	const int width = FontStrW(subMenu2->name);
+	if (width > maxWidth)
+	{
+		maxWidth = width;
+	}
+	CA_FOREACH_END()
+	// Limit max width if it is larger than the menu system size
+	maxWidth = MIN(ms->size.x, maxWidth);
+	// Add extra width for options menus
+	switch (subMenu->type)
+	{
+	case MENU_TYPE_SET_OPTION_RANGE:
+	case MENU_TYPE_SET_OPTION_SEED:					  // fallthrough
+	case MENU_TYPE_SET_OPTION_UP_DOWN_VOID_FUNC_VOID: // fallthrough
+	case MENU_TYPE_SET_OPTION_RANGE_GET_SET:		  // fallthrough
+		switch (subMenu->u.option.displayStyle)
+		{
+		case MENU_OPTION_DISPLAY_STYLE_NONE:
+			// Do nothing
+			break;
+		case MENU_OPTION_DISPLAY_STYLE_STR_FUNC:
+		case MENU_OPTION_DISPLAY_STYLE_INT_TO_STR_FUNC: // fallthrough
+			maxWidth += 80;
+			break;
+		default:
+			CASSERT(false, "unknown menu display type");
+			break;
+		}
+		break;
+	case MENU_TYPE_SET_OPTION_TOGGLE:
+	case MENU_TYPE_SET_OPTION_CHANGE_KEY: // fallthrough
+		maxWidth += 80;
+		break;
+	default:
+		// do nothing
+		break;
+	}
+
+	return svec2i(maxWidth, FontStrH(subMenu->name));
+}
+static Rect2i MenuGetSubmenuBounds(const MenuSystem *ms, const int idx)
+{
+	const menu_t *menu = ms->current;
+	if (menu == NULL || !MenuTypeHasSubMenus(menu->type))
+	{
+		return Rect2iZero();
+	}
+	// Calculate first/last indices
+	const int maxItems = menu->u.normal.maxItems;
+	const int iStart = maxItems > 0 ? menu->u.normal.scroll : 0;
+	if (idx < iStart)
+	{
+		return Rect2iZero();
+	}
+	const int maxIEnd =
+		MIN(maxItems > 0 ? iStart + maxItems : 99,
+			(int)menu->u.normal.subMenus.size);
+	int numMenuLines = 0;
+	// Count the number of menu items that can fit
+	// This is to account for multi-line items
+	int iEnd;
+	for (iEnd = iStart; iEnd < maxIEnd; iEnd++)
+	{
+		const menu_t *subMenu = CArrayGet(&menu->u.normal.subMenus, iEnd);
+		const int numLines = FontStrNumLines(subMenu->name);
+		if (menu->u.normal.maxItems != 0 &&
+			numMenuLines + numLines > menu->u.normal.maxItems)
+		{
+			break;
+		}
+		numMenuLines += numLines;
+	}
+	if (iEnd <= idx)
+	{
+		return Rect2iZero();
+	}
+
+	int maxWidth = 0;
+	CA_FOREACH(const menu_t, subMenu, menu->u.normal.subMenus)
+	const int width = FontStrW(subMenu->name);
+	if (width > maxWidth)
+	{
+		maxWidth = width;
+	}
+	CA_FOREACH_END()
+	// Limit max width if it is larger than the menu system size
+	maxWidth = MIN(ms->size.x, maxWidth);
+
+	struct vec2i pos;
+	const bool isCentered = menu->type == MENU_TYPE_NORMAL;
+	switch (ms->align)
+	{
+	case MENU_ALIGN_CENTER:
+		pos.x = MS_CENTER_X(*ms, maxWidth);
+		if (!isCentered)
+		{
+			pos.x -= 20;
+		}
+		break;
+	case MENU_ALIGN_LEFT:
+		pos.x = ms->pos.x;
+		break;
+	default:
+		CASSERT(false, "unknown alignment");
+		return Rect2iZero();
+	}
+
+	pos.y = MS_CENTER_Y(*ms, numMenuLines * FontH());
+	for (int i = iStart; i < idx; i++)
+	{
+		const struct vec2i size = SubmenuGetSize(ms, menu, i);
+		pos.y += size.y;
+	}
+	return Rect2iNew(pos, SubmenuGetSize(ms, menu, idx));
+}
+
 static GameLoopResult DefaultMenuUpdate(GameLoopData *data, LoopRunner *l);
 static void DefaultMenuDraw(GameLoopData *data);
 GameLoopData *MenuLoop(MenuSystem *menu)
@@ -149,28 +275,65 @@ static void DefaultMenuDraw(GameLoopData *data)
 {
 	MenuDraw(data->Data);
 }
+void MenuUpdateMouse(MenuSystem *ms)
+{
+	if (!MouseHasMoved(&ms->handlers->mouse))
+	{
+		return;
+	}
+	menu_t *menu = ms->current;
+	if (menu == NULL || !MenuTypeHasSubMenus(menu->type))
+	{
+		return;
+	}
+	// Get mouse position and change menu
+	menu->mouseHover = false;
+	for (int i = 0; i < (int)menu->u.normal.subMenus.size; i++)
+	{
+		const Rect2i bounds = MenuGetSubmenuBounds(ms, i);
+		if (!Rect2iIsInside(bounds, ms->handlers->mouse.currentPos))
+		{
+			continue;
+		}
+		menu->mouseHover = true;
+		if (menu->u.normal.index != i)
+		{
+			const menu_t *subMenu = CArrayGet(&menu->u.normal.subMenus, i);
+			if (!subMenu->isDisabled)
+			{
+				menu->u.normal.index = i;
+				MenuPlaySound(MENU_SOUND_SWITCH);
+			}
+		}
+		break;
+	}
+}
 GameLoopResult MenuUpdate(MenuSystem *ms)
 {
-	if (ms->current->type == MENU_TYPE_KEYS &&
+	if (ms->current->type == MENU_TYPE_OPTIONS &&
 		ms->current->u.normal.changeKeyMenu != NULL)
 	{
 		MenuProcessChangeKey(ms->current);
 	}
 	else
 	{
-		const int cmd = GetMenuCmd(ms->handlers);
+		const int cmd = GetMenuCmd(ms->handlers, ms->current->mouseHover);
 		if (cmd)
 		{
 			MenuProcessCmd(ms, cmd);
 		}
+		else
+		{
+			MenuUpdateMouse(ms);
+		}
 	}
 	// Check if anyone pressed escape, or we need a hard exit
 	int cmds[MAX_LOCAL_PLAYERS];
 	memset(cmds, 0, sizeof cmds);
-	GetPlayerCmds(&gEventHandlers, &cmds);
+	GetPlayerCmds(ms->handlers, &cmds);
 	const bool aborted =
 		ms->allowAborts &&
-		EventIsEscape(&gEventHandlers, cmds, GetMenuCmd(&gEventHandlers));
+		EventIsEscape(ms->handlers, cmds, GetMenuCmd(ms->handlers, false));
 	if (aborted || ms->handlers->HasQuit)
 	{
 		ms->hasAbort = true;
@@ -263,14 +426,22 @@ void MenuSetDisabled(menu_t *menu, const bool isDisabled)
 menu_t *MenuGetSubmenuByName(menu_t *menu, const char *name)
 {
 	CA_FOREACH(menu_t, subMenu, menu->u.normal.subMenus)
-		if (strcmp(subMenu->name, name) == 0)
-		{
-			return subMenu;
-		}
+	if (strcmp(subMenu->name, name) == 0)
+	{
+		return subMenu;
+	}
 	CA_FOREACH_END()
 	return NULL;
 }
 
+int MenuGetNumMenuItemsShown(const menu_t *menu)
+{
+	CASSERT(MenuTypeHasSubMenus(menu->type), "invalid menu type");
+	return menu->u.normal.maxItems > 0 ? MIN(menu->u.normal.maxItems,
+											 (int)menu->u.normal.subMenus.size)
+									   : (int)menu->u.normal.subMenus.size;
+}
+
 void ShowControls(void)
 {
 	FontOpts opts = FontOptsNew();
@@ -283,42 +454,35 @@ void ShowControls(void)
 		"(use joystick or D pad + START + SELECT)", svec2i_zero(), opts);
 #else
 	FontStrOpt(
-		"(use joystick 1 or arrow keys + Enter/Backspace)", svec2i_zero(), opts);
+		"(use joystick 1 or arrow keys + Enter/Backspace)", svec2i_zero(),
+		opts);
 #endif
 }
 
 struct vec2i DisplayMenuItem(
-	struct vec2i pos, const char *s, int selected, int isDisabled, color_t color)
+	GraphicsDevice *g, const Rect2i bounds, const char *s, const bool selected,
+	const bool isDisabled, const color_t color)
 {
+	if (isDisabled)
+	{
+		return FontStrMask(s, bounds.Pos, colorDarkGray);
+	}
 	if (selected)
 	{
-		return FontStrMask(s, pos, colorRed);
+		const color_t bg = {0, 255, 255, 64};
+		// Add 1px padding
+		const struct vec2i bgPos = svec2i_subtract(bounds.Pos, svec2i_one());
+		const struct vec2i bgSize = svec2i_add(bounds.Size, svec2i(2, 2));
+		DrawRectangle(g, bgPos, bgSize, bg, true);
+		return FontStrMask(s, bounds.Pos, colorRed);
 	}
-	else if (isDisabled)
+	if (!ColorEquals(color, colorTransparent))
 	{
-		color_t dark = { 64, 64, 64, 255 };
-		return FontStrMask(s, pos, dark);
-	}
-	else if (!ColorEquals(color, colorTransparent))
-	{
-		return FontStrMask(s, pos, color);
-	}
-	else
-	{
-		return FontStr(s, pos);
+		return FontStrMask(s, bounds.Pos, color);
 	}
+	return FontStr(s, bounds.Pos);
 }
 
-
-int MenuTypeHasSubMenus(menu_type_e type)
-{
-	return
-		type == MENU_TYPE_NORMAL ||
-		type == MENU_TYPE_OPTIONS ||
-		type == MENU_TYPE_KEYS;
-}
-
-
 menu_t *MenuCreate(const char *name, menu_type_e type)
 {
 	menu_t *menu;
@@ -331,10 +495,7 @@ menu_t *MenuCreate(const char *name, menu_type_e type)
 }
 
 menu_t *MenuCreateNormal(
-	const char *name,
-	const char *title,
-	menu_type_e type,
-	int displayItems)
+	const char *name, const char *title, menu_type_e type, int displayItems)
 {
 	menu_t *menu = MenuCreate(name, type);
 	strcpy(menu->u.normal.title, title);
@@ -353,14 +514,14 @@ menu_t *MenuCreateNormal(
 static void UpdateSubmenuParentPtrs(menu_t *menu)
 {
 	CA_FOREACH(menu_t, subMenu, menu->u.normal.subMenus)
-		subMenu->parentMenu = menu;
-		if (MenuTypeHasSubMenus(subMenu->type))
-		{
-			UpdateSubmenuParentPtrs(subMenu);
-		}
+	subMenu->parentMenu = menu;
+	if (MenuTypeHasSubMenus(subMenu->type))
+	{
+		UpdateSubmenuParentPtrs(subMenu);
+	}
 	CA_FOREACH_END()
 }
-void MenuAddSubmenu(menu_t *menu, menu_t *subMenu)
+menu_t *MenuAddSubmenu(menu_t *menu, menu_t *subMenu)
 {
 	CArrayPushBack(&menu->u.normal.subMenus, subMenu);
 	if (subMenu->type == MENU_TYPE_QUIT)
@@ -374,6 +535,9 @@ void MenuAddSubmenu(menu_t *menu, menu_t *subMenu)
 
 	// move cursor in case first menu item(s) are disabled
 	MoveIndexToNextEnabledSubmenu(menu, true);
+
+	return CArrayGet(
+		&menu->u.normal.subMenus, menu->u.normal.subMenus.size - 1);
 }
 
 void MenuSetPostInputFunc(menu_t *menu, MenuPostInputFunc func, void *data)
@@ -404,28 +568,6 @@ void MenuSetCustomDisplay(menu_t *menu, MenuDisplayFunc func, const void *data)
 	menu->customDisplayData = data;
 }
 
-menu_t *MenuCreateConfigOptions(
-	const char *name, const char *title, const Config *c, MenuSystem *ms,
-	const bool backOrReturn)
-{
-	menu_t *menu = MenuCreateNormal(name, title, MENU_TYPE_OPTIONS, 0);
-	CASSERT(c->Type == CONFIG_TYPE_GROUP,
-		"Cannot make menu from non-group config");
-	CA_FOREACH(Config, child, c->u.Group)
-		MenuAddConfigOptionsItem(menu, child);
-	CA_FOREACH_END()
-	MenuAddSubmenu(menu, MenuCreateSeparator(""));
-	if (backOrReturn)
-	{
-		MenuAddSubmenu(menu, MenuCreateBack("Done"));
-	}
-	else
-	{
-		MenuAddSubmenu(menu, MenuCreateReturn("Done", 0));
-	}
-	MenuSetPostInputFunc(menu, PostInputConfigApply, ms);
-	return menu;
-}
 void MenuAddConfigOptionsItem(menu_t *menu, Config *c)
 {
 	char nameBuf[256];
@@ -440,10 +582,9 @@ void MenuAddConfigOptionsItem(menu_t *menu, Config *c)
 		MenuAddSubmenu(
 			menu,
 			MenuCreateOptionRange(
-			nameBuf, (int *)&c->u.Int.Value,
-			c->u.Int.Min, c->u.Int.Max, c->u.Int.Increment,
-			MENU_OPTION_DISPLAY_STYLE_INT_TO_STR_FUNC,
-			(void (*)(void))c->u.Int.IntToStr));
+				nameBuf, (int *)&c->u.Int.Value, c->u.Int.Min, c->u.Int.Max,
+				c->u.Int.Increment, MENU_OPTION_DISPLAY_STYLE_INT_TO_STR_FUNC,
+				(void (*)(void))c->u.Int.IntToStr));
 		break;
 	case CONFIG_TYPE_FLOAT:
 		CASSERT(false, "Unimplemented");
@@ -456,10 +597,9 @@ void MenuAddConfigOptionsItem(menu_t *menu, Config *c)
 		MenuAddSubmenu(
 			menu,
 			MenuCreateOptionRange(
-			nameBuf, (int *)&c->u.Enum.Value,
-			c->u.Enum.Min, c->u.Enum.Max, 1,
-			MENU_OPTION_DISPLAY_STYLE_INT_TO_STR_FUNC,
-			(void(*)(void))c->u.Enum.EnumToStr));
+				nameBuf, (int *)&c->u.Enum.Value, c->u.Enum.Min, c->u.Enum.Max,
+				1, MENU_OPTION_DISPLAY_STYLE_INT_TO_STR_FUNC,
+				(void (*)(void))c->u.Enum.EnumToStr));
 		break;
 	case CONFIG_TYPE_GROUP:
 		// Do nothing
@@ -479,9 +619,7 @@ menu_t *MenuCreateOptionToggle(const char *name, bool *config)
 }
 
 menu_t *MenuCreateOptionRange(
-	const char *name,
-	int *config,
-	int low, int high, int increment,
+	const char *name, int *config, int low, int high, int increment,
 	menu_option_display_style_e style, void (*func)(void))
 {
 	menu_t *menu = MenuCreate(name, MENU_TYPE_SET_OPTION_RANGE);
@@ -510,11 +648,11 @@ menu_t *MenuCreateOptionSeed(const char *name, unsigned int *seed)
 }
 
 menu_t *MenuCreateOptionUpDownFunc(
-	const char *name,
-	void(*upFunc)(void), void(*downFunc)(void),
+	const char *name, void (*upFunc)(void), void (*downFunc)(void),
 	menu_option_display_style_e style, char *(*strFunc)(void))
 {
-	menu_t *menu = MenuCreate(name, MENU_TYPE_SET_OPTION_UP_DOWN_VOID_FUNC_VOID);
+	menu_t *menu =
+		MenuCreate(name, MENU_TYPE_SET_OPTION_UP_DOWN_VOID_FUNC_VOID);
 	menu->u.option.uHook.upDownFuncs.upFunc = upFunc;
 	menu->u.option.uHook.upDownFuncs.downFunc = downFunc;
 	menu->u.option.displayStyle = style;
@@ -522,8 +660,7 @@ menu_t *MenuCreateOptionUpDownFunc(
 	return menu;
 }
 
-menu_t *MenuCreateVoidFunc(
-	const char *name, void (*func)(void *), void *data)
+menu_t *MenuCreateVoidFunc(const char *name, void (*func)(void *), void *data)
 {
 	menu_t *menu = MenuCreate(name, MENU_TYPE_VOID_FUNC);
 	menu->u.option.uHook.voidFunc.func = func;
@@ -533,10 +670,9 @@ menu_t *MenuCreateVoidFunc(
 }
 
 menu_t *MenuCreateOptionRangeGetSet(
-	const char *name,
-	int(*getFunc)(void), void(*setFunc)(int),
-	int low, int high, int increment,
-	menu_option_display_style_e style, void (*func)(void))
+	const char *name, int (*getFunc)(void), void (*setFunc)(int), int low,
+	int high, int increment, menu_option_display_style_e style,
+	void (*func)(void))
 {
 	menu_t *menu = MenuCreate(name, MENU_TYPE_SET_OPTION_RANGE_GET_SET);
 	menu->u.option.uHook.optionRangeGetSet.getFunc = getFunc;
@@ -573,8 +709,7 @@ menu_t *MenuCreateReturn(const char *name, int returnCode)
 }
 
 menu_t *MenuCreateCustom(
-	const char *name,
-	MenuDisplayFunc displayFunc, MenuInputFunc inputFunc,
+	const char *name, MenuDisplayFunc displayFunc, MenuInputFunc inputFunc,
 	void *data)
 {
 	menu_t *menu = MenuCreate(name, MENU_TYPE_CUSTOM);
@@ -584,7 +719,6 @@ menu_t *MenuCreateCustom(
 	return menu;
 }
 
-
 static void MenuDisplayItems(const MenuSystem *ms);
 static void MenuDisplaySubmenus(const MenuSystem *ms);
 void MenuDisplay(const MenuSystem *ms)
@@ -595,7 +729,8 @@ void MenuDisplay(const MenuSystem *ms)
 		if (menu->type == MENU_TYPE_CUSTOM)
 		{
 			menu->u.customData.displayFunc(
-				menu, ms->graphics, ms->pos, ms->size, menu->u.customData.data);
+				menu, ms->graphics, ms->pos, ms->size,
+				menu->u.customData.data);
 		}
 		else
 		{
@@ -614,7 +749,7 @@ void MenuDisplay(const MenuSystem *ms)
 		}
 	}
 	CA_FOREACH(MenuCustomDisplayFunc, cdf, ms->customDisplayFuncs)
-		cdf->Func(NULL, ms->graphics, ms->pos, ms->size, cdf->Data);
+	cdf->Func(NULL, ms->graphics, ms->pos, ms->size, cdf->Data);
 	CA_FOREACH_END()
 	if (menu != NULL && menu->customDisplayFunc)
 	{
@@ -634,7 +769,9 @@ static void MenuDisplayItems(const MenuSystem *ms)
 		const Pic *logo = PicManagerGetPic(&gPicManager, "logo");
 		const struct vec2i pos = svec2i(
 			MS_CENTER_X(*ms, logo->size.x), ms->pos.y + ms->size.y / 12);
-		Blit(&gGraphicsDevice, logo, pos);
+		PicRender(
+			logo, ms->graphics->gameWindow.renderer, pos, colorWhite, 0,
+			svec2_one(), SDL_FLIP_NONE, Rect2iZero());
 
 		FontOpts opts = FontOptsNew();
 		opts.HAlign = ALIGN_END;
@@ -646,249 +783,125 @@ static void MenuDisplayItems(const MenuSystem *ms)
 static int MenuOptionGetIntValue(const menu_t *menu);
 static void MenuDisplaySubmenus(const MenuSystem *ms)
 {
-	int x = 0, yStart = 0;
 	const menu_t *menu = ms->current;
-
-	switch (menu->type)
+	if (!MenuTypeHasSubMenus(menu->type))
 	{
-	// TODO: refactor the three menu types (normal, options, campaign) into one
-	case MENU_TYPE_NORMAL:
-	case MENU_TYPE_OPTIONS:
-		{
-			int iStart = 0;
-			int iEnd;
-			int numMenuLines = 0;
-			int maxIEnd = (int)menu->u.normal.subMenus.size;
-			if (menu->u.normal.maxItems > 0)
-			{
-				// Calculate first/last indices
-				if (menu->u.normal.scroll != 0)
-				{
-					iStart = menu->u.normal.scroll;
-				}
-				maxIEnd = iStart + menu->u.normal.maxItems;
-			}
-			// Count the number of menu items that can fit
-			// This is to account for multi-line items
-			for (iEnd = iStart;
-				iEnd < maxIEnd && iEnd < (int)menu->u.normal.subMenus.size;
-				iEnd++)
-			{
-				const menu_t *subMenu =
-					CArrayGet(&menu->u.normal.subMenus, iEnd);
-				const int numLines = FontStrNumLines(subMenu->name);
-				if (menu->u.normal.maxItems > 0 &&
-					numMenuLines + numLines > menu->u.normal.maxItems)
-				{
-					break;
-				}
-				numMenuLines += numLines;
-			}
-
-			int maxWidth = 0;
-			CA_FOREACH(const menu_t, subMenu, menu->u.normal.subMenus)
-				const int width = FontStrW(subMenu->name);
-				if (width > maxWidth)
-				{
-					maxWidth = width;
-				}
-			CA_FOREACH_END()
-			// Limit max width if it is larger than the menu system size
-			maxWidth = MIN(ms->size.x, maxWidth);
-			const bool isCentered = menu->type == MENU_TYPE_NORMAL;
-			switch (ms->align)
-			{
-			case MENU_ALIGN_CENTER:
-				x = MS_CENTER_X(*ms, maxWidth);
-				if (!isCentered)
-				{
-					x -= 20;
-				}
-				break;
-			case MENU_ALIGN_LEFT:
-				x = ms->pos.x;
-				break;
-			default:
-				assert(0 && "unknown alignment");
-				break;
-			}
-
-			yStart = MS_CENTER_Y(*ms, numMenuLines * FontH());
-			if (menu->u.normal.maxItems > 0)
-			{
-				// Display scroll arrows
-				if (menu->u.normal.scroll != 0)
-				{
-					DisplayMenuItem(
-						svec2i(
-							MS_CENTER_X(*ms, FontW('^')),
-							yStart - 2 - FontH()),
-						"^",
-						0, 0,
-						colorBlack);
-				}
-				if (iEnd < (int)menu->u.normal.subMenus.size - 1)
-				{
-					DisplayMenuItem(
-						svec2i(
-							MS_CENTER_X(*ms, FontW('v')),
-							yStart + numMenuLines*FontH() + 2),
-						"v",
-						0, 0,
-						colorBlack);
-				}
-			}
-			const int xOptions = x + maxWidth + 10;
-
-			// Display normal menu items
-			struct vec2i pos = svec2i(x, yStart);
-			for (int i = iStart; i < iEnd; i++)
-			{
-				const menu_t *subMenu = CArrayGet(&menu->u.normal.subMenus, i);
-				char *nameBuf;
-				CMALLOC(nameBuf, strlen(subMenu->name) + 3);
-				if (subMenu->type == MENU_TYPE_NORMAL &&
-					subMenu->u.normal.isSubmenusAlt)
-				{
-					sprintf(nameBuf, "%s >", subMenu->name);
-				}
-				else
-				{
-					strcpy(nameBuf, subMenu->name);
-				}
-
-				switch (menu->u.normal.align)
-				{
-				case MENU_ALIGN_CENTER:
-					pos.x = MS_CENTER_X(*ms, FontStrW(nameBuf));
-					break;
-				case MENU_ALIGN_LEFT:
-					// Do nothing
-					break;
-				default:
-					assert(0 && "unknown alignment");
-					break;
-				}
-
-				const int yNext = DisplayMenuItem(
-					pos,
-					nameBuf,
-					i == menu->u.normal.index,
-					subMenu->isDisabled,
-					subMenu->color).y + FontH();
-				CFREE(nameBuf);
-
-				// display option value
-				const int optionInt = MenuOptionGetIntValue(subMenu);
-				const struct vec2i valuePos = svec2i(xOptions, pos.y);
-				if (subMenu->type == MENU_TYPE_SET_OPTION_RANGE ||
-					subMenu->type == MENU_TYPE_SET_OPTION_SEED ||
-					subMenu->type == MENU_TYPE_SET_OPTION_UP_DOWN_VOID_FUNC_VOID ||
-					subMenu->type == MENU_TYPE_SET_OPTION_RANGE_GET_SET)
-				{
-					switch (subMenu->u.option.displayStyle)
-					{
-					case MENU_OPTION_DISPLAY_STYLE_NONE:
-						// Do nothing
-						break;
-					case MENU_OPTION_DISPLAY_STYLE_STR_FUNC:
-						FontStr(subMenu->u.option.uFunc.str(), valuePos);
-						break;
-					case MENU_OPTION_DISPLAY_STYLE_INT_TO_STR_FUNC:
-						FontStr(
-							subMenu->u.option.uFunc.intToStr(optionInt),
-							valuePos);
-						break;
-					default:
-						CASSERT(false, "unknown menu display type");
-						break;
-					}
-				}
-				else if (subMenu->type == MENU_TYPE_SET_OPTION_TOGGLE)
-				{
-					FontStr(optionInt ? "Yes" : "No", valuePos);
-				}
-
-				pos.y = yNext;
-			}
-		}
-		break;
-	case MENU_TYPE_KEYS:
-		{
-			x = MS_CENTER_X(*ms, (FontW('a') * 10)) / 2;
-			const int xKeys = x * 3;
-			yStart = (gGraphicsDevice.cachedConfig.Res.y / 2) - (FontH() * 10);
-
-			CA_FOREACH(const menu_t, subMenu, menu->u.normal.subMenus)
-				int y = yStart + _ca_index * FontH();
-				bool isSelected = _ca_index == menu->u.normal.index;
-
-				const char *name = subMenu->name;
-				if (isSelected &&
-					subMenu->type != MENU_TYPE_SET_OPTION_CHANGE_KEY)
-				{
-					FontStrMask(name, svec2i(x, y), colorRed);
-				}
-				else
-				{
-					FontStr(name, svec2i(x, y));
-				}
-
-				if (subMenu->type == MENU_TYPE_SET_OPTION_CHANGE_KEY)
-				{
-					const char *keyName;
-					if (menu->u.normal.changeKeyMenu == subMenu)
-					{
-						keyName = "Press a key";
-					}
-					else
-					{
-						const int pi = subMenu->u.changeKey.playerIndex;
-						const InputKeys *keys =
-							&gEventHandlers.keyboard.PlayerKeys[pi];
-						const SDL_Scancode sc = KeyGet(
-							keys, subMenu->u.changeKey.code);
-						keyName = SDL_GetScancodeName(sc);
-						if (sc == SDL_SCANCODE_UNKNOWN || keyName == NULL)
-						{
-							keyName = "<Unset>";
-						}
-					}
-					DisplayMenuItem(
-						svec2i(xKeys, y),
-						keyName,
-						isSelected,
-						0,
-						colorBlack);
-				}
-			CA_FOREACH_END()
-		}
-		break;
-	default:
-		// No submenus, don't display anything
-		break;
+		return;
 	}
-}
 
+	int maxWidth = 0;
+	CA_FOREACH(const menu_t, subMenu, menu->u.normal.subMenus)
+	const int width = FontStrW(subMenu->name);
+	if (width > maxWidth)
+	{
+		maxWidth = width;
+	}
+	CA_FOREACH_END()
+
+	CA_FOREACH(const menu_t, subMenu, menu->u.normal.subMenus)
+	const Rect2i bounds = MenuGetSubmenuBounds(ms, _ca_index);
+	if (Rect2iIsZero(bounds))
+	{
+		continue;
+	}
+
+	const int xOptions = bounds.Pos.x + maxWidth + 10;
+	char nameBuf[512];
+	if (subMenu->type == MENU_TYPE_NORMAL && subMenu->u.normal.isSubmenusAlt)
+	{
+		snprintf(nameBuf, sizeof(nameBuf), "%s " ARROW_RIGHT, subMenu->name);
+	}
+	else
+	{
+		snprintf(nameBuf, sizeof(nameBuf), "%s", subMenu->name);
+	}
+
+	const bool isSelected = _ca_index == menu->u.normal.index;
+	DisplayMenuItem(
+		ms->graphics, bounds, nameBuf, isSelected,
+		menu->isDisabled || subMenu->isDisabled, subMenu->color);
+
+	// display option value
+	const int optionInt = MenuOptionGetIntValue(subMenu);
+	const struct vec2i valuePos = svec2i(xOptions, bounds.Pos.y);
+	const char *option = NULL;
+	if (subMenu->type == MENU_TYPE_SET_OPTION_RANGE ||
+		subMenu->type == MENU_TYPE_SET_OPTION_SEED ||
+		subMenu->type == MENU_TYPE_SET_OPTION_UP_DOWN_VOID_FUNC_VOID ||
+		subMenu->type == MENU_TYPE_SET_OPTION_RANGE_GET_SET)
+	{
+		switch (subMenu->u.option.displayStyle)
+		{
+		case MENU_OPTION_DISPLAY_STYLE_NONE:
+			// Do nothing
+			break;
+		case MENU_OPTION_DISPLAY_STYLE_STR_FUNC:
+			option = subMenu->u.option.uFunc.str();
+			break;
+		case MENU_OPTION_DISPLAY_STYLE_INT_TO_STR_FUNC:
+			option = subMenu->u.option.uFunc.intToStr(optionInt);
+			break;
+		default:
+			CASSERT(false, "unknown menu display type");
+			break;
+		}
+	}
+	else if (subMenu->type == MENU_TYPE_SET_OPTION_TOGGLE)
+	{
+		option = optionInt ? "Yes" : "No";
+	}
+	else if (subMenu->type == MENU_TYPE_SET_OPTION_CHANGE_KEY)
+	{
+		if (menu->u.normal.changeKeyMenu == subMenu)
+		{
+			option = "Press a key";
+		}
+		else
+		{
+			const int pi = subMenu->u.changeKey.playerIndex;
+			const InputKeys *keys = &ms->handlers->keyboard.PlayerKeys[pi];
+			const SDL_Scancode sc = KeyGet(keys, subMenu->u.changeKey.code);
+			option = SDL_GetScancodeName(sc);
+			if (sc == SDL_SCANCODE_UNKNOWN || option == NULL)
+			{
+				option = "Unset";
+			}
+		}
+	}
+	if (option != NULL)
+	{
+		char buf[256];
+		if (isSelected)
+		{
+			sprintf(buf, ARROW_LEFT " %s " ARROW_RIGHT, option);
+		}
+		else
+		{
+			strcpy(buf, option);
+		}
+		FontStr(buf, valuePos);
+	}
+	CA_FOREACH_END()
+}
 
 void MenuPlaySound(MenuSound s)
 {
 	switch (s)
 	{
 	case MENU_SOUND_ENTER:
-		SoundPlay(&gSoundDevice, StrSound("mg"));
+		SoundPlay(&gSoundDevice, StrSound("menu_enter"));
 		break;
 	case MENU_SOUND_BACK:
-		SoundPlay(&gSoundDevice, StrSound("pickup"));
+		SoundPlay(&gSoundDevice, StrSound("menu_back"));
 		break;
 	case MENU_SOUND_SWITCH:
-		SoundPlay(&gSoundDevice, StrSound("door"));
+		SoundPlay(&gSoundDevice, StrSound("menu_switch"));
 		break;
 	case MENU_SOUND_START:
-		SoundPlay(&gSoundDevice, StrSound("hahaha"));
+		SoundPlay(&gSoundDevice, StrSound("menu_start"));
 		break;
 	case MENU_SOUND_ERROR:
-		SoundPlay(&gSoundDevice, StrSound("aargh/man"));
+		SoundPlay(&gSoundDevice, StrSound("menu_error"));
 		break;
 	default:
 		break;
@@ -920,7 +933,7 @@ static void MenuTerminateSubmenus(menu_t *menu)
 		return;
 	}
 	CA_FOREACH(menu_t, subMenu, menu->u.normal.subMenus)
-		MenuTerminate(subMenu);
+	MenuTerminate(subMenu);
 	CA_FOREACH_END()
 	CArrayTerminate(&menu->u.normal.subMenus);
 }
@@ -962,6 +975,10 @@ void MenuProcessCmd(MenuSystem *ms, int cmd)
 {
 	menu_t *menu = ms->current;
 	menu_t *menuToChange = NULL;
+	if (menu == NULL)
+	{
+		goto bail;
+	}
 	if (cmd == CMD_ESC || (cmd & CMD_BUTTON2) ||
 		((cmd & CMD_LEFT) && menu->u.normal.isSubmenusAlt))
 	{
@@ -994,7 +1011,7 @@ void MenuProcessCmd(MenuSystem *ms, int cmd)
 	}
 
 bail:
-	if (menu->customPostInputFunc)
+	if (menu && menu->customPostInputFunc)
 	{
 		menu->customPostInputFunc(menu, cmd, menu->customPostInputData);
 	}
@@ -1028,7 +1045,6 @@ menu_t *MenuProcessEscCmd(menu_t *menu)
 	return menuToChange;
 }
 
-
 void MenuActivate(MenuSystem *ms, menu_t *menu, int cmd);
 
 menu_t *MenuProcessButtonCmd(MenuSystem *ms, menu_t *menu, int cmd)
@@ -1049,10 +1065,9 @@ menu_t *MenuProcessButtonCmd(MenuSystem *ms, menu_t *menu, int cmd)
 		{
 		case MENU_TYPE_NORMAL:
 		case MENU_TYPE_OPTIONS:
-		case MENU_TYPE_KEYS:
 		case MENU_TYPE_CUSTOM:
-			if (subMenu->u.normal.isSubmenusAlt ?
-				(cmd & CMD_RIGHT) : (cmd & CMD_BUTTON1))
+			if (subMenu->u.normal.isSubmenusAlt ? (cmd & CMD_RIGHT)
+												: (cmd & CMD_BUTTON1))
 			{
 				return subMenu;
 			}
@@ -1066,7 +1081,7 @@ menu_t *MenuProcessButtonCmd(MenuSystem *ms, menu_t *menu, int cmd)
 		case MENU_TYPE_QUIT:
 			if (cmd & CMD_BUTTON1)
 			{
-				return subMenu;	// caller will check if subMenu type is QUIT
+				return subMenu; // caller will check if subMenu type is QUIT
 			}
 			break;
 		case MENU_TYPE_RETURN:
@@ -1086,8 +1101,8 @@ menu_t *MenuProcessButtonCmd(MenuSystem *ms, menu_t *menu, int cmd)
 static bool KeyAvailable(
 	const SDL_Scancode key, const key_code_e code, const int playerIndex)
 {
-	if (key == SDL_SCANCODE_ESCAPE ||
-		key == SDL_SCANCODE_F9 || key == SDL_SCANCODE_F10)
+	if (key == SDL_SCANCODE_ESCAPE || key == SDL_SCANCODE_F9 ||
+		key == SDL_SCANCODE_F10)
 	{
 		return false;
 	}
@@ -1115,12 +1130,9 @@ static bool KeyAvailable(
 	// Check if the other player is using the key
 	sprintf(buf, "Input.PlayerCodes%d", 1 - playerIndex);
 	const InputKeys keysOther = KeyLoadPlayerKeys(ConfigGet(&gConfig, buf));
-	if (keysOther.left == key ||
-		keysOther.right == key ||
-		keysOther.up == key ||
-		keysOther.down == key ||
-		keysOther.button1 == key ||
-		keysOther.button2 == key)
+	if (keysOther.left == key || keysOther.right == key ||
+		keysOther.up == key || keysOther.down == key ||
+		keysOther.button1 == key || keysOther.button2 == key)
 	{
 		return false;
 	}
@@ -1176,7 +1188,6 @@ static void ChangeKey(
 	MenuPlaySound(MENU_SOUND_ENTER);
 }
 
-
 void MenuChangeIndex(menu_t *menu, int cmd)
 {
 	// Ignore if no submenus
@@ -1205,17 +1216,17 @@ void MenuChangeIndex(menu_t *menu, int cmd)
 		MoveIndexToNextEnabledSubmenu(menu, true);
 		MenuPlaySound(MENU_SOUND_SWITCH);
 	}
-	menu->u.normal.scroll =
-		CLAMP(menu->u.normal.scroll,
-			MAX(0, menu->u.normal.index - menu->u.normal.maxItems + 1),
-			MIN((int)menu->u.normal.subMenus.size - 1, menu->u.normal.index + menu->u.normal.maxItems - 1));
+	const int nMenuItems = MenuGetNumMenuItemsShown(menu);
+	menu->u.normal.scroll = CLAMP(
+		menu->u.normal.scroll, MAX(0, menu->u.normal.index - nMenuItems + 1),
+		MIN((int)menu->u.normal.subMenus.size - 1,
+			menu->u.normal.index + nMenuItems - 1));
 	if (menu->u.normal.index < menu->u.normal.scroll)
 	{
 		menu->u.normal.scroll = menu->u.normal.index;
 	}
 }
 
-
 void MenuActivate(MenuSystem *ms, menu_t *menu, int cmd)
 {
 	UNUSED(ms);
@@ -1226,84 +1237,83 @@ void MenuActivate(MenuSystem *ms, menu_t *menu, int cmd)
 		// do nothing
 		return;
 	case MENU_TYPE_SET_OPTION_TOGGLE:
-		*menu->u.option.uHook.optionToggle = !*menu->u.option.uHook.optionToggle;
+		*menu->u.option.uHook.optionToggle =
+			!*menu->u.option.uHook.optionToggle;
 		break;
-	case MENU_TYPE_SET_OPTION_RANGE:
+	case MENU_TYPE_SET_OPTION_RANGE: {
+		int option = *menu->u.option.uHook.optionRange.option;
+		int increment = menu->u.option.uHook.optionRange.increment;
+		int low = menu->u.option.uHook.optionRange.low;
+		int high = menu->u.option.uHook.optionRange.high;
+		if (Left(cmd))
 		{
-			int option = *menu->u.option.uHook.optionRange.option;
-			int increment = menu->u.option.uHook.optionRange.increment;
-			int low = menu->u.option.uHook.optionRange.low;
-			int high = menu->u.option.uHook.optionRange.high;
-			if (Left(cmd))
+			if (low == option)
 			{
-				if (low == option)
-				{
-					option = high;
-				}
-				else if (low + increment > option)
-				{
-					option = low;
-				}
-				else
-				{
-					option -= increment;
-				}
+				option = high;
 			}
-			else if (Right(cmd))
+			else if (low + increment > option)
 			{
-				if (high == option)
-				{
-					option = low;
-				}
-				else if (high - increment < option)
-				{
-					option = high;
-				}
-				else
-				{
-					option += increment;
-				}
+				option = low;
+			}
+			else
+			{
+				option -= increment;
 			}
-			*menu->u.option.uHook.optionRange.option = option;
 		}
-		break;
-	case MENU_TYPE_SET_OPTION_SEED:
+		else if (Right(cmd))
 		{
-			unsigned int seed = *menu->u.option.uHook.seed;
-			unsigned int increment = 1;
-			if (Button1(cmd))
+			if (high == option)
 			{
-				increment *= 10;
+				option = low;
 			}
-			if (Button2(cmd))
+			else if (high - increment < option)
 			{
-				increment *= 100;
+				option = high;
 			}
-			if (Left(cmd))
+			else
 			{
-				if (increment > seed)
-				{
-					seed = 0;
-				}
-				else
-				{
-					seed -= increment;
-				}
+				option += increment;
 			}
-			else if (Right(cmd))
-			{
-				if (UINT_MAX - increment < seed)
-				{
-					seed = UINT_MAX;
-				}
-				else
-				{
-					seed += increment;
-				}
-			}
-			*menu->u.option.uHook.seed = seed;
 		}
-		break;
+		*menu->u.option.uHook.optionRange.option = option;
+	}
+	break;
+	case MENU_TYPE_SET_OPTION_SEED: {
+		unsigned int seed = *menu->u.option.uHook.seed;
+		unsigned int increment = 1;
+		if (Button1(cmd))
+		{
+			increment *= 10;
+		}
+		if (Button2(cmd))
+		{
+			increment *= 100;
+		}
+		if (Left(cmd))
+		{
+			if (increment > seed)
+			{
+				seed = 0;
+			}
+			else
+			{
+				seed -= increment;
+			}
+		}
+		else if (Right(cmd))
+		{
+			if (UINT_MAX - increment < seed)
+			{
+				seed = UINT_MAX;
+			}
+			else
+			{
+				seed += increment;
+			}
+		}
+		*menu->u.option.uHook.seed = seed;
+	}
+	break;
 	case MENU_TYPE_SET_OPTION_UP_DOWN_VOID_FUNC_VOID:
 		if (Left(cmd))
 		{
@@ -1314,35 +1324,36 @@ void MenuActivate(MenuSystem *ms, menu_t *menu, int cmd)
 			menu->u.option.uHook.upDownFuncs.downFunc();
 		}
 		break;
-	case MENU_TYPE_SET_OPTION_RANGE_GET_SET:
+	case MENU_TYPE_SET_OPTION_RANGE_GET_SET: {
+		int option = menu->u.option.uHook.optionRangeGetSet.getFunc();
+		int increment = menu->u.option.uHook.optionRangeGetSet.increment;
+		if (Left(cmd))
 		{
-			int option = menu->u.option.uHook.optionRangeGetSet.getFunc();
-			int increment = menu->u.option.uHook.optionRangeGetSet.increment;
-			if (Left(cmd))
+			if (menu->u.option.uHook.optionRangeGetSet.low + increment >
+				option)
 			{
-				if (menu->u.option.uHook.optionRangeGetSet.low + increment > option)
-				{
-					option = menu->u.option.uHook.optionRangeGetSet.low;
-				}
-				else
-				{
-					option -= increment;
-				}
+				option = menu->u.option.uHook.optionRangeGetSet.low;
 			}
-			else if (Right(cmd))
+			else
 			{
-				if (menu->u.option.uHook.optionRangeGetSet.high - increment < option)
-				{
-					option = menu->u.option.uHook.optionRangeGetSet.high;
-				}
-				else
-				{
-					option += increment;
-				}
+				option -= increment;
 			}
-			menu->u.option.uHook.optionRangeGetSet.setFunc(option);
 		}
-		break;
+		else if (Right(cmd))
+		{
+			if (menu->u.option.uHook.optionRangeGetSet.high - increment <
+				option)
+			{
+				option = menu->u.option.uHook.optionRangeGetSet.high;
+			}
+			else
+			{
+				option += increment;
+			}
+		}
+		menu->u.option.uHook.optionRangeGetSet.setFunc(option);
+	}
+	break;
 	case MENU_TYPE_VOID_FUNC:
 		if (AnyButton(cmd))
 		{
@@ -1359,31 +1370,3 @@ void MenuActivate(MenuSystem *ms, menu_t *menu, int cmd)
 		break;
 	}
 }
-
-void PostInputConfigApply(menu_t *menu, int cmd, void *data)
-{
-	UNUSED(menu);
-	UNUSED(cmd);
-	if (!ConfigApply(&gConfig))
-	{
-		LOG(LM_MAIN, LL_ERROR, "Failed to apply config; reset to last used");
-		ConfigResetChanged(&gConfig);
-	}
-	else
-	{
-		// Save config immediately
-		ConfigSave(&gConfig, GetConfigFilePath(CONFIG_FILE));
-	}
-
-	MenuResetSize(data);
-}
-
-void MenuResetSize(MenuSystem *ms)
-{
-	// Update menu system so that resolution changes don't
-	// affect menu positions
-	ms->pos = svec2i_zero();
-	ms->size = svec2i(
-		ms->graphics->cachedConfig.Res.x,
-		ms->graphics->cachedConfig.Res.y);
-}
