@@ -2,7 +2,7 @@
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
 
-	Copyright (c) 2013-2016, Cong Xu
+	Copyright (c) 2013-2016, 2018-2021 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -35,44 +35,94 @@
 #include <cdogs/draw/draw.h>
 #include <cdogs/draw/drawtools.h>
 #include <cdogs/font.h>
+#include <cdogs/material.h>
 #include <cdogs/player_template.h>
 
+static char letters[] = "1234567890-QWERTYUIOP!ASDFGHJKL:'ZXCVBNM, .?";
+static char smallLetters[] = "1234567890-qwertyuiop!asdfghjkl:'zxcvbnm, .?";
 
-static char letters[] = "1234567890-QWERTYUIOP!ASDFGHJKL:#ZXCVBNM  .?";
-static char smallLetters[] = "1234567890-qwertyuiop!asdfghjkl:#zxcvbnm  .?";
-
+static Rect2i NameMenuItemBounds(
+	const Rect2i bounds, const int i, const char *label)
+{
+#define ENTRY_COLS 11
+#define ENTRY_SPACING 7
+	const int dy = FontH();
+	const int y = CENTER_Y(
+		bounds.Pos, bounds.Size,
+		dy * (((int)strlen(letters) - 1) / ENTRY_COLS));
+	const struct vec2i menuPos = svec2i(
+		bounds.Pos.x + (i % ENTRY_COLS) * ENTRY_SPACING,
+		y + (i / ENTRY_COLS) * dy);
+	const int w = MAX(FontStrW(label), ENTRY_SPACING);
+	return Rect2iNew(menuPos, svec2i(w, dy));
+}
 static void DrawNameMenu(
-	const menu_t *menu, GraphicsDevice *g,
-	const struct vec2i pos, const struct vec2i size, const void *data)
+	const menu_t *menu, GraphicsDevice *g, const struct vec2i pos,
+	const struct vec2i size, const void *data)
 {
 	const PlayerSelectMenuData *d = data;
-
-#define ENTRY_COLS	11
-#define	ENTRY_SPACING	7
-
-	int x = pos.x;
-	int y = CENTER_Y(
-		pos, size, FontH() * (((int)strlen(letters) - 1) / ENTRY_COLS));
-
 	UNUSED(menu);
-	UNUSED(g);
 
+	Rect2i itemBounds;
 	int i;
 	for (i = 0; i < (int)strlen(letters); i++)
 	{
-		struct vec2i menuPos = svec2i(
-			x + (i % ENTRY_COLS) * ENTRY_SPACING,
-			y + (i / ENTRY_COLS) * FontH());
-		FontChMask(
-			letters[i], menuPos,
-			i == d->nameMenuSelection ? colorRed : colorWhite);
+		char buf[2];
+		sprintf(buf, "%c", letters[i]);
+		itemBounds = NameMenuItemBounds(Rect2iNew(pos, size), i, buf);
+		DisplayMenuItem(
+			g, itemBounds, buf, i == d->nameMenuSelection, false, colorWhite);
 	}
 
+	const char *label = "(End)";
+	itemBounds = NameMenuItemBounds(Rect2iNew(pos, size), i, label);
 	DisplayMenuItem(
-		svec2i(
-			x + (i % ENTRY_COLS) * ENTRY_SPACING,
-			y + (i / ENTRY_COLS) * FontH()),
-		"(End)", i == d->nameMenuSelection, 0, colorBlack);
+		g, itemBounds, label, i == d->nameMenuSelection, false, colorWhite);
+}
+
+static void NameMenuPostUpdate(menu_t *menu, void *data)
+{
+	PlayerSelectMenuData *d = data;
+	UNUSED(menu);
+
+	// Change mouse selection
+	if (MouseHasMoved(&gEventHandlers.mouse))
+	{
+		menu->mouseHover = false;
+		Rect2i itemBounds;
+		int i;
+		for (i = 0; i < (int)strlen(letters); i++)
+		{
+			char buf[2];
+			sprintf(buf, "%c", letters[i]);
+			itemBounds =
+				NameMenuItemBounds(Rect2iNew(d->ms->pos, d->ms->size), i, buf);
+			if (!Rect2iIsInside(itemBounds, gEventHandlers.mouse.currentPos))
+			{
+				continue;
+			}
+			menu->mouseHover = true;
+			if (d->nameMenuSelection != i)
+			{
+				d->nameMenuSelection = i;
+				MenuPlaySound(MENU_SOUND_SWITCH);
+			}
+			break;
+		}
+
+		const char *label = "(End)";
+		itemBounds =
+			NameMenuItemBounds(Rect2iNew(d->ms->pos, d->ms->size), i, label);
+		if (Rect2iIsInside(itemBounds, gEventHandlers.mouse.currentPos))
+		{
+			menu->mouseHover = true;
+			if (d->nameMenuSelection != i)
+			{
+				d->nameMenuSelection = i;
+				MenuPlaySound(MENU_SOUND_SWITCH);
+			}
+		}
+	}
 }
 
 static int HandleInputNameMenu(int cmd, void *data)
@@ -161,17 +211,18 @@ static int HandleInputNameMenu(int cmd, void *data)
 }
 
 static void PostInputRotatePlayer(menu_t *menu, int cmd, void *data);
+static void CheckReenableHairHatMenu(menu_t *menu, void *data);
 
 static void PostInputFaceMenu(menu_t *menu, int cmd, void *data);
 static menu_t *CreateFaceMenu(MenuDisplayPlayerData *data)
 {
-	menu_t *menu = MenuCreateNormal("Face", "", MENU_TYPE_NORMAL, 0);
+	menu_t *menu = MenuCreateNormal("Face / Body", "", MENU_TYPE_NORMAL, 0);
 	menu->u.normal.maxItems = 11;
 	CA_FOREACH(const CharacterClass, c, gCharacterClasses.Classes)
-		MenuAddSubmenu(menu, MenuCreateBack(c->Name));
+	MenuAddSubmenu(menu, MenuCreateBack(c->Name));
 	CA_FOREACH_END()
 	CA_FOREACH(const CharacterClass, c, gCharacterClasses.CustomClasses)
-		MenuAddSubmenu(menu, MenuCreateBack(c->Name));
+	MenuAddSubmenu(menu, MenuCreateBack(c->Name));
 	CA_FOREACH_END()
 	MenuSetPostInputFunc(menu, PostInputFaceMenu, data);
 	return menu;
@@ -195,15 +246,46 @@ static void PostInputFaceMenu(menu_t *menu, int cmd, void *data)
 	PostInputRotatePlayer(menu, cmd, data);
 }
 
+static void PostInputHairMenu(menu_t *menu, int cmd, void *data);
+static menu_t *CreateHairMenu(MenuDisplayPlayerData *data)
+{
+	menu_t *menu = MenuCreateNormal("Hair/hat", "", MENU_TYPE_NORMAL, 0);
+	menu->u.normal.maxItems = 11;
+	MenuAddSubmenu(menu, MenuCreateBack("(None)"));
+	CA_FOREACH(const char *, h, gPicManager.hairstyleNames)
+	MenuAddSubmenu(menu, MenuCreateBack(*h));
+	CA_FOREACH_END()
+	MenuSetPostInputFunc(menu, PostInputHairMenu, data);
+	return menu;
+}
+static void PostInputHairMenu(menu_t *menu, int cmd, void *data)
+{
+	const MenuDisplayPlayerData *d = data;
+	// Change player hairstyle based on current menu selection
+	PlayerData *p = PlayerDataGetByUID(d->PlayerUID);
+	Character *c = &p->Char;
+	CFREE(c->Hair);
+	c->Hair = NULL;
+	if (menu->u.normal.index > 0)
+	{
+		const char **hair =
+			CArrayGet(&gPicManager.hairstyleNames, menu->u.normal.index - 1);
+		CSTRDUP(c->Hair, *hair);
+	}
+	PostInputRotatePlayer(menu, cmd, data);
+}
+
 static void DrawColorMenu(
-	const menu_t *menu, GraphicsDevice *g,
-	const struct vec2i pos, const struct vec2i size, const void *data);
+	const menu_t *menu, GraphicsDevice *g, const struct vec2i pos,
+	const struct vec2i size, const void *data);
+static void ColorMenuPostUpdate(menu_t *menu, void *data);
 static int HandleInputColorMenu(int cmd, void *data);
 static menu_t *CreateColorMenu(
-	const char *name, ColorMenuData *data,
-	const CharColorType type, const int playerUID)
+	const char *name, ColorMenuData *data, const MenuSystem *ms, const CharColorType type,
+	const int playerUID)
 {
 	data->Type = type;
+	data->ms = ms;
 	data->PlayerUID = playerUID;
 	data->palette = PicManagerGetPic(&gPicManager, "palette");
 	// Find the closest palette colour available, and select it
@@ -225,7 +307,7 @@ static menu_t *CreateColorMenu(
 			const int er = (int)colour.r - currentColour.r;
 			const int eg = (int)colour.g - currentColour.g;
 			const int eb = (int)colour.b - currentColour.b;
-			const int error = er*er + eg*eg + eb*eb;
+			const int error = er * er + eg * eg + eb * eb;
 			if (errorLowest == -1 || error < errorLowest)
 			{
 				errorLowest = error;
@@ -233,43 +315,86 @@ static menu_t *CreateColorMenu(
 			}
 		}
 	}
-	return MenuCreateCustom(name, DrawColorMenu, HandleInputColorMenu, data);
+	menu_t *menu = MenuCreateCustom(name, DrawColorMenu, HandleInputColorMenu, data);
+	MenuSetPostUpdateFunc(menu, ColorMenuPostUpdate, data, false);
+	return menu;
+}
+static Rect2i ColorSwatchBounds(const Rect2i bounds, const Pic *palette, const struct vec2i v)
+{
+	const struct vec2i swatchSize = svec2i(4, 4);
+	const struct vec2i drawPos =
+		svec2i(bounds.Pos.x, CENTER_Y(bounds.Pos, bounds.Size, palette->size.y * swatchSize.y));
+	return Rect2iNew(svec2i_add(drawPos, svec2i_multiply(v, swatchSize)), swatchSize);
 }
 static void DrawColorMenu(
-	const menu_t *menu, GraphicsDevice *g,
-	const struct vec2i pos, const struct vec2i size, const void *data)
+	const menu_t *menu, GraphicsDevice *g, const struct vec2i pos,
+	const struct vec2i size, const void *data)
 {
 	UNUSED(menu);
 	const ColorMenuData *d = data;
+	Rect2i itemBounds;
+
 	// Draw colour squares from the palette
-	const struct vec2i swatchSize = svec2i(4, 4);
-	const struct vec2i drawPos = svec2i(
-		pos.x, CENTER_Y(pos, size, d->palette->size.y * swatchSize.y));
-	struct vec2i v;
-	for (v.y = 0; v.y < d->palette->size.y; v.y++)
+	RECT_FOREACH(Rect2iNew(svec2i_zero(), d->palette->size))
+	const int idx = _v.x + _v.y * d->palette->size.x;
+	const color_t colour =
+		PIXEL2COLOR(d->palette->Data[idx]);
+	if (colour.a == 0)
 	{
-		for (v.x = 0; v.x < d->palette->size.x; v.x++)
-		{
-			const color_t colour = PIXEL2COLOR(
-				d->palette->Data[v.x + v.y * d->palette->size.x]);
-			if (colour.a == 0)
-			{
-				continue;
-			}
-			DrawRectangle(
-				g, svec2i_add(drawPos, svec2i_multiply(v, swatchSize)), swatchSize,
-				colour, 0);
-		}
+		continue;
 	}
+	itemBounds = ColorSwatchBounds(Rect2iNew(pos, size), d->palette, _v);
+	DrawRectangle(g, itemBounds.Pos, itemBounds.Size, colour, true);
+	RECT_FOREACH_END()
+
 	// Draw a highlight around the selected colour
+	itemBounds = ColorSwatchBounds(Rect2iNew(pos, size), d->palette, d->selectedColor);
 	DrawRectangle(
 		g,
-		svec2i_add(
-			drawPos,
-			svec2i_subtract(
-				svec2i_multiply(d->selectedColor, swatchSize), svec2i(1, 1))),
-		svec2i_add(swatchSize, svec2i(2, 2)),
-		colorWhite, DRAW_FLAG_LINE);
+		svec2i_subtract(itemBounds.Pos, svec2i(1, 1)),
+		svec2i_add(itemBounds.Size, svec2i(2, 2)), colorWhite, false);
+}
+static void ColorMenuOnChange(ColorMenuData *d, const struct vec2i v)
+{
+	if (!Rect2iIsInside(Rect2iNew(svec2i_zero(), d->palette->size), v))
+	{
+		return;
+	}
+	if (svec2i_is_equal(d->selectedColor, v))
+	{
+		return;
+	}
+	const color_t colour = PIXEL2COLOR(
+		d->palette->Data[v.x + v.y * d->palette->size.x]);
+	if (colour.a != 0)
+	{
+		d->selectedColor = v;
+		PlayerData *p = PlayerDataGetByUID(d->PlayerUID);
+		*CharColorGetByType(&p->Char.Colors, d->Type) = colour;
+		MenuPlaySound(MENU_SOUND_SWITCH);
+	}
+}
+static void ColorMenuPostUpdate(menu_t *menu, void *data)
+{
+	ColorMenuData *d = data;
+	UNUSED(menu);
+
+	// Change mouse selection
+	if (MouseHasMoved(&gEventHandlers.mouse))
+	{
+		menu->mouseHover = false;
+		Rect2i itemBounds;
+		RECT_FOREACH(Rect2iNew(svec2i_zero(), d->palette->size))
+		itemBounds = ColorSwatchBounds(Rect2iNew(d->ms->pos, d->ms->size), d->palette, _v);
+		if (!Rect2iIsInside(itemBounds, gEventHandlers.mouse.currentPos))
+		{
+			continue;
+		}
+		menu->mouseHover = true;
+		ColorMenuOnChange(d, _v);
+		break;
+		RECT_FOREACH_END()
+	}
 }
 static int HandleInputColorMenu(int cmd, void *data)
 {
@@ -277,30 +402,26 @@ static int HandleInputColorMenu(int cmd, void *data)
 	struct vec2i selected = d->selectedColor;
 	switch (cmd)
 	{
-	case CMD_LEFT: selected.x--; break;
-	case CMD_RIGHT: selected.x++; break;
-	case CMD_UP: selected.y--; break;
-	case CMD_DOWN: selected.y++; break;
+	case CMD_LEFT:
+		selected.x--;
+		break;
+	case CMD_RIGHT:
+		selected.x++;
+		break;
+	case CMD_UP:
+		selected.y--;
+		break;
+	case CMD_DOWN:
+		selected.y++;
+		break;
 	case CMD_BUTTON1:
-	case CMD_BUTTON2:	// fallthrough
+	case CMD_BUTTON2: // fallthrough
 		MenuPlaySound(MENU_SOUND_ENTER);
 		return cmd;
 	default:
 		break;
 	}
-	if (selected.x >= 0 && selected.x < d->palette->size.x &&
-		selected.y >= 0 && selected.y < d->palette->size.y)
-	{
-		const color_t colour = PIXEL2COLOR(
-			d->palette->Data[selected.x + selected.y * d->palette->size.x]);
-		if (colour.a != 0)
-		{
-			d->selectedColor = selected;
-			PlayerData *p = PlayerDataGetByUID(d->PlayerUID);
-			*CharColorGetByType(&p->Char.Colors, d->Type) = colour;
-			MenuPlaySound(MENU_SOUND_SWITCH);
-		}
-	}
+	ColorMenuOnChange(d, selected);
 	return 0;
 }
 
@@ -311,14 +432,20 @@ static void PostInputLoadTemplate(menu_t *menu, int cmd, void *data)
 		PlayerSelectMenuData *d = data;
 		PlayerData *p = PlayerDataGetByUID(d->display.PlayerUID);
 		const PlayerTemplate *t =
-			CArrayGet(&gPlayerTemplates, menu->u.normal.index);
+			PlayerTemplateGetById(&gPlayerTemplates, menu->u.normal.index);
 		memset(p->name, 0, sizeof p->name);
-		strncpy(p->name, t->name, sizeof p->name - 1);
+		strcpy(p->name, t->name);
 		p->Char.Class = StrCharacterClass(t->CharClassName);
 		if (p->Char.Class == NULL)
 		{
 			p->Char.Class = StrCharacterClass("Jones");
 		}
+		CFREE(p->Char.Hair);
+		p->Char.Hair = NULL;
+		if (t->Hair)
+		{
+			CSTRDUP(p->Char.Hair, t->Hair);
+		}
 		p->Char.Colors = t->Colors;
 	}
 }
@@ -327,19 +454,17 @@ static void PostInputLoadTemplate(menu_t *menu, int cmd, void *data)
 static void PostEnterLoadTemplateNames(menu_t *menu, void *data)
 {
 	bool *isSave = (bool *)data;
-	for (int i = 0; i < (int)gPlayerTemplates.size; i++)
+	MenuClearSubmenus(menu);
+	for (int i = 0;; i++)
 	{
-		// Add menu if necessary
-		if (i == (int)menu->u.normal.subMenus.size)
+		const PlayerTemplate *pt = PlayerTemplateGetById(&gPlayerTemplates, i);
+		if (pt == NULL)
 		{
-			MenuAddSubmenu(menu, MenuCreateBack(""));
+			break;
 		}
-		menu_t *subMenu = CArrayGet(&menu->u.normal.subMenus, i);
-		const PlayerTemplate *pt = CArrayGet(&gPlayerTemplates, i);
-		CFREE(subMenu->name);
-		CSTRDUP(subMenu->name, pt->name);
+		MenuAddSubmenu(menu, MenuCreateBack(pt->name));
 	}
-	if (*isSave && menu->u.normal.subMenus.size == gPlayerTemplates.size)
+	if (*isSave)
 	{
 		MenuAddSubmenu(menu, MenuCreateBack("(new)"));
 	}
@@ -363,25 +488,32 @@ static void PostInputSaveTemplate(menu_t *menu, int cmd, void *data)
 	}
 	PlayerSelectMenuData *d = data;
 	PlayerData *p = PlayerDataGetByUID(d->display.PlayerUID);
-	while (menu->u.normal.index >= (int)gPlayerTemplates.size)
-	{
-		PlayerTemplate empty;
-		memset(&empty, 0, sizeof empty);
-		CArrayPushBack(&gPlayerTemplates, &empty);
-	}
 	PlayerTemplate *t =
-		CArrayGet(&gPlayerTemplates, menu->u.normal.index);
+		PlayerTemplateGetById(&gPlayerTemplates, menu->u.normal.index);
+	if (t == NULL)
+	{
+		PlayerTemplate nt;
+		memset(&nt, 0, sizeof nt);
+		CArrayPushBack(&gPlayerTemplates.Classes, &nt);
+		t = CArrayGet(
+			&gPlayerTemplates.Classes, gPlayerTemplates.Classes.size - 1);
+	}
 	memset(t->name, 0, sizeof t->name);
-	strncpy(t->name, p->name, sizeof t->name - 1);
+	strcpy(t->name, p->name);
 	CFREE(t->CharClassName);
 	CSTRDUP(t->CharClassName, p->Char.Class->Name);
+	if (p->Char.Hair)
+	{
+		CFREE(t->Hair);
+		CSTRDUP(t->Hair, p->Char.Hair);
+	}
 	t->Colors = p->Char.Colors;
-	SavePlayerTemplates(&gPlayerTemplates, PLAYER_TEMPLATE_FILE);
+	PlayerTemplatesSave(&gPlayerTemplates);
 }
 
 static void SaveTemplateDisplayTitle(
-	const menu_t *menu, GraphicsDevice *g,
-	const struct vec2i pos, const struct vec2i size, const void *data)
+	const menu_t *menu, GraphicsDevice *g, const struct vec2i pos,
+	const struct vec2i size, const void *data)
 {
 	UNUSED(g);
 	const PlayerSelectMenuData *d = data;
@@ -412,16 +544,14 @@ static void CheckReenableLoadMenu(menu_t *menu, void *data)
 	menu_t *loadMenu = MenuGetSubmenuByName(menu, "Load");
 	UNUSED(data);
 	assert(loadMenu);
-	loadMenu->isDisabled = gPlayerTemplates.size == 0;
+	loadMenu->isDisabled = PlayerTemplateGetById(&gPlayerTemplates, 0) == NULL;
 }
 static menu_t *CreateCustomizeMenu(
 	const char *name, PlayerSelectMenuData *data);
 static void ShuffleAppearance(void *data);
 void PlayerSelectMenusCreate(
-	PlayerSelectMenu *menu,
-	int numPlayers, int player, const int playerUID,
-	EventHandlers *handlers, GraphicsDevice *graphics,
-	const NameGen *ng)
+	PlayerSelectMenu *menu, int numPlayers, int player, const int playerUID,
+	EventHandlers *handlers, GraphicsDevice *graphics, const NameGen *ng)
 {
 	MenuSystem *ms = &menu->ms;
 	PlayerSelectMenuData *data = &menu->data;
@@ -429,6 +559,7 @@ void PlayerSelectMenusCreate(
 	int w = graphics->cachedConfig.Res.x;
 	int h = graphics->cachedConfig.Res.y;
 
+	data->ms = ms;
 	data->nameMenuSelection = (int)strlen(letters);
 	data->display.PlayerUID = playerUID;
 	data->display.currentMenu = &ms->current;
@@ -461,19 +592,16 @@ void PlayerSelectMenusCreate(
 	}
 	MenuSystemInit(ms, handlers, graphics, pos, size);
 	ms->align = MENU_ALIGN_LEFT;
-	ms->root = ms->current = MenuCreateNormal(
-		"",
-		"",
-		MENU_TYPE_NORMAL,
-		0);
+	ms->root = ms->current = MenuCreateNormal("", "", MENU_TYPE_NORMAL, 0);
 	MenuSetPostInputFunc(ms->root, PostInputRotatePlayer, &data->display);
-	MenuAddSubmenu(ms->root, MenuCreateCustom(
-		"Name", DrawNameMenu, HandleInputNameMenu, data));
+	menu_t *nameMenu =
+		MenuCreateCustom("Name", DrawNameMenu, HandleInputNameMenu, data);
+	MenuSetPostUpdateFunc(nameMenu, NameMenuPostUpdate, data, false);
+	MenuAddSubmenu(ms->root, nameMenu);
 
 	MenuAddSubmenu(ms->root, CreateCustomizeMenu("Customize...", data));
 	MenuAddSubmenu(
-		ms->root,
-		MenuCreateVoidFunc("Shuffle", ShuffleAppearance, data));
+		ms->root, MenuCreateVoidFunc("Shuffle", ShuffleAppearance, data));
 
 	MenuAddSubmenu(ms->root, CreateUseTemplateMenu("Load", data));
 	MenuAddSubmenu(ms->root, CreateSaveTemplateMenu("Save", data));
@@ -499,22 +627,39 @@ static menu_t *CreateCustomizeMenu(
 	menu_t *menu = MenuCreateNormal(name, "", MENU_TYPE_NORMAL, 0);
 
 	MenuAddSubmenu(menu, CreateFaceMenu(&data->display));
+	MenuAddSubmenu(menu, CreateHairMenu(&data->display));
 
-	MenuAddSubmenu(menu, CreateColorMenu(
-		"Skin", &data->skinData, CHAR_COLOR_SKIN, data->display.PlayerUID));
-	MenuAddSubmenu(menu, CreateColorMenu(
-		"Hair", &data->hairData, CHAR_COLOR_HAIR, data->display.PlayerUID));
-	MenuAddSubmenu(menu, CreateColorMenu(
-		"Arms", &data->armsData, CHAR_COLOR_ARMS, data->display.PlayerUID));
-	MenuAddSubmenu(menu, CreateColorMenu(
-		"Body", &data->bodyData, CHAR_COLOR_BODY, data->display.PlayerUID));
-	MenuAddSubmenu(menu, CreateColorMenu(
-		"Legs", &data->legsData, CHAR_COLOR_LEGS, data->display.PlayerUID));
+	MenuAddSubmenu(
+		menu, CreateColorMenu(
+				  "Skin Color", &data->skinData, data->ms, CHAR_COLOR_SKIN,
+				  data->display.PlayerUID));
+	MenuAddSubmenu(
+		menu, CreateColorMenu(
+				  "Hair Color", &data->hairData, data->ms, CHAR_COLOR_HAIR,
+				  data->display.PlayerUID));
+	MenuAddSubmenu(
+		menu, CreateColorMenu(
+				  "Arms Color", &data->armsData, data->ms, CHAR_COLOR_ARMS,
+				  data->display.PlayerUID));
+	MenuAddSubmenu(
+		menu, CreateColorMenu(
+				  "Body Color", &data->bodyData, data->ms, CHAR_COLOR_BODY,
+				  data->display.PlayerUID));
+	MenuAddSubmenu(
+		menu, CreateColorMenu(
+				  "Legs Color", &data->legsData, data->ms, CHAR_COLOR_LEGS,
+				  data->display.PlayerUID));
+	MenuAddSubmenu(
+		menu, CreateColorMenu(
+				  "Feet Color", &data->feetData, data->ms, CHAR_COLOR_FEET,
+				  data->display.PlayerUID));
 
 	MenuAddSubmenu(menu, MenuCreateSeparator(""));
 	MenuAddSubmenu(menu, MenuCreateBack("Back"));
 
 	MenuSetPostInputFunc(menu, PostInputRotatePlayer, &data->display);
+	MenuSetPostEnterFunc(
+		menu, CheckReenableHairHatMenu, &data->display, false);
 
 	return menu;
 }
@@ -524,7 +669,7 @@ static void ShuffleAppearance(void *data)
 	char buf[512];
 	NameGenMake(pData->nameGenerator, buf);
 	PlayerData *p = PlayerDataGetByUID(pData->display.PlayerUID);
-	strncpy(p->name, buf, 20);
+	strcpy(p->name, buf);
 	Character *c = &p->Char;
 	CharacterShuffleAppearance(c);
 }
@@ -539,6 +684,17 @@ static void PostInputRotatePlayer(menu_t *menu, int cmd, void *data)
 	{
 		d->Dir = (direction_e)CLAMP_OPPOSITE(
 			(int)d->Dir + dx, DIRECTION_UP, DIRECTION_UPLEFT);
-		SoundPlay(&gSoundDevice, StrSound("footsteps"));
+		char buf[CDOGS_PATH_MAX];
+		const PlayerData *p = PlayerDataGetByUID(d->PlayerUID);
+		MatGetFootstepSound(p->Char.Class, NULL, buf);
+		SoundPlay(&gSoundDevice, StrSound(buf));
 	}
 }
+static void CheckReenableHairHatMenu(menu_t *menu, void *data)
+{
+	menu_t *hairMenu = MenuGetSubmenuByName(menu, "Hair/hat");
+	CASSERT(hairMenu, "cannot find menu");
+	MenuDisplayPlayerData *d = data;
+	const PlayerData *p = PlayerDataGetByUID(d->PlayerUID);
+	hairMenu->isDisabled = !p->Char.Class->HasHair;
+}
