@@ -32,7 +32,7 @@
 #include <cdogs/player.h>
 
 #include "prep.h"
-#include "weapon_menu.h"
+#include "equip_menu.h"
 
 static void AddPlayerWeapons(CArray *weapons, const WeaponClass **guns);
 static void RemoveUnavailableWeapons(
@@ -40,7 +40,7 @@ static void RemoveUnavailableWeapons(
 static void AddDefaultGuns(PlayerData *p, const int idx, const CArray *guns);
 typedef struct
 {
-	WeaponMenu menus[MAX_LOCAL_PLAYERS];
+	EquipMenu menus[MAX_LOCAL_PLAYERS];
 	EventWaitResult waitResult;
 } PlayerEquipData;
 static void PlayerEquipTerminate(GameLoopData *data);
@@ -91,7 +91,7 @@ GameLoopData *PlayerEquip(void)
 			prevWeapons = &prevMission->Weapons;
 		}
 
-		WeaponMenuCreate(
+		EquipMenuCreate(
 			&data->menus[idx], &weapons, prevWeapons,
 			GetNumPlayers(PLAYER_ANY, false, true), idx, p->UID,
 			&gEventHandlers, &gGraphicsDevice);
@@ -133,28 +133,30 @@ static void AddDefaultGuns(PlayerData *p, const int idx, const CArray *guns)
 		{"Machine gun", "Flamer", "Knife", "Dynamite"},
 	};
 
-	// Attempt to give the player default guns; if the guns are missing, find
-	// guns available in the mission
-	// Look for the default guns
-	for (int i = 0; i < MAX_WEAPONS; i++)
+	if (!gCampaign.Setting.BuyAndSell)
 	{
-		const WeaponClass *defaultWC = NULL;
+		// Attempt to give the player default guns; if the guns are missing,
+		// find guns available in the mission Look for the default guns
+		for (int i = 0; i < MAX_WEAPONS; i++)
+		{
+			const WeaponClass *defaultWC = NULL;
+			CA_FOREACH(const WeaponClass *, wc, *guns)
+			const char *gunName = defaultGuns[idx][i];
+			if (strcmp((*wc)->name, gunName) == 0)
+			{
+				defaultWC = *wc;
+			}
+			CA_FOREACH_END()
+			if (defaultWC != NULL)
+			{
+				PlayerAddWeapon(p, defaultWC);
+			}
+		}
+		// Look for guns available in the mission
 		CA_FOREACH(const WeaponClass *, wc, *guns)
-		const char *gunName = defaultGuns[idx][i];
-		if (strcmp((*wc)->name, gunName) == 0)
-		{
-			defaultWC = *wc;
-		}
+		PlayerAddWeapon(p, *wc);
 		CA_FOREACH_END()
-		if (defaultWC != NULL)
-		{
-			PlayerAddWeapon(p, defaultWC);
-		}
 	}
-	// Look for guns available in the mission
-	CA_FOREACH(const WeaponClass *, wc, *guns)
-	PlayerAddWeapon(p, *wc);
-	CA_FOREACH_END()
 	PlayerAddMinimalWeapons(p);
 }
 static bool HasWeapon(const CArray *weapons, const WeaponClass *wc)
@@ -173,7 +175,7 @@ static void PlayerEquipTerminate(GameLoopData *data)
 
 	for (int i = 0; i < GetNumPlayers(PLAYER_ANY, false, true); i++)
 	{
-		WeaponMenuTerminate(&pData->menus[i]);
+		EquipMenuTerminate(&pData->menus[i]);
 	}
 	CFREE(pData);
 }
@@ -237,8 +239,8 @@ static GameLoopResult PlayerEquipUpdate(GameLoopData *data, LoopRunner *l)
 	bool isDone = true;
 	for (int i = 0; i < GetNumPlayers(PLAYER_ANY, false, true); i++)
 	{
-		WeaponMenuUpdate(&pData->menus[i], cmds[i]);
-		isDone = isDone && WeaponMenuIsDone(&pData->menus[i]);
+		EquipMenuUpdate(&pData->menus[i], cmds[i]);
+		isDone = isDone && EquipMenuIsDone(&pData->menus[i]);
 	}
 	if (isDone)
 	{
@@ -265,7 +267,7 @@ static void PlayerEquipDraw(GameLoopData *data)
 	BlitClearBuf(&gGraphicsDevice);
 	for (int i = 0; i < GetNumPlayers(PLAYER_ANY, false, true); i++)
 	{
-		WeaponMenuDraw(&pData->menus[i]);
+		EquipMenuDraw(&pData->menus[i]);
 	}
 	BlitUpdateFromBuf(&gGraphicsDevice, gGraphicsDevice.screen);
 }
