@@ -8,89 +8,9 @@
 #include <actors.h>
 #include <game_mode.h>
 #include <net_client.h>
+#include <player_template.h>
 #include <utils.h>
 
-// Stubs
-CampaignOptions gCampaign;
-NetClient gNetClient;
-CArray gPlayerTemplates;
-Config gConfig;
-TActor *ActorGetByUID(const int uid)
-{
-	UNUSED(uid);
-	return NULL;
-}
-void ActorDestroy(TActor *a)
-{
-	UNUSED(a);
-}
-void CharacterSetColors(Character *c)
-{
-	UNUSED(c);
-}
-NCharColors CharColors2Net(const CharColors c)
-{
-	UNUSED(c);
-	NCharColors co = NCharColors_init_default;
-	return co;
-}
-NColor Color2Net(const color_t c)
-{
-	NColor co;
-	co.RGBA = (c.r << 24) | (c.g << 16) | (c.b << 8) | c.a;
-	return co;
-}
-const char *JoyName(const int deviceIndex)
-{
-	UNUSED(deviceIndex);
-	return NULL;
-}
-int ModeLives(const GameMode mode)
-{
-	UNUSED(mode);
-	return 0;
-}
-int ModeMaxHealth(const GameMode mode)
-{
-	UNUSED(mode);
-	return 0;
-}
-CharColors Net2CharColors(const NCharColors c)
-{
-	UNUSED(c);
-	CharColors co;
-	memset(&co, 0, sizeof co);
-	return co;
-}
-NPlayerData NMakePlayerData(const PlayerData *p)
-{
-	UNUSED(p);
-	NPlayerData d = NPlayerData_init_default;
-	return d;
-}
-const WeaponClass *StrWeaponClass(const char *s)
-{
-	UNUSED(s);
-	return NULL;
-}
-const CharacterClass *StrCharacterClass(const char *s)
-{
-	UNUSED(s);
-	return NULL;
-}
-bool ConfigGetBool(Config *c, const char *name)
-{
-	UNUSED(c); UNUSED(name); return false;
-}
-bool IsPVP(const GameMode mode)
-{
-	UNUSED(mode); return false;
-}
-bool InputHasGrenadeButton(const input_device_e d, const int dIndex)
-{
-	UNUSED(d); UNUSED(dIndex); return false;
-}
-
 
 FEATURE(assign_unused, "Assign unused input device")
 	// This feature is used to assign input devices to players, before the game
@@ -125,7 +45,7 @@ FEATURE(assign_unused, "Assign unused input device")
 			const int idx = 0;
 			p->deviceIndex = idx;
 		WHEN("I assign it with a different input device")
-			bool res = PlayerTrySetUnusedInputDevice(p, INPUT_DEVICE_MOUSE, 0);
+			bool res = PlayerTrySetUnusedInputDevice(p, INPUT_DEVICE_JOYSTICK, 0);
 		THEN("the assignment should fail")
 			SHOULD_BE_FALSE(res);
 		AND("the player's input device should be unchanged")
